@@ -1,15 +1,8 @@
 #!/bin/sh
-#BHEADER**********************************************************************
-# Copyright (c) 2008,  Lawrence Livermore National Security, LLC.
-# Produced at the Lawrence Livermore National Laboratory.
-# This file is part of HYPRE.  See file COPYRIGHT for details.
+# Copyright 1998-2019 Lawrence Livermore National Security, LLC and other
+# HYPRE Project Developers. See the top-level COPYRIGHT file for details.
 #
-# HYPRE is free software; you can redistribute it and/or modify it under the
-# terms of the GNU Lesser General Public License (as published by the Free
-# Software Foundation) version 2.1 dated February 1999.
-#
-# $Revision$
-#EHEADER**********************************************************************
+# SPDX-License-Identifier: (Apache-2.0 OR MIT)
 
 testname=`basename $0 .sh`
 
@@ -44,7 +37,7 @@ src_dir=`cd $1; pwd`
 shift
 
 # Parse the rest of the command line
-copts=""
+copts="-DHYPRE_BUILD_TESTS=ON"
 mopts=""
 ropts=""
 eopts=""
@@ -81,7 +74,6 @@ src_dir=`pwd`
 # Clean up the cmbuild directories (do it from src_dir as a precaution)
 cd $src_dir
 rm -fr `echo cmbuild/* | sed 's/[^ ]*README.txt//g'`
-rm -fr `echo test/cmbuild/* | sed 's/[^ ]*README.txt//g'`
 
 # Clean up the previous install
 cd $src_dir
@@ -93,10 +85,9 @@ cmake $copts ..
 make $mopts install
 
 # Make
-cd $src_dir/test/cmbuild
-cmake ..
+cd $src_dir/cmbuild/test
 make $mopts
-mv -f $drivers ..
+mv -f $drivers ../../test
 
 cd $test_dir
 
@@ -121,7 +112,6 @@ done
 # Clean up
 cd $src_dir
 rm -fr `echo cmbuild/* | sed 's/[^ ]*README.txt//g'`
-rm -fr `echo test/cmbuild/* | sed 's/[^ ]*README.txt//g'`
 rm -fr hypre
 ( cd $src_dir/test; rm -f $drivers; ./cleantest.sh )
 
