@@ -1,17 +1,9 @@
+!     Copyright 1998-2019 Lawrence Livermore National Security, LLC and other
+!     HYPRE Project Developers. See the top-level COPYRIGHT file for details.
+!
+!     SPDX-License-Identifier: (Apache-2.0 OR MIT)
+
 ! -*- fortran -*-
-!BHEADER**********************************************************************
-! Copyright (c) 2008,  Lawrence Livermore National Security, LLC.
-! Produced at the Lawrence Livermore National Laboratory.
-! This file is part of HYPRE.  See file COPYRIGHT for details.
-!
-! HYPRE is free software; you can redistribute it and/or modify it under the
-! terms of the GNU Lesser General Public License (as published by the Free
-! Software Foundation) version 2.1 dated February 1999.
-!
-! $Revision$
-!EHEADER**********************************************************************
-
-
 !******************************************************************************
 ! 
 !  Header file for HYPRE library
