@@ -1,17 +1,9 @@
-/*BHEADER**********************************************************************
- * Copyright (c) 2008,  Lawrence Livermore National Security, LLC.
- * Produced at the Lawrence Livermore National Laboratory.
- * This file is part of HYPRE.  See file COPYRIGHT for details.
+/******************************************************************************
+ * Copyright 1998-2019 Lawrence Livermore National Security, LLC and other
+ * HYPRE Project Developers. See the top-level COPYRIGHT file for details.
  *
- * HYPRE is free software; you can redistribute it and/or modify it under the
- * terms of the GNU Lesser General Public License (as published by the Free
- * Software Foundation) version 2.1 dated February 1999.
- *
- * $Revision$
- ***********************************************************************EHEADER*/
-
-
-
+ * SPDX-License-Identifier: (Apache-2.0 OR MIT)
+ ******************************************************************************/
 
 /******************************************************************************
  *
@@ -35,16 +27,17 @@ typedef struct hypre_IJMatrix_struct
 
    HYPRE_Int     object_type;         /* Indicates the type of "object" */
    void         *object;              /* Structure for storing local portion */
-   void         *translator;          /* optional storage_type specfic structure
-                                       for holding additional local info */
-   void         *assumed_part;	   /* IJMatrix assumed partition */
-   HYPRE_Int     assemble_flag;       /* indicates whether matrix has been 
-				       assembled */
+   void         *translator;          /* optional storage_type specific structure
+                                         for holding additional local info */
+   void         *assumed_part;        /* IJMatrix assumed partition */
+   HYPRE_Int     assemble_flag;       /* indicates whether matrix has been
+                                         assembled */
 
-   HYPRE_BigInt  global_first_row;    /* these for data items are necessary */
-   HYPRE_BigInt  global_first_col;    /*   to be able to avoind using the global */
-   HYPRE_BigInt  global_num_rows;     /*   global partition */ 
+   HYPRE_BigInt  global_first_row;    /* these four data items are necessary */
+   HYPRE_BigInt  global_first_col;    /* to be able to avoid using the global */
+   HYPRE_BigInt  global_num_rows;     /* global partition */
    HYPRE_BigInt  global_num_cols;
+
    HYPRE_Int     omp_flag;
    HYPRE_Int     print_level;
 
@@ -83,7 +76,7 @@ typedef struct hypre_IJMatrix_struct
 HYPRE_Int
 hypre_GetIJMatrixParCSRMatrix( HYPRE_IJMatrix IJmatrix, Mat *reference )
 #endif
-  
+
 #ifdef ISIS_AVAILABLE
 /* IJMatrix_isis.c */
 HYPRE_Int
