@@ -1,3 +1,10 @@
+/******************************************************************************
+ * Copyright 1998-2019 Lawrence Livermore National Security, LLC and other
+ * HYPRE Project Developers. See the top-level COPYRIGHT file for details.
+ *
+ * SPDX-License-Identifier: (Apache-2.0 OR MIT)
+ ******************************************************************************/
+
 /*
    Example 3
 
@@ -367,6 +374,7 @@ int main (int argc, char *argv[])
 
       /* Clean up */
       HYPRE_StructPCGDestroy(solver);
+      HYPRE_StructSMGDestroy(precond);
    }
 
    if (solver_id == 1)
