@@ -1,14 +1,9 @@
-/*BHEADER**********************************************************************
- * Copyright (c) 2008,  Lawrence Livermore National Security, LLC.
- * Produced at the Lawrence Livermore National Laboratory.
- * This file is part of HYPRE.  See file COPYRIGHT for details.
+/******************************************************************************
+ * Copyright 1998-2019 Lawrence Livermore National Security, LLC and other
+ * HYPRE Project Developers. See the top-level COPYRIGHT file for details.
  *
- * HYPRE is free software; you can redistribute it and/or modify it under the
- * terms of the GNU Lesser General Public License (as published by the Free
- * Software Foundation) version 2.1 dated February 1999.
- *
- * $Revision$
- ***********************************************************************EHEADER*/
+ * SPDX-License-Identifier: (Apache-2.0 OR MIT)
+ ******************************************************************************/
 
 /******************************************************************************
  *
@@ -22,7 +17,7 @@
 
 #include <HYPRE_config.h>
 
-#include "general.h"
+#include "hypre_general.h"
 
 #include "_hypre_utilities.h"
 #include "HYPRE.h"
@@ -38,8 +33,8 @@
  * HYPRE_ConvertParCSRMatrixToDistributedMatrix
  *--------------------------------------------------------------------------*/
 
-HYPRE_Int 
-HYPRE_ConvertParCSRMatrixToDistributedMatrix( 
+HYPRE_Int
+HYPRE_ConvertParCSRMatrixToDistributedMatrix(
    HYPRE_ParCSRMatrix parcsr_matrix,
    HYPRE_DistributedMatrix *DistributedMatrix )
 {
@@ -56,7 +51,7 @@ HYPRE_ConvertParCSRMatrixToDistributedMatrix(
    if (!parcsr_matrix)
    {
       hypre_error(HYPRE_ERROR_ARG);
-      return hypre_error_flag; 
+      return hypre_error_flag;
    }
 
    HYPRE_ParCSRMatrixGetComm( parcsr_matrix, &comm);
@@ -68,7 +63,7 @@ HYPRE_ConvertParCSRMatrixToDistributedMatrix(
    HYPRE_DistributedMatrixInitialize( *DistributedMatrix );
 
    HYPRE_DistributedMatrixSetLocalStorage( *DistributedMatrix, parcsr_matrix );
-   
+
 
    HYPRE_ParCSRMatrixGetDims( parcsr_matrix, &M, &N);
    HYPRE_DistributedMatrixSetDims( *DistributedMatrix, M, N);
@@ -80,6 +75,6 @@ HYPRE_ConvertParCSRMatrixToDistributedMatrix(
    /* hypre_FinalizeTiming( timer ); */
 #endif
 
-   return hypre_error_flag; 
+   return hypre_error_flag;
 }
 
