@@ -1,14 +1,9 @@
-/*BHEADER**********************************************************************
- * Copyright (c) 2008,  Lawrence Livermore National Security, LLC.
- * Produced at the Lawrence Livermore National Laboratory.
- * This file is part of HYPRE.  See file COPYRIGHT for details.
+/******************************************************************************
+ * Copyright 1998-2019 Lawrence Livermore National Security, LLC and other
+ * HYPRE Project Developers. See the top-level COPYRIGHT file for details.
  *
- * HYPRE is free software; you can redistribute it and/or modify it under the
- * terms of the GNU Lesser General Public License (as published by the Free
- * Software Foundation) version 2.1 dated February 1999.
- *
- * $Revision$
- ***********************************************************************EHEADER*/
+ * SPDX-License-Identifier: (Apache-2.0 OR MIT)
+ ******************************************************************************/
 
 #include "_hypre_parcsr_block_mv.h"
 
@@ -52,20 +47,20 @@ hypre_BoomerAMGBlockCreateNodalA(hypre_ParCSRBlockMatrix *A,
    HYPRE_Int                num_nonzeros_diag;
    HYPRE_Int                num_nonzeros_offd = 0;
    HYPRE_Int                num_cols_offd = 0;
-                  
+
    hypre_ParCSRMatrix *AN;
    hypre_CSRMatrix    *AN_diag;
    HYPRE_Int          *AN_diag_i;
    HYPRE_Int          *AN_diag_j=NULL;
-   HYPRE_Real         *AN_diag_data = NULL; 
+   HYPRE_Real         *AN_diag_data = NULL;
    hypre_CSRMatrix    *AN_offd;
    HYPRE_Int          *AN_offd_i;
    HYPRE_Int          *AN_offd_j = NULL;
-   HYPRE_Real         *AN_offd_data = NULL; 
+   HYPRE_Real         *AN_offd_data = NULL;
    HYPRE_BigInt       *col_map_offd_AN = NULL;
    HYPRE_BigInt       *row_starts_AN;
 
-                 
+
    hypre_ParCSRCommPkg *comm_pkg = hypre_ParCSRBlockMatrixCommPkg(A);
    HYPRE_Int            num_sends;
    HYPRE_Int            num_recvs;
@@ -83,7 +78,7 @@ hypre_BoomerAMGBlockCreateNodalA(hypre_ParCSRBlockMatrix *A,
    HYPRE_Int           *recv_vec_starts_AN = NULL;
 
    HYPRE_Int            i;
-                      
+
    HYPRE_Int            ierr = 0;
 
    HYPRE_Int            num_procs;
@@ -94,7 +89,7 @@ hypre_BoomerAMGBlockCreateNodalA(hypre_ParCSRBlockMatrix *A,
    HYPRE_Int            num_nodes;
 
    HYPRE_Int            index, k;
-   
+
    HYPRE_Real           tmp;
    HYPRE_Real           sum;
 
@@ -139,8 +134,8 @@ hypre_BoomerAMGBlockCreateNodalA(hypre_ParCSRBlockMatrix *A,
       AN_diag_i[i] = A_diag_i[i];
    }
 
-   AN_diag_j = hypre_CTAlloc(HYPRE_Int,  num_nonzeros_diag, HYPRE_MEMORY_HOST);     
-   AN_diag_data = hypre_CTAlloc(HYPRE_Real,  num_nonzeros_diag, HYPRE_MEMORY_HOST);      
+   AN_diag_j = hypre_CTAlloc(HYPRE_Int,  num_nonzeros_diag, HYPRE_MEMORY_HOST);
+   AN_diag_data = hypre_CTAlloc(HYPRE_Real,  num_nonzeros_diag, HYPRE_MEMORY_HOST);
 
 
    AN_diag = hypre_CSRMatrixCreate(num_nodes, num_nodes, num_nonzeros_diag);
@@ -151,11 +146,11 @@ hypre_BoomerAMGBlockCreateNodalA(hypre_ParCSRBlockMatrix *A,
    for (i=0; i< num_nonzeros_diag; i++)
    {
       AN_diag_j[i]  = A_diag_j[i];
-      hypre_CSRBlockMatrixBlockNorm(norm_type, &A_diag_data[i*bnnz], 
+      hypre_CSRBlockMatrixBlockNorm(norm_type, &A_diag_data[i*bnnz],
                                     &tmp, block_size);
       AN_diag_data[i] = tmp;
    }
-   
+
 
    if (diag_option ==1 )
    {
@@ -164,24 +159,24 @@ hypre_BoomerAMGBlockCreateNodalA(hypre_ParCSRBlockMatrix *A,
       /* the diagonal is the first element listed in each row - */
       for (i=0; i < num_nodes; i++)
       {
-         index = AN_diag_i[i]; 
+         index = AN_diag_i[i];
          sum = 0.0;
          for (k = AN_diag_i[i]+1; k < AN_diag_i[i+1]; k++)
          {
             sum += AN_diag_data[k];
-            
+
          }
 
          AN_diag_data[index] = -sum;
       }
-      
+
    }
    else if (diag_option == 2)
    {
-      
+
       /*  make all diagonal entries negative */
       /* the diagonal is the first element listed in each row - */
-      
+
       for (i=0; i < num_nodes; i++)
       {
          index = AN_diag_i[i];
@@ -204,7 +199,7 @@ hypre_BoomerAMGBlockCreateNodalA(hypre_ParCSRBlockMatrix *A,
       send_procs = hypre_ParCSRCommPkgSendProcs(comm_pkg);
       send_map_starts = hypre_ParCSRCommPkgSendMapStarts(comm_pkg);
       send_map_elmts = hypre_ParCSRCommPkgSendMapElmts(comm_pkg);
-      if (num_sends) 
+      if (num_sends)
       {
          send_procs_AN = hypre_CTAlloc(HYPRE_Int,  num_sends, HYPRE_MEMORY_HOST);
          send_map_elmts_AN = hypre_CTAlloc(HYPRE_Int,  send_map_starts[num_sends], HYPRE_MEMORY_HOST);
@@ -235,7 +230,7 @@ hypre_BoomerAMGBlockCreateNodalA(hypre_ParCSRBlockMatrix *A,
       {
          recv_procs_AN[i] = recv_procs[i];
          recv_vec_starts_AN[i+1] = recv_vec_starts[i+1];
-         
+
       }
       hypre_ParCSRCommPkgRecvProcs(comm_pkg_AN) = recv_procs_AN;
       hypre_ParCSRCommPkgRecvVecStarts(comm_pkg_AN) = recv_vec_starts_AN;
@@ -257,24 +252,24 @@ hypre_BoomerAMGBlockCreateNodalA(hypre_ParCSRBlockMatrix *A,
    {
       AN_offd_i[i] = A_offd_i[i];
    }
-      
-   AN_offd_j = hypre_CTAlloc(HYPRE_Int,  num_nonzeros_offd, HYPRE_MEMORY_HOST);     
+
+   AN_offd_j = hypre_CTAlloc(HYPRE_Int,  num_nonzeros_offd, HYPRE_MEMORY_HOST);
    AN_offd_data = hypre_CTAlloc(HYPRE_Real,  num_nonzeros_offd, HYPRE_MEMORY_HOST);
 
    for (i=0; i< num_nonzeros_offd; i++)
    {
       AN_offd_j[i]  = A_offd_j[i];
-      hypre_CSRBlockMatrixBlockNorm(norm_type, &A_offd_data[i*bnnz], 
+      hypre_CSRBlockMatrixBlockNorm(norm_type, &A_offd_data[i*bnnz],
                                     &tmp, block_size);
       AN_offd_data[i] = tmp;
    }
-   
+
    AN_offd = hypre_CSRMatrixCreate(num_nodes, num_cols_offd, num_nonzeros_offd);
-  
+
    hypre_CSRMatrixI(AN_offd) = AN_offd_i;
    hypre_CSRMatrixJ(AN_offd) = AN_offd_j;
    hypre_CSRMatrixData(AN_offd) = AN_offd_data;
-   
+
    if (diag_option ==1 )
    {
       /* make the diag entry the negative of the sum of off-diag entries (here
@@ -286,16 +281,16 @@ hypre_BoomerAMGBlockCreateNodalA(hypre_ParCSRBlockMatrix *A,
          for (k = AN_offd_i[i]; k < AN_offd_i[i+1]; k++)
          {
             sum += AN_offd_data[k];
-            
+
          }
-         index = AN_diag_i[i];/* location of diag entry in data */ 
+         index = AN_diag_i[i];/* location of diag entry in data */
          AN_diag_data[index] -= sum; /* subtract from current value */
       }
-      
+
    }
 
-   /* now create AN */   
-    
+   /* now create AN */
+
    AN = hypre_ParCSRMatrixCreate(comm, global_num_nodes, global_num_nodes,
                                  row_starts_AN, row_starts_AN, num_cols_offd,
                                  num_nonzeros_diag, num_nonzeros_offd);
@@ -307,6 +302,8 @@ hypre_BoomerAMGBlockCreateNodalA(hypre_ParCSRBlockMatrix *A,
    hypre_ParCSRMatrixDiag(AN) = AN_diag;
    hypre_ParCSRMatrixOffd(AN) = AN_offd;
 
+   hypre_CSRMatrixMemoryLocation(AN_diag) = HYPRE_MEMORY_HOST;
+   hypre_CSRMatrixMemoryLocation(AN_offd) = HYPRE_MEMORY_HOST;
 
    hypre_ParCSRMatrixColMapOffd(AN) = col_map_offd_AN;
    hypre_ParCSRMatrixCommPkg(AN) = comm_pkg_AN;
@@ -315,3 +312,4 @@ hypre_BoomerAMGBlockCreateNodalA(hypre_ParCSRBlockMatrix *A,
 
    return (ierr);
 }
+
