@@ -1,14 +1,9 @@
-/*BHEADER**********************************************************************
- * Copyright (c) 2008,  Lawrence Livermore National Security, LLC.
- * Produced at the Lawrence Livermore National Laboratory.
- * This file is part of HYPRE.  See file COPYRIGHT for details.
+/******************************************************************************
+ * Copyright 1998-2019 Lawrence Livermore National Security, LLC and other
+ * HYPRE Project Developers. See the top-level COPYRIGHT file for details.
  *
- * HYPRE is free software; you can redistribute it and/or modify it under the
- * terms of the GNU Lesser General Public License (as published by the Free
- * Software Foundation) version 2.1 dated February 1999.
- *
- * $Revision$
- ***********************************************************************EHEADER*/
+ * SPDX-License-Identifier: (Apache-2.0 OR MIT)
+ ******************************************************************************/
 
 /******************************************************************************
  *
@@ -389,6 +384,22 @@ hypre_F90_IFACE(hypre_parcsrhybridsettruncfacto, HYPRE_PARCSRHYBRIDSETTRUNCFACTO
           hypre_F90_PassReal (trunc_factor) ));
 }
 
+/*--------------------------------------------------------------------------
+ * HYPRE_ParCSRHybridSetPMaxElmts
+ *--------------------------------------------------------------------------*/
+
+void
+hypre_F90_IFACE(hypre_parcsrhybridsetpmaxelmts, HYPRE_PARCSRHYBRIDSETPMAXELMTS)
+   (hypre_F90_Obj *solver,
+    hypre_F90_Int *p_max_elmts,
+    hypre_F90_Int *ierr)
+{
+   *ierr = (hypre_F90_Int)
+      (HYPRE_ParCSRHybridSetPMaxElmts(
+          hypre_F90_PassObj (HYPRE_Solver, solver),
+          hypre_F90_PassInt (p_max_elmts) ));
+}
+
 /*--------------------------------------------------------------------------
  * HYPRE_ParCSRHybridSetMaxLevels
  *--------------------------------------------------------------------------*/
@@ -437,6 +448,22 @@ hypre_F90_IFACE(hypre_parcsrhybridsetcoarsentyp, HYPRE_PARCSRHYBRIDSETCOARSENTYP
           hypre_F90_PassInt (coarsen_type)  ));
 }
 
+/*--------------------------------------------------------------------------
+ * HYPRE_ParCSRHybridSetInterpType
+ *--------------------------------------------------------------------------*/
+
+void
+hypre_F90_IFACE(hypre_parcsrhybridsetinterptyp, HYPRE_PARCSRHYBRIDSETINTERPTYP)
+   (hypre_F90_Obj *solver,
+    hypre_F90_Int *interp_type,
+    hypre_F90_Int *ierr)
+{
+   *ierr = (hypre_F90_Int)
+      (HYPRE_ParCSRHybridSetCoarsenType(
+          hypre_F90_PassObj (HYPRE_Solver, solver),
+          hypre_F90_PassInt (interp_type)  ));
+}
+
 /*--------------------------------------------------------------------------
  * HYPRE_ParCSRHybridSetCycleType
  *--------------------------------------------------------------------------*/
@@ -469,7 +496,7 @@ hypre_F90_IFACE(hypre_parcsrhybridsetnumgridswe, HYPRE_PARCSRHYBRIDSETNUMGRIDSWE
           hypre_F90_PassIntArray (num_grid_sweeps) ));
 }
 
-/*--------------------------------------------------------------------------
+/*------------------------------------------------------------------------
  * HYPRE_ParCSRHybridSetGridRelaxType
  *--------------------------------------------------------------------------*/
 
