@@ -1,3 +1,10 @@
+/******************************************************************************
+ * Copyright 1998-2019 Lawrence Livermore National Security, LLC and other
+ * HYPRE Project Developers. See the top-level COPYRIGHT file for details.
+ *
+ * SPDX-License-Identifier: (Apache-2.0 OR MIT)
+ ******************************************************************************/
+
 #include "_hypre_parcsr_ls.h"
 #include "par_amg.h"
 
@@ -17,18 +24,18 @@ HYPRE_Int hypre_seqAMGSetup( hypre_ParAMGData *amg_data,
    /* Par Data Structure variables */
    hypre_ParCSRMatrix **Par_A_array = hypre_ParAMGDataAArray(amg_data);
 
-   MPI_Comm 	      comm = hypre_ParCSRMatrixComm(Par_A_array[0]); 
-   MPI_Comm 	      new_comm, seq_comm;
+   MPI_Comm      comm = hypre_ParCSRMatrixComm(Par_A_array[0]);
+   MPI_Comm      new_comm, seq_comm;
 
    hypre_ParCSRMatrix   *A_seq = NULL;
    hypre_CSRMatrix  *A_seq_diag;
    hypre_CSRMatrix  *A_seq_offd;
    hypre_ParVector   *F_seq = NULL;
    hypre_ParVector   *U_seq = NULL;
- 
+
    hypre_ParCSRMatrix *A;
 
-   HYPRE_Int               **dof_func_array;   
+   HYPRE_Int               **dof_func_array;
    HYPRE_Int                num_procs, my_id;
 
    HYPRE_Int                level;
@@ -43,11 +50,11 @@ HYPRE_Int hypre_seqAMGSetup( hypre_ParAMGData *amg_data,
    redundant = hypre_ParAMGDataRedundant(amg_data);
 
    /*MPI Stuff */
-   hypre_MPI_Comm_size(comm, &num_procs);   
-  
+   hypre_MPI_Comm_size(comm, &num_procs);
+
    /*initial */
    level = p_level;
-   
+
    /* convert A at this level to sequential */
    A = Par_A_array[level];
 
@@ -66,7 +73,7 @@ HYPRE_Int hypre_seqAMGSetup( hypre_ParAMGData *amg_data,
       HYPRE_Int *displs = NULL;
       HYPRE_Int *displs2 = NULL;
       HYPRE_Int i, j, size, num_nonzeros, total_nnz, cnt;
-  
+
       hypre_CSRMatrix *A_diag = hypre_ParCSRMatrixDiag(A);
       hypre_CSRMatrix *A_offd = hypre_ParCSRMatrixOffd(A);
       HYPRE_BigInt *col_map_offd = hypre_ParCSRMatrixColMapOffd(A);
@@ -81,10 +88,10 @@ HYPRE_Int hypre_seqAMGSetup( hypre_ParAMGData *amg_data,
       HYPRE_Int new_num_procs;
       HYPRE_BigInt *row_starts;
 
-      hypre_GenerateSubComm(comm, num_rows, &new_comm); 
+      hypre_GenerateSubComm(comm, num_rows, &new_comm);
 
 
-      /*hypre_MPI_Group orig_group, new_group; 
+      /*hypre_MPI_Group orig_group, new_group;
       HYPRE_Int *ranks, new_num_procs, *row_starts;
 
       info = hypre_CTAlloc(HYPRE_Int,  num_procs, HYPRE_MEMORY_HOST);
@@ -95,7 +102,7 @@ HYPRE_Int hypre_seqAMGSetup( hypre_ParAMGData *amg_data,
 
       new_num_procs = 0;
       for (i=0; i < num_procs; i++)
-         if (info[i]) 
+         if (info[i])
          {
             ranks[new_num_procs] = i;
             info[new_num_procs++] = info[i];
@@ -110,7 +117,7 @@ HYPRE_Int hypre_seqAMGSetup( hypre_ParAMGData *amg_data,
       if (num_rows)
       {
          hypre_ParAMGDataParticipate(amg_data) = 1;
-         hypre_MPI_Comm_size(new_comm, &new_num_procs); 
+         hypre_MPI_Comm_size(new_comm, &new_num_procs);
          hypre_MPI_Comm_rank(new_comm, &my_id);
          info = hypre_CTAlloc(HYPRE_Int,  new_num_procs, HYPRE_MEMORY_HOST);
 
@@ -124,31 +131,31 @@ HYPRE_Int hypre_seqAMGSetup( hypre_ParAMGData *amg_data,
          {
             HYPRE_BoomerAMGCreate(&coarse_solver);
             HYPRE_BoomerAMGSetMaxRowSum(coarse_solver,
-		hypre_ParAMGDataMaxRowSum(amg_data)); 
+                  hypre_ParAMGDataMaxRowSum(amg_data));
             HYPRE_BoomerAMGSetStrongThreshold(coarse_solver,
-		hypre_ParAMGDataStrongThreshold(amg_data)); 
+                  hypre_ParAMGDataStrongThreshold(amg_data));
             HYPRE_BoomerAMGSetCoarsenType(coarse_solver,
-		hypre_ParAMGDataCoarsenType(amg_data)); 
+                  hypre_ParAMGDataCoarsenType(amg_data));
             HYPRE_BoomerAMGSetInterpType(coarse_solver,
-		hypre_ParAMGDataInterpType(amg_data)); 
-            HYPRE_BoomerAMGSetTruncFactor(coarse_solver, 
-		hypre_ParAMGDataTruncFactor(amg_data)); 
-            HYPRE_BoomerAMGSetPMaxElmts(coarse_solver, 
-		hypre_ParAMGDataPMaxElmts(amg_data)); 
-	    if (hypre_ParAMGDataUserRelaxType(amg_data) > -1) 
-               HYPRE_BoomerAMGSetRelaxType(coarse_solver, 
-		hypre_ParAMGDataUserRelaxType(amg_data)); 
-            HYPRE_BoomerAMGSetRelaxOrder(coarse_solver, 
-		hypre_ParAMGDataRelaxOrder(amg_data)); 
-            HYPRE_BoomerAMGSetRelaxWt(coarse_solver, 
-		hypre_ParAMGDataUserRelaxWeight(amg_data)); 
-	    if (hypre_ParAMGDataUserNumSweeps(amg_data) > -1) 
-               HYPRE_BoomerAMGSetNumSweeps(coarse_solver, 
-		hypre_ParAMGDataUserNumSweeps(amg_data)); 
-            HYPRE_BoomerAMGSetNumFunctions(coarse_solver, 
-		num_functions);
-            HYPRE_BoomerAMGSetMaxIter(coarse_solver, 1); 
-            HYPRE_BoomerAMGSetTol(coarse_solver, 0); 
+                  hypre_ParAMGDataInterpType(amg_data));
+            HYPRE_BoomerAMGSetTruncFactor(coarse_solver,
+                  hypre_ParAMGDataTruncFactor(amg_data));
+            HYPRE_BoomerAMGSetPMaxElmts(coarse_solver,
+                  hypre_ParAMGDataPMaxElmts(amg_data));
+            if (hypre_ParAMGDataUserRelaxType(amg_data) > -1)
+               HYPRE_BoomerAMGSetRelaxType(coarse_solver,
+                     hypre_ParAMGDataUserRelaxType(amg_data));
+            HYPRE_BoomerAMGSetRelaxOrder(coarse_solver,
+                  hypre_ParAMGDataRelaxOrder(amg_data));
+            HYPRE_BoomerAMGSetRelaxWt(coarse_solver,
+                  hypre_ParAMGDataUserRelaxWeight(amg_data));
+            if (hypre_ParAMGDataUserNumSweeps(amg_data) > -1)
+               HYPRE_BoomerAMGSetNumSweeps(coarse_solver,
+                     hypre_ParAMGDataUserNumSweeps(amg_data));
+            HYPRE_BoomerAMGSetNumFunctions(coarse_solver,
+                  num_functions);
+            HYPRE_BoomerAMGSetMaxIter(coarse_solver, 1);
+            HYPRE_BoomerAMGSetTol(coarse_solver, 0);
          }
 
          /* Create CSR Matrix, will be Diag part of new matrix */
@@ -167,15 +174,15 @@ HYPRE_Int hypre_seqAMGSetup( hypre_ParAMGData *amg_data,
          for (i=0; i < num_rows; i++)
          {
             for (j=A_diag_i[i]; j < A_diag_i[i+1]; j++)
-	    {
-	       A_tmp_j[cnt] = A_diag_j[j]+(HYPRE_Int)first_row_index;
-	       A_tmp_data[cnt++] = A_diag_data[j];
-	    }
+            {
+               A_tmp_j[cnt] = A_diag_j[j]+(HYPRE_Int)first_row_index;
+               A_tmp_data[cnt++] = A_diag_data[j];
+            }
             for (j=A_offd_i[i]; j < A_offd_i[i+1]; j++)
-	    {
-	       A_tmp_j[cnt] = (HYPRE_Int)col_map_offd[A_offd_j[j]];
-	       A_tmp_data[cnt++] = A_offd_data[j];
-	    }
+            {
+               A_tmp_j[cnt] = (HYPRE_Int)col_map_offd[A_offd_j[j]];
+               A_tmp_data[cnt++] = A_offd_data[j];
+            }
          }
 
          displs = hypre_CTAlloc(HYPRE_Int,  new_num_procs+1, HYPRE_MEMORY_HOST);
@@ -183,7 +190,7 @@ HYPRE_Int hypre_seqAMGSetup( hypre_ParAMGData *amg_data,
          for (i=1; i < new_num_procs+1; i++)
             displs[i] = displs[i-1]+info[i-1];
          size = displs[new_num_procs];
-  
+
          if (redundant || my_id == 0)
          {
             A_seq_i = hypre_CTAlloc(HYPRE_Int,  size+1, HYPRE_MEMORY_SHARED);
@@ -197,24 +204,24 @@ HYPRE_Int hypre_seqAMGSetup( hypre_ParAMGData *amg_data,
                         displs, HYPRE_MPI_INT, new_comm );
             if (num_functions > 1)
             {
-	       hypre_MPI_Allgatherv ( dof_func_array[level], num_rows, HYPRE_MPI_INT, 
-			seq_dof_func, info, displs, HYPRE_MPI_INT, new_comm );
-	       HYPRE_BoomerAMGSetDofFunc(coarse_solver, seq_dof_func);
+               hypre_MPI_Allgatherv ( dof_func_array[level], num_rows, HYPRE_MPI_INT,
+                     seq_dof_func, info, displs, HYPRE_MPI_INT, new_comm );
+               HYPRE_BoomerAMGSetDofFunc(coarse_solver, seq_dof_func);
             }
          }
          else
          {
-            if (A_seq_i) 
-		hypre_MPI_Gatherv ( &A_tmp_i[1], num_rows, HYPRE_MPI_INT, &A_seq_i[1], info,
+            if (A_seq_i)
+               hypre_MPI_Gatherv ( &A_tmp_i[1], num_rows, HYPRE_MPI_INT, &A_seq_i[1], info,
                         displs, HYPRE_MPI_INT, 0, new_comm );
             else
-		hypre_MPI_Gatherv ( &A_tmp_i[1], num_rows, HYPRE_MPI_INT, A_seq_i, info,
+               hypre_MPI_Gatherv ( &A_tmp_i[1], num_rows, HYPRE_MPI_INT, A_seq_i, info,
                         displs, HYPRE_MPI_INT, 0, new_comm );
             if (num_functions > 1)
             {
-	       hypre_MPI_Gatherv ( dof_func_array[level], num_rows, HYPRE_MPI_INT, 
-			seq_dof_func, info, displs, HYPRE_MPI_INT, 0, new_comm );
-	       if (my_id == 0) HYPRE_BoomerAMGSetDofFunc(coarse_solver, seq_dof_func);
+               hypre_MPI_Gatherv ( dof_func_array[level], num_rows, HYPRE_MPI_INT,
+                     seq_dof_func, info, displs, HYPRE_MPI_INT, 0, new_comm );
+               if (my_id == 0) HYPRE_BoomerAMGSetDofFunc(coarse_solver, seq_dof_func);
             }
          }
 
@@ -275,17 +282,17 @@ HYPRE_Int hypre_seqAMGSetup( hypre_ParAMGData *amg_data,
          if (redundant || my_id == 0)
          {
             hypre_TFree(displs2, HYPRE_MEMORY_HOST);
-   
+
             row_starts = hypre_CTAlloc(HYPRE_BigInt, 2, HYPRE_MEMORY_HOST);
-            row_starts[0] = 0; 
+            row_starts[0] = 0;
             row_starts[1] = size;
- 
+
             /* Create 1 proc communicator */
             seq_comm = hypre_MPI_COMM_SELF;
 
             A_seq = hypre_ParCSRMatrixCreate(seq_comm,size,size,
-					  row_starts, row_starts,
-						0,total_nnz,0); 
+                  row_starts, row_starts,
+                  0,total_nnz,0);
 
             A_seq_diag = hypre_ParCSRMatrixDiag(A_seq);
             A_seq_offd = hypre_ParCSRMatrixOffd(A_seq);
@@ -327,7 +334,7 @@ hypre_seqAMGCycle( hypre_ParAMGData *amg_data,
                    hypre_ParVector  **Par_F_array,
                    hypre_ParVector  **Par_U_array   )
 {
-   
+
    hypre_ParVector    *Aux_U;
    hypre_ParVector    *Aux_F;
 
@@ -337,12 +344,12 @@ hypre_seqAMGCycle( hypre_ParAMGData *amg_data,
 
    HYPRE_Int n;
    HYPRE_Int i;
-   
+
    hypre_Vector   *u_local;
    HYPRE_Real     *u_data;
-   
-   HYPRE_Int	   first_index;
-   
+
+   HYPRE_Int       first_index;
+
    /* Acquire seq data */
    MPI_Comm new_comm = hypre_ParAMGDataNewComm(amg_data);
    HYPRE_Solver coarse_solver = hypre_ParAMGDataCoarseSolver(amg_data);
@@ -366,14 +373,14 @@ hypre_seqAMGCycle( hypre_ParAMGData *amg_data,
       HYPRE_Real     *f_data;
       hypre_Vector   *f_local;
       hypre_Vector   *tmp_vec;
-      
+
       HYPRE_Int nf;
       HYPRE_Int local_info;
       HYPRE_Real *recv_buf = NULL;
       HYPRE_Int *displs = NULL;
       HYPRE_Int *info = NULL;
       HYPRE_Int new_num_procs, my_id;
-      
+
       hypre_MPI_Comm_size(new_comm, &new_num_procs);
       hypre_MPI_Comm_rank(new_comm, &my_id);
 
@@ -394,9 +401,9 @@ hypre_seqAMGCycle( hypre_ParAMGData *amg_data,
          displs = hypre_CTAlloc(HYPRE_Int,  new_num_procs+1, HYPRE_MEMORY_HOST);
          displs[0] = 0;
          for (i=1; i < new_num_procs+1; i++)
-            displs[i] = displs[i-1]+info[i-1]; 
-      
-         if (F_coarse) 
+            displs[i] = displs[i-1]+info[i-1];
+
+         if (F_coarse)
          {
             tmp_vec =  hypre_ParVectorLocalVector(F_coarse);
             recv_buf = hypre_VectorData(tmp_vec);
@@ -417,7 +424,7 @@ hypre_seqAMGCycle( hypre_ParAMGData *amg_data,
          tmp_vec =  hypre_ParVectorLocalVector(U_coarse);
          recv_buf = hypre_VectorData(tmp_vec);
       }
-      
+
       /*then u */
       if (redundant)
       {
@@ -431,7 +438,7 @@ hypre_seqAMGCycle( hypre_ParAMGData *amg_data,
          hypre_MPI_Gatherv ( u_data, n, HYPRE_MPI_REAL,
                        recv_buf, info, displs,
                        HYPRE_MPI_REAL, 0, new_comm );
-         
+
       /* clean up */
       if (redundant || my_id ==0)
       {
@@ -473,20 +480,24 @@ hypre_seqAMGCycle( hypre_ParAMGData *amg_data,
 
 /* generate sub communicator, which contains no idle processors */
 
-HYPRE_Int hypre_GenerateSubComm(MPI_Comm comm, HYPRE_Int participate, MPI_Comm *new_comm_ptr) 
+HYPRE_Int hypre_GenerateSubComm(MPI_Comm comm, HYPRE_Int participate, MPI_Comm *new_comm_ptr)
 {
    MPI_Comm new_comm;
-   hypre_MPI_Group orig_group, new_group; 
+   hypre_MPI_Group orig_group, new_group;
    hypre_MPI_Op hypre_MPI_MERGE;
    HYPRE_Int *info, *ranks, new_num_procs, my_info, my_id, num_procs;
    HYPRE_Int *list_len;
 
-   hypre_MPI_Comm_rank(comm,&my_id);
+   hypre_MPI_Comm_rank(comm, &my_id);
 
-   if (participate) 
+   if (participate)
+   {
       my_info = 1;
+   }
    else
+   {
       my_info = 0;
+   }
 
    hypre_MPI_Allreduce(&my_info, &new_num_procs, 1, HYPRE_MPI_INT, hypre_MPI_SUM, comm);
 
@@ -496,26 +507,32 @@ HYPRE_Int hypre_GenerateSubComm(MPI_Comm comm, HYPRE_Int participate, MPI_Comm *
       *new_comm_ptr = new_comm;
       return 0;
    }
-   ranks =  hypre_CTAlloc(HYPRE_Int,  new_num_procs+2, HYPRE_MEMORY_HOST);
+
+   ranks = hypre_CTAlloc(HYPRE_Int, new_num_procs+2, HYPRE_MEMORY_HOST);
+
    if (new_num_procs == 1)
    {
-      if (participate) my_info = my_id;
+      if (participate)
+      {
+         my_info = my_id;
+      }
       hypre_MPI_Allreduce(&my_info, &ranks[2], 1, HYPRE_MPI_INT, hypre_MPI_SUM, comm);
    }
    else
    {
-      info =  hypre_CTAlloc(HYPRE_Int,  new_num_procs+2, HYPRE_MEMORY_HOST);
-      list_len =  hypre_CTAlloc(HYPRE_Int,  1, HYPRE_MEMORY_HOST);
+      info = hypre_CTAlloc(HYPRE_Int, new_num_procs+2, HYPRE_MEMORY_HOST);
+      list_len = hypre_CTAlloc(HYPRE_Int, 1, HYPRE_MEMORY_HOST);
 
-
-      if (participate) 
+      if (participate)
       {
          info[0] = 1;
          info[1] = 1;
          info[2] = my_id;
       }
       else
+      {
          info[0] = 0;
+      }
 
       list_len[0] = new_num_procs + 2;
 
@@ -525,20 +542,21 @@ HYPRE_Int hypre_GenerateSubComm(MPI_Comm comm, HYPRE_Int participate, MPI_Comm *
 
       hypre_MPI_Op_free (&hypre_MPI_MERGE);
 
-       hypre_TFree(list_len, HYPRE_MEMORY_HOST);
-       hypre_TFree(info, HYPRE_MEMORY_HOST);
+      hypre_TFree(list_len, HYPRE_MEMORY_HOST);
+      hypre_TFree(info, HYPRE_MEMORY_HOST);
    }
-   hypre_MPI_Comm_size(comm,&num_procs);
+
+   hypre_MPI_Comm_size(comm, &num_procs);
    hypre_MPI_Comm_group(comm, &orig_group);
    hypre_MPI_Group_incl(orig_group, new_num_procs, &ranks[2], &new_group);
    hypre_MPI_Comm_create(comm, new_group, &new_comm);
    hypre_MPI_Group_free(&new_group);
    hypre_MPI_Group_free(&orig_group);
 
-    hypre_TFree(ranks, HYPRE_MEMORY_HOST);
-   
+   hypre_TFree(ranks, HYPRE_MEMORY_HOST);
+
    *new_comm_ptr = new_comm;
-   
+
    return 0;
 }
 
@@ -547,7 +565,7 @@ void hypre_merge_lists (HYPRE_Int *list1, HYPRE_Int* list2, hypre_int *np1, hypr
 {
    HYPRE_Int i, len1, len2, indx1, indx2;
 
-   if (list1[0] == 0 || (list2[0] == 0 && list1[0] == 0))
+   if (list1[0] == 0)
    {
       return;
    }
@@ -557,12 +575,15 @@ void hypre_merge_lists (HYPRE_Int *list1, HYPRE_Int* list2, hypre_int *np1, hypr
       len1 = list1[1];
       len2 = list2[1];
       list2[1] = len1+len2;
-      if ((hypre_int)(list2[1]) > *np1+2) printf("segfault in MPI User function merge_list\n");
+      if ((hypre_int)(list2[1]) > *np1+2) // RL:???
+      {
+         printf("segfault in MPI User function merge_list\n");
+      }
       indx1 = len1+1;
       indx2 = len2+1;
       for (i=len1+len2+1; i > 1; i--)
       {
-	 if (indx2 > 1 && indx1 > 1 && list1[indx1] > list2[indx2])
+         if (indx2 > 1 && indx1 > 1 && list1[indx1] > list2[indx2])
          {
             list2[i] = list1[indx1];
             indx1--;
