@@ -1,18 +1,9 @@
-/*BHEADER**********************************************************************
- * Copyright (c) 2008,  Lawrence Livermore National Security, LLC.
- * Produced at the Lawrence Livermore National Laboratory.
- * This file is part of HYPRE.  See file COPYRIGHT for details.
+/******************************************************************************
+ * Copyright 1998-2019 Lawrence Livermore National Security, LLC and other
+ * HYPRE Project Developers. See the top-level COPYRIGHT file for details.
  *
- * HYPRE is free software; you can redistribute it and/or modify it under the
- * terms of the GNU Lesser General Public License (as published by the Free
- * Software Foundation) version 2.1 dated February 1999.
- *
- * $Revision$
- ***********************************************************************EHEADER*/
-
-
-
-
+ * SPDX-License-Identifier: (Apache-2.0 OR MIT)
+ ******************************************************************************/
 
 #ifndef hypre_ParAMG_DATA_HEADER
 #define hypre_ParAMG_DATA_HEADER
@@ -31,15 +22,17 @@ typedef struct
 
    /* setup params */
    HYPRE_Int      max_levels;
-   HYPRE_Real   strong_threshold;
-   HYPRE_Real   max_row_sum;
-   HYPRE_Real   trunc_factor;
-   HYPRE_Real   agg_trunc_factor;
-   HYPRE_Real   agg_P12_trunc_factor;
-   HYPRE_Real   jacobi_trunc_threshold;
-   HYPRE_Real   S_commpkg_switch;
-   HYPRE_Real   CR_rate;
-   HYPRE_Real   CR_strong_th;
+   HYPRE_Real     strong_threshold;
+   HYPRE_Real     max_row_sum;
+   HYPRE_Real     trunc_factor;
+   HYPRE_Real     agg_trunc_factor;
+   HYPRE_Real     agg_P12_trunc_factor;
+   HYPRE_Real     jacobi_trunc_threshold;
+   HYPRE_Real     S_commpkg_switch;
+   HYPRE_Real     CR_rate;
+   HYPRE_Real     CR_strong_th;
+   HYPRE_Real     A_drop_tol;
+   HYPRE_Int      A_drop_type;
    HYPRE_Int      measure_type;
    HYPRE_Int      setup_type;
    HYPRE_Int      coarsen_type;
@@ -50,6 +43,8 @@ typedef struct
    HYPRE_Int      agg_P_max_elmts;
    HYPRE_Int      agg_P12_max_elmts;
    HYPRE_Int      restr_par;
+   HYPRE_Int      is_triangular;
+   HYPRE_Int      gmres_switch;
    HYPRE_Int      agg_num_levels;
    HYPRE_Int      num_paths;
    HYPRE_Int      post_interp_type;
@@ -66,20 +61,20 @@ typedef struct
    /* solve params */
    HYPRE_Int      max_iter;
    HYPRE_Int      min_iter;
-   HYPRE_Int      cycle_type;    
-   HYPRE_Int     *num_grid_sweeps;  
-   HYPRE_Int     *grid_relax_type;   
+   HYPRE_Int      cycle_type;
+   HYPRE_Int     *num_grid_sweeps;
+   HYPRE_Int     *grid_relax_type;
    HYPRE_Int    **grid_relax_points;
    HYPRE_Int      relax_order;
-   HYPRE_Int      user_coarse_relax_type;   
-   HYPRE_Int      user_relax_type;   
+   HYPRE_Int      user_coarse_relax_type;
+   HYPRE_Int      user_relax_type;
    HYPRE_Int      user_num_sweeps;
-   HYPRE_Real     user_relax_weight;   
+   HYPRE_Real     user_relax_weight;
    HYPRE_Real     outer_wt;
-   HYPRE_Real  *relax_weight; 
-   HYPRE_Real  *omega;
-   HYPRE_Int    converge_type;
-   HYPRE_Real   tol;
+   HYPRE_Real    *relax_weight;
+   HYPRE_Real    *omega;
+   HYPRE_Int      converge_type;
+   HYPRE_Real     tol;
 
    /* problem data */
    hypre_ParCSRMatrix  *A;
@@ -90,7 +85,7 @@ typedef struct
    HYPRE_Int      nodal_diag;
    HYPRE_Int      num_points;
    HYPRE_Int     *dof_func;
-   HYPRE_Int     *dof_point;           
+   HYPRE_Int     *dof_point;
    HYPRE_Int     *point_dof_map;
 
    /* data generated in the setup phase */
@@ -99,11 +94,11 @@ typedef struct
    hypre_ParVector    **U_array;
    hypre_ParCSRMatrix **P_array;
    hypre_ParCSRMatrix **R_array;
-   HYPRE_Int                **CF_marker_array;
-   HYPRE_Int                **dof_func_array;
-   HYPRE_Int                **dof_point_array;
-   HYPRE_Int                **point_dof_map_array;
-   HYPRE_Int                  num_levels;
+   HYPRE_Int          **CF_marker_array;
+   HYPRE_Int          **dof_func_array;
+   HYPRE_Int          **dof_point_array;
+   HYPRE_Int          **point_dof_map_array;
+   HYPRE_Int            num_levels;
    HYPRE_Real         **l1_norms;
 
    /* Block data */
@@ -114,35 +109,35 @@ typedef struct
    HYPRE_Int block_mode;
 
    /* data for more complex smoothers */
-   HYPRE_Int                  smooth_num_levels;
-   HYPRE_Int                  smooth_type;
-   HYPRE_Solver        *smoother;
-   HYPRE_Int			smooth_num_sweeps;
-   HYPRE_Int                  schw_variant;
-   HYPRE_Int                  schw_overlap;
-   HYPRE_Int                  schw_domain_type;
-   HYPRE_Real		schwarz_rlx_weight;
-   HYPRE_Int                  schwarz_use_nonsymm;
-   HYPRE_Int			ps_sym;
-   HYPRE_Int			ps_level;
-   HYPRE_Int			pi_max_nz_per_row;
-   HYPRE_Int			eu_level;
-   HYPRE_Int			eu_bj;
-   HYPRE_Real		ps_threshold;
-   HYPRE_Real		ps_filter;
-   HYPRE_Real		pi_drop_tol;
-   HYPRE_Real		eu_sparse_A;
-   char		       *euclidfile;
+   HYPRE_Int           smooth_num_levels;
+   HYPRE_Int           smooth_type;
+   HYPRE_Solver       *smoother;
+   HYPRE_Int           smooth_num_sweeps;
+   HYPRE_Int           schw_variant;
+   HYPRE_Int           schw_overlap;
+   HYPRE_Int           schw_domain_type;
+   HYPRE_Real          schwarz_rlx_weight;
+   HYPRE_Int           schwarz_use_nonsymm;
+   HYPRE_Int           ps_sym;
+   HYPRE_Int           ps_level;
+   HYPRE_Int           pi_max_nz_per_row;
+   HYPRE_Int           eu_level;
+   HYPRE_Int           eu_bj;
+   HYPRE_Real          ps_threshold;
+   HYPRE_Real          ps_filter;
+   HYPRE_Real          pi_drop_tol;
+   HYPRE_Real          eu_sparse_A;
+   char               *euclidfile;
 
-   HYPRE_Real          *max_eig_est;
-   HYPRE_Real          *min_eig_est;
-   HYPRE_Int            cheby_eig_est;
-   HYPRE_Int            cheby_order;
-   HYPRE_Int            cheby_variant;
-   HYPRE_Int            cheby_scale;
-   HYPRE_Real           cheby_fraction;
-   HYPRE_Real         **cheby_ds;
-   HYPRE_Real         **cheby_coefs;
+   HYPRE_Real         *max_eig_est;
+   HYPRE_Real         *min_eig_est;
+   HYPRE_Int           cheby_eig_est;
+   HYPRE_Int           cheby_order;
+   HYPRE_Int           cheby_variant;
+   HYPRE_Int           cheby_scale;
+   HYPRE_Real          cheby_fraction;
+   HYPRE_Real        **cheby_ds;
+   HYPRE_Real        **cheby_coefs;
 
    /* data needed for non-Galerkin option */
    HYPRE_Int           nongalerk_num_tol;
@@ -160,8 +155,8 @@ typedef struct
    hypre_ParVector   *Ztemp;
 
    /* fields used by GSMG and LS interpolation */
-   HYPRE_Int                 gsmg;        /* nonzero indicates use of GSMG */
-   HYPRE_Int                 num_samples; /* number of sample vectors */
+   HYPRE_Int          gsmg;        /* nonzero indicates use of GSMG */
+   HYPRE_Int          num_samples; /* number of sample vectors */
 
    /* log info */
    HYPRE_Int      logging;
@@ -188,22 +183,22 @@ typedef struct
    /* data for fitting vectors in interpolation */
    HYPRE_Int               num_interp_vectors;
    HYPRE_Int               num_levels_interp_vectors; /* not set by user */
-   hypre_ParVector **interp_vectors;
-   hypre_ParVector ***interp_vectors_array;   
+   hypre_ParVector       **interp_vectors;
+   hypre_ParVector       ***interp_vectors_array;
    HYPRE_Int               interp_vec_variant;
    HYPRE_Int               interp_vec_first_level;
-   HYPRE_Real        interp_vectors_abs_q_trunc;
+   HYPRE_Real              interp_vectors_abs_q_trunc;
    HYPRE_Int               interp_vectors_q_max;
    HYPRE_Int               interp_refine;
    HYPRE_Int               smooth_interp_vectors;
-   HYPRE_Real       *expandp_weights; /* currently not set by user */
+   HYPRE_Real             *expandp_weights; /* currently not set by user */
 
    /* enable redundant coarse grid solve */
-   HYPRE_Solver   coarse_solver;
+   HYPRE_Solver         coarse_solver;
    hypre_ParCSRMatrix  *A_coarse;
-   hypre_ParVector  *f_coarse;
-   hypre_ParVector  *u_coarse;
-   MPI_Comm   new_comm;
+   hypre_ParVector     *f_coarse;
+   hypre_ParVector     *u_coarse;
+   MPI_Comm             new_comm;
 
    /* store matrix, vector and communication info for Gaussian elimination */
    HYPRE_Real *A_mat;
@@ -227,15 +222,14 @@ typedef struct
 
    /* Use 2 mat-mat-muls instead of triple product*/
    HYPRE_Int rap2;
-
    HYPRE_Int keepTranspose;
-	
-   /* information for preserving indeces as coarse grid points */
-   HYPRE_Int C_point_coarse_level;
-   HYPRE_Int num_C_point_marker;
-   HYPRE_Int   **C_point_marker_array;
 
-#ifdef HAVE_DSUPERLU
+   /* information for preserving indeces as coarse grid points */
+   HYPRE_Int   C_point_coarse_level;
+   HYPRE_Int   num_C_point_marker;
+   HYPRE_Int **C_point_marker_array;
+
+#ifdef HYPRE_USING_DSUPERLU
  /* Parameters and data for SuperLU_Dist */
    HYPRE_Int dslu_threshold;
    HYPRE_Solver dslu_solver;
@@ -248,8 +242,10 @@ typedef struct
  *--------------------------------------------------------------------------*/
 
 /* setup params */
-		  		      
+
 #define hypre_ParAMGDataRestriction(amg_data) ((amg_data)->restr_par)
+#define hypre_ParAMGDataIsTriangular(amg_data) ((amg_data)->is_triangular)
+#define hypre_ParAMGDataGMRESSwitchR(amg_data) ((amg_data)->gmres_switch)
 #define hypre_ParAMGDataMaxLevels(amg_data) ((amg_data)->max_levels)
 #define hypre_ParAMGDataStrongThreshold(amg_data) \
 ((amg_data)->strong_threshold)
@@ -274,13 +270,15 @@ typedef struct
 #define hypre_ParAMGDataNumCRRelaxSteps(amg_data) ((amg_data)->num_CR_relax_steps)
 #define hypre_ParAMGDataCRRate(amg_data) ((amg_data)->CR_rate)
 #define hypre_ParAMGDataCRStrongTh(amg_data) ((amg_data)->CR_strong_th)
+#define hypre_ParAMGDataADropTol(amg_data) ((amg_data)->A_drop_tol)
+#define hypre_ParAMGDataADropType(amg_data) ((amg_data)->A_drop_type)
 #define hypre_ParAMGDataISType(amg_data) ((amg_data)->IS_type)
 #define hypre_ParAMGDataCRUseCG(amg_data) ((amg_data)->CR_use_CG)
 #define hypre_ParAMGDataL1Norms(amg_data) ((amg_data)->l1_norms)
- #define hypre_ParAMGDataCGCIts(amg_data) ((amg_data)->cgc_its)
- #define hypre_ParAMGDataMaxCoarseSize(amg_data) ((amg_data)->max_coarse_size)
- #define hypre_ParAMGDataMinCoarseSize(amg_data) ((amg_data)->min_coarse_size)
- #define hypre_ParAMGDataSeqThreshold(amg_data) ((amg_data)->seq_threshold)
+#define hypre_ParAMGDataCGCIts(amg_data) ((amg_data)->cgc_its)
+#define hypre_ParAMGDataMaxCoarseSize(amg_data) ((amg_data)->max_coarse_size)
+#define hypre_ParAMGDataMinCoarseSize(amg_data) ((amg_data)->min_coarse_size)
+#define hypre_ParAMGDataSeqThreshold(amg_data) ((amg_data)->seq_threshold)
 
 /* solve params */
 
@@ -324,34 +322,34 @@ typedef struct
 #define hypre_ParAMGDataDofFuncArray(amg_data) ((amg_data)->dof_func_array)
 #define hypre_ParAMGDataDofPointArray(amg_data) ((amg_data)->dof_point_array)
 #define hypre_ParAMGDataPointDofMapArray(amg_data) \
-((amg_data)->point_dof_map_array) 
-#define hypre_ParAMGDataNumLevels(amg_data) ((amg_data)->num_levels)	
+((amg_data)->point_dof_map_array)
+#define hypre_ParAMGDataNumLevels(amg_data) ((amg_data)->num_levels)
 #define hypre_ParAMGDataSmoothType(amg_data) ((amg_data)->smooth_type)
 #define hypre_ParAMGDataSmoothNumLevels(amg_data) \
 ((amg_data)->smooth_num_levels)
 #define hypre_ParAMGDataSmoothNumSweeps(amg_data) \
-((amg_data)->smooth_num_sweeps)	
-#define hypre_ParAMGDataSmoother(amg_data) ((amg_data)->smoother)	
-#define hypre_ParAMGDataVariant(amg_data) ((amg_data)->schw_variant)	
-#define hypre_ParAMGDataOverlap(amg_data) ((amg_data)->schw_overlap)	
-#define hypre_ParAMGDataDomainType(amg_data) ((amg_data)->schw_domain_type)	
+((amg_data)->smooth_num_sweeps)
+#define hypre_ParAMGDataSmoother(amg_data) ((amg_data)->smoother)
+#define hypre_ParAMGDataVariant(amg_data) ((amg_data)->schw_variant)
+#define hypre_ParAMGDataOverlap(amg_data) ((amg_data)->schw_overlap)
+#define hypre_ParAMGDataDomainType(amg_data) ((amg_data)->schw_domain_type)
 #define hypre_ParAMGDataSchwarzRlxWeight(amg_data) \
 ((amg_data)->schwarz_rlx_weight)
 #define hypre_ParAMGDataSchwarzUseNonSymm(amg_data) \
 ((amg_data)->schwarz_use_nonsymm)
-#define hypre_ParAMGDataSym(amg_data) ((amg_data)->ps_sym)	
-#define hypre_ParAMGDataLevel(amg_data) ((amg_data)->ps_level)	
+#define hypre_ParAMGDataSym(amg_data) ((amg_data)->ps_sym)
+#define hypre_ParAMGDataLevel(amg_data) ((amg_data)->ps_level)
 #define hypre_ParAMGDataMaxNzPerRow(amg_data) ((amg_data)->pi_max_nz_per_row)
-#define hypre_ParAMGDataThreshold(amg_data) ((amg_data)->ps_threshold)	
-#define hypre_ParAMGDataFilter(amg_data) ((amg_data)->ps_filter)	
-#define hypre_ParAMGDataDropTol(amg_data) ((amg_data)->pi_drop_tol)	
-#define hypre_ParAMGDataEuclidFile(amg_data) ((amg_data)->euclidfile)	
-#define hypre_ParAMGDataEuLevel(amg_data) ((amg_data)->eu_level)	
+#define hypre_ParAMGDataThreshold(amg_data) ((amg_data)->ps_threshold)
+#define hypre_ParAMGDataFilter(amg_data) ((amg_data)->ps_filter)
+#define hypre_ParAMGDataDropTol(amg_data) ((amg_data)->pi_drop_tol)
+#define hypre_ParAMGDataEuclidFile(amg_data) ((amg_data)->euclidfile)
+#define hypre_ParAMGDataEuLevel(amg_data) ((amg_data)->eu_level)
 #define hypre_ParAMGDataEuSparseA(amg_data) ((amg_data)->eu_sparse_A)
 #define hypre_ParAMGDataEuBJ(amg_data) ((amg_data)->eu_bj)
 
-#define hypre_ParAMGDataMaxEigEst(amg_data) ((amg_data)->max_eig_est)	
-#define hypre_ParAMGDataMinEigEst(amg_data) ((amg_data)->min_eig_est)	
+#define hypre_ParAMGDataMaxEigEst(amg_data) ((amg_data)->max_eig_est)
+#define hypre_ParAMGDataMinEigEst(amg_data) ((amg_data)->min_eig_est)
 #define hypre_ParAMGDataChebyEigEst(amg_data) ((amg_data)->cheby_eig_est)
 #define hypre_ParAMGDataChebyVariant(amg_data) ((amg_data)->cheby_variant)
 #define hypre_ParAMGDataChebyScale(amg_data) ((amg_data)->cheby_scale)
@@ -457,13 +455,12 @@ typedef struct
 #define hypre_ParAMGDataCPointKeepLevel(amg_data) ((amg_data)-> C_point_keep_level)
 #define hypre_ParAMGDataNumCPointKeep(amg_data) ((amg_data)-> num_C_point_marker)
 
-#ifdef HAVE_DSUPERLU
+#ifdef HYPRE_USING_DSUPERLU
  /* Parameters and data for SuperLU_Dist */
 #define hypre_ParAMGDataDSLUThreshold(amg_data) ((amg_data)->dslu_threshold)
 #define hypre_ParAMGDataDSLUSolver(amg_data) ((amg_data)->dslu_solver)
 #endif
 
-
 #endif
 
 
