@@ -1,17 +1,9 @@
-/*BHEADER**********************************************************************
- * Copyright (c) 2008,  Lawrence Livermore National Security, LLC.
- * Produced at the Lawrence Livermore National Laboratory.
- * This file is part of HYPRE.  See file COPYRIGHT for details.
+/******************************************************************************
+ * Copyright 1998-2019 Lawrence Livermore National Security, LLC and other
+ * HYPRE Project Developers. See the top-level COPYRIGHT file for details.
  *
- * HYPRE is free software; you can redistribute it and/or modify it under the
- * terms of the GNU Lesser General Public License (as published by the Free
- * Software Foundation) version 2.1 dated February 1999.
- *
- * $Revision$
- ***********************************************************************EHEADER*/
-
-
-
+ * SPDX-License-Identifier: (Apache-2.0 OR MIT)
+ ******************************************************************************/
 
 /******************************************************************************
  *
@@ -71,7 +63,7 @@
   _hypre_parcsr_ls.h
 
   @return Error code.
-  
+
   @param A [IN]
   coefficient matrix
   @param strength_threshold [IN]
@@ -80,12 +72,12 @@
   parameter used to modify definition of strength for diagonal dominant matrices
   @param S_ptr [OUT]
   strength matrix
-  
+
   @see */
 /*--------------------------------------------------------------------------*/
 
 HYPRE_Int
-hypre_BoomerAMGCreateS(hypre_ParCSRMatrix    *A,
+hypre_BoomerAMGCreateSHost(hypre_ParCSRMatrix    *A,
                        HYPRE_Real             strength_threshold,
                        HYPRE_Real             max_row_sum,
                        HYPRE_Int                    num_functions,
@@ -116,7 +108,7 @@ hypre_BoomerAMGCreateS(hypre_ParCSRMatrix    *A,
    HYPRE_Int 	       num_nonzeros_diag;
    HYPRE_Int 	       num_nonzeros_offd = 0;
    HYPRE_Int 	       num_cols_offd = 0;
-                  
+
    hypre_ParCSRMatrix *S;
    hypre_CSRMatrix    *S_diag;
    HYPRE_Int          *S_diag_i;
@@ -126,10 +118,10 @@ hypre_BoomerAMGCreateS(hypre_ParCSRMatrix    *A,
    HYPRE_Int          *S_offd_i = NULL;
    HYPRE_Int          *S_offd_j = NULL;
    /* HYPRE_Real         *S_offd_data; */
-                 
+
    HYPRE_Real          diag, row_scale, row_sum;
    HYPRE_Int           i, jA, jS;
-                      
+
    HYPRE_Int           ierr = 0;
 
    HYPRE_Int          *dof_func_offd;
@@ -138,7 +130,7 @@ hypre_BoomerAMGCreateS(hypre_ParCSRMatrix    *A,
    HYPRE_Int		index, start, j;
 
    HYPRE_Int *prefix_sum_workspace;
-   
+
    /*--------------------------------------------------------------
     * Compute a  ParCSR strength matrix, S.
     *
@@ -160,8 +152,8 @@ hypre_BoomerAMGCreateS(hypre_ParCSRMatrix    *A,
    num_nonzeros_offd = A_offd_i[num_variables];
 
    S = hypre_ParCSRMatrixCreate(comm, global_num_vars, global_num_vars,
-			row_starts, row_starts,
-			num_cols_offd, num_nonzeros_diag, num_nonzeros_offd);
+         row_starts, row_starts,
+         num_cols_offd, num_nonzeros_diag, num_nonzeros_offd);
 /* row_starts is owned by A, col_starts = row_starts */
    hypre_ParCSRMatrixSetRowStartsOwner(S,0);
    S_diag = hypre_ParCSRMatrixDiag(S);
@@ -187,7 +179,9 @@ hypre_BoomerAMGCreateS(hypre_ParCSRMatrix    *A,
         HYPRE_BigInt *col_map_offd_S = hypre_TAlloc(HYPRE_BigInt,  num_cols_offd, HYPRE_MEMORY_HOST);
         hypre_ParCSRMatrixColMapOffd(S) = col_map_offd_S;
         if (num_functions > 1)
+        {
 	   dof_func_offd = hypre_CTAlloc(HYPRE_Int,  num_cols_offd, HYPRE_MEMORY_HOST);
+        }
 
         S_offd_j = hypre_TAlloc(HYPRE_Int, num_nonzeros_offd, HYPRE_MEMORY_HOST);
 
@@ -196,39 +190,39 @@ hypre_BoomerAMGCreateS(hypre_ParCSRMatrix    *A,
 #pragma omp parallel for private(i) HYPRE_SMP_SCHEDULE
 #endif
         for (i = 0; i < num_cols_offd; i++)
+        {
            col_map_offd_S[i] = col_map_offd_A[i];
+        }
    }
 
-
   /*-------------------------------------------------------------------
     * Get the dof_func data for the off-processor columns
     *-------------------------------------------------------------------*/
-
    if (!comm_pkg)
    {
 	hypre_MatvecCommPkgCreate(A);
 
-	comm_pkg = hypre_ParCSRMatrixCommPkg(A); 
+	comm_pkg = hypre_ParCSRMatrixCommPkg(A);
    }
 
    num_sends = hypre_ParCSRCommPkgNumSends(comm_pkg);
    if (num_functions > 1)
    {
-      int_buf_data = hypre_CTAlloc(HYPRE_Int, hypre_ParCSRCommPkgSendMapStart(comm_pkg, 
-						num_sends), HYPRE_MEMORY_HOST);
+      int_buf_data = hypre_CTAlloc(HYPRE_Int, hypre_ParCSRCommPkgSendMapStart(comm_pkg, num_sends),
+                                   HYPRE_MEMORY_HOST);
       index = 0;
       for (i = 0; i < num_sends; i++)
       {
 	 start = hypre_ParCSRCommPkgSendMapStart(comm_pkg, i);
 	 for (j=start; j < hypre_ParCSRCommPkgSendMapStart(comm_pkg, i+1); j++)
-		int_buf_data[index++] 
-		 = dof_func[hypre_ParCSRCommPkgSendMapElmt(comm_pkg,j)];
+         {
+            int_buf_data[index++] = dof_func[hypre_ParCSRCommPkgSendMapElmt(comm_pkg,j)];
+         }
       }
-	
-      comm_handle = hypre_ParCSRCommHandleCreate( 11, comm_pkg, int_buf_data, 
-	dof_func_offd);
 
-      hypre_ParCSRCommHandleDestroy(comm_handle);   
+      comm_handle = hypre_ParCSRCommHandleCreate( 11, comm_pkg, int_buf_data, dof_func_offd);
+
+      hypre_ParCSRCommHandleDestroy(comm_handle);
       hypre_TFree(int_buf_data, HYPRE_MEMORY_HOST);
    }
 
@@ -236,7 +230,6 @@ hypre_BoomerAMGCreateS(hypre_ParCSRMatrix    *A,
    prefix_sum_workspace = hypre_TAlloc(HYPRE_Int,  2*(hypre_NumThreads() + 1), HYPRE_MEMORY_HOST);
 
    /* give S same nonzero structure as A */
-
 #ifdef HYPRE_USING_OPENMP
 #pragma omp parallel private(i,diag,row_scale,row_sum,jA,jS)
 #endif
@@ -352,9 +345,9 @@ hypre_BoomerAMGCreateS(hypre_ParCSRMatrix    *A,
       else
       {
          if (num_functions > 1)
-         { 
-            if (diag < 0) 
-            { 
+         {
+            if (diag < 0)
+            {
                for (jA = A_diag_i[i]+1; jA < A_diag_i[i+1]; jA++)
                {
                   if (A_diag_data[jA] <= strength_threshold * row_scale
@@ -414,8 +407,8 @@ hypre_BoomerAMGCreateS(hypre_ParCSRMatrix    *A,
          } /* num_functions > 1 */
          else
          {
-            if (diag < 0) 
-            { 
+            if (diag < 0)
+            {
                for (jA = A_diag_i[i]+1; jA < A_diag_i[i+1]; jA++)
                {
                   if (A_diag_data[jA] <= strength_threshold * row_scale)
@@ -517,6 +510,9 @@ hypre_BoomerAMGCreateS(hypre_ParCSRMatrix    *A,
    hypre_CSRMatrixJ(S_diag) = S_diag_j;
    hypre_CSRMatrixJ(S_offd) = S_offd_j;
 
+   hypre_CSRMatrixMemoryLocation(S_diag) = HYPRE_MEMORY_HOST;
+   hypre_CSRMatrixMemoryLocation(S_offd) = HYPRE_MEMORY_HOST;
+
    hypre_ParCSRMatrixCommPkg(S) = NULL;
 
    *S_ptr        = S;
@@ -533,14 +529,55 @@ hypre_BoomerAMGCreateS(hypre_ParCSRMatrix    *A,
    return (ierr);
 }
 
-/* 
-   Create Strength matrix from CF marker array data. Provides a more 
+/* ----------------------------------------------------------------------- */
+HYPRE_Int
+hypre_BoomerAMGCreateS(hypre_ParCSRMatrix    *A,
+                       HYPRE_Real             strength_threshold,
+                       HYPRE_Real             max_row_sum,
+                       HYPRE_Int              num_functions,
+                       HYPRE_Int             *dof_func,
+                       hypre_ParCSRMatrix   **S_ptr)
+{
+#if defined(HYPRE_USING_CUDA)
+   //hypre_SetExecPolicy(HYPRE_EXEC_DEVICE);
+#endif
+
+   HYPRE_Int exec = hypre_GetExecPolicy1( hypre_CSRMatrixMemoryLocation(hypre_ParCSRMatrixDiag(A)) );
+
+   hypre_assert(exec != HYPRE_EXEC_UNSET);
+
+   HYPRE_Int ierr = 0;
+
+   if (exec == HYPRE_EXEC_HOST)
+   {
+      /*      printf(" createS Host \n");*/
+      ierr = hypre_BoomerAMGCreateSHost(A,strength_threshold,max_row_sum,num_functions,dof_func,S_ptr);
+   }
+#if defined(HYPRE_USING_CUDA)
+   else
+   {
+      /*      printf(" createS Device \n");*/
+      ierr = hypre_BoomerAMGCreateSDevice(A,strength_threshold,max_row_sum,num_functions,dof_func,S_ptr);
+   }
+#endif
+
+#if defined(HYPRE_USING_CUDA)
+   //hypre_SetExecPolicy(HYPRE_EXEC_HOST);
+#endif
+
+   return ierr;
+}
+
+
+/* ----------------------------------------------------------------------- */
+/*
+   Create Strength matrix from CF marker array data. Provides a more
    general form to build S for specific nodes of the 'global' matrix
-   (for example, F points or A_FF part), given the entire matrix. 
+   (for example, F points or A_FF part), given the entire matrix.
    These nodes have the SMRK tag.
-   
-   Currently assumes num_functions == 1, hence separate routine is used 
-   for now. Could possibly be merged with BoomerAMGCreateS() to yield a 
+
+   Currently assumes num_functions == 1, hence separate routine is used
+   for now. Could possibly be merged with BoomerAMGCreateS() to yield a
    more general function.
  */
 HYPRE_Int
@@ -574,7 +611,7 @@ hypre_BoomerAMGCreateSFromCFMarker(hypre_ParCSRMatrix    *A,
    HYPRE_Int           num_nonzeros_diag;
    HYPRE_Int           num_nonzeros_offd = 0;
    HYPRE_Int           num_cols_offd = 0;
-                  
+
    hypre_ParCSRMatrix *S;
    hypre_CSRMatrix    *S_diag;
    HYPRE_Int          *S_diag_i;
@@ -584,14 +621,14 @@ hypre_BoomerAMGCreateSFromCFMarker(hypre_ParCSRMatrix    *A,
    HYPRE_Int          *S_offd_i = NULL;
    HYPRE_Int          *S_offd_j = NULL;
    /* HYPRE_Real         *S_offd_data; */
-                 
+
    HYPRE_Real         diag, row_scale, row_sum;
    HYPRE_Int          i, jj, jA, jS;
-                      
+
    HYPRE_Int          ierr = 0;
 
    HYPRE_Int *prefix_sum_workspace;
-   
+
    /*--------------------------------------------------------------
     * Compute a  ParCSR strength matrix, S.
     *
@@ -672,7 +709,7 @@ hypre_BoomerAMGCreateSFromCFMarker(hypre_ParCSRMatrix    *A,
    {
   hypre_MatvecCommPkgCreate(A);
 
-  comm_pkg = hypre_ParCSRMatrixCommPkg(A); 
+  comm_pkg = hypre_ParCSRMatrixCommPkg(A);
    }
 
    /*HYPRE_Int prefix_sum_workspace[2*(hypre_NumThreads() + 1)];*/
@@ -729,7 +766,7 @@ hypre_BoomerAMGCreateSFromCFMarker(hypre_ParCSRMatrix    *A,
           if (CF_marker[jj] == SMRK) {
             row_scale = hypre_min(row_scale, A_diag_data[jA]);
             row_sum += A_diag_data[jA];
-          } 
+          }
         }
         for (jA = A_offd_i[i]; jA < A_offd_i[i+1]; jA++)
         {
@@ -764,8 +801,8 @@ hypre_BoomerAMGCreateSFromCFMarker(hypre_ParCSRMatrix    *A,
       }
       else
       {
-        if (diag < 0) 
-        { 
+        if (diag < 0)
+        {
           for (jA = A_diag_i[i]+1; jA < A_diag_i[i+1]; jA++)
           {
             jj = A_diag_j[jA];
@@ -909,6 +946,9 @@ hypre_BoomerAMGCreateSFromCFMarker(hypre_ParCSRMatrix    *A,
    hypre_CSRMatrixJ(S_diag) = S_diag_j;
    hypre_CSRMatrixJ(S_offd) = S_offd_j;
 
+   hypre_CSRMatrixMemoryLocation(S_diag) = HYPRE_MEMORY_HOST;
+   hypre_CSRMatrixMemoryLocation(S_offd) = HYPRE_MEMORY_HOST;
+
    hypre_ParCSRMatrixCommPkg(S) = NULL;
 
    *S_ptr        = S;
@@ -971,7 +1011,7 @@ hypre_BoomerAMGCreateSFromCFMarker(hypre_ParCSRMatrix    *A,
   _hypre_parcsr_ls.h
 
   @return Error code.
-  
+
   @param A [IN]
   coefficient matrix
   @param strength_threshold [IN]
@@ -980,7 +1020,7 @@ hypre_BoomerAMGCreateSFromCFMarker(hypre_ParCSRMatrix    *A,
   parameter used to modify definition of strength for diagonal dominant matrices
   @param S_ptr [OUT]
   strength matrix
-  
+
   @see */
 /*--------------------------------------------------------------------------*/
 
@@ -1012,7 +1052,7 @@ hypre_BoomerAMGCreateSabs(hypre_ParCSRMatrix    *A,
    HYPRE_Int 	       num_nonzeros_diag;
    HYPRE_Int 	       num_nonzeros_offd = 0;
    HYPRE_Int 	       num_cols_offd = 0;
-                  
+
    hypre_ParCSRMatrix *S;
    hypre_CSRMatrix    *S_diag;
    HYPRE_Int          *S_diag_i;
@@ -1022,17 +1062,17 @@ hypre_BoomerAMGCreateSabs(hypre_ParCSRMatrix    *A,
    HYPRE_Int          *S_offd_i = NULL;
    HYPRE_Int          *S_offd_j = NULL;
    /* HYPRE_Real         *S_offd_data; */
-                 
+
    HYPRE_Real          diag, row_scale, row_sum;
    HYPRE_Int           i, jA, jS;
-                      
+
    HYPRE_Int           ierr = 0;
 
    HYPRE_Int          *dof_func_offd;
    HYPRE_Int		num_sends;
    HYPRE_Int	       *int_buf_data;
    HYPRE_Int		index, start, j;
-   
+
    /*--------------------------------------------------------------
     * Compute a  ParCSR strength matrix, S.
     *
@@ -1056,6 +1096,7 @@ hypre_BoomerAMGCreateSabs(hypre_ParCSRMatrix    *A,
    S = hypre_ParCSRMatrixCreate(comm, global_num_vars, global_num_vars,
 			row_starts, row_starts,
 			num_cols_offd, num_nonzeros_diag, num_nonzeros_offd);
+
 /* row_starts is owned by A, col_starts = row_starts */
    hypre_ParCSRMatrixSetRowStartsOwner(S,0);
    S_diag = hypre_ParCSRMatrixDiag(S);
@@ -1068,6 +1109,9 @@ hypre_BoomerAMGCreateSabs(hypre_ParCSRMatrix    *A,
    S_diag_j = hypre_CSRMatrixJ(S_diag);
    S_offd_i = hypre_CSRMatrixI(S_offd);
 
+   hypre_CSRMatrixMemoryLocation(S_diag) = HYPRE_MEMORY_HOST;
+   hypre_CSRMatrixMemoryLocation(S_offd) = HYPRE_MEMORY_HOST;
+
    dof_func_offd = NULL;
 
    if (num_cols_offd)
@@ -1089,27 +1133,26 @@ hypre_BoomerAMGCreateSabs(hypre_ParCSRMatrix    *A,
    {
 	hypre_MatvecCommPkgCreate(A);
 
-	comm_pkg = hypre_ParCSRMatrixCommPkg(A); 
+	comm_pkg = hypre_ParCSRMatrixCommPkg(A);
    }
 
    num_sends = hypre_ParCSRCommPkgNumSends(comm_pkg);
    if (num_functions > 1)
    {
-      int_buf_data = hypre_CTAlloc(HYPRE_Int, hypre_ParCSRCommPkgSendMapStart(comm_pkg, 
+      int_buf_data = hypre_CTAlloc(HYPRE_Int, hypre_ParCSRCommPkgSendMapStart(comm_pkg,
 						num_sends), HYPRE_MEMORY_HOST);
       index = 0;
       for (i = 0; i < num_sends; i++)
       {
 	 start = hypre_ParCSRCommPkgSendMapStart(comm_pkg, i);
 	 for (j=start; j < hypre_ParCSRCommPkgSendMapStart(comm_pkg, i+1); j++)
-		int_buf_data[index++] 
+		int_buf_data[index++]
 		 = dof_func[hypre_ParCSRCommPkgSendMapElmt(comm_pkg,j)];
       }
-	
-      comm_handle = hypre_ParCSRCommHandleCreate( 11, comm_pkg, int_buf_data, 
-	dof_func_offd);
 
-      hypre_ParCSRCommHandleDestroy(comm_handle);   
+      comm_handle = hypre_ParCSRCommHandleCreate( 11, comm_pkg, int_buf_data, dof_func_offd);
+
+      hypre_ParCSRCommHandleDestroy(comm_handle);
       hypre_TFree(int_buf_data, HYPRE_MEMORY_HOST);
    }
 
@@ -1125,7 +1168,7 @@ hypre_BoomerAMGCreateSabs(hypre_ParCSRMatrix    *A,
 
       /* compute scaling factor and row sum */
       row_scale = 0.0;
-      row_sum = diag;
+      row_sum = fabs(diag);
       if (num_functions > 1)
       {
             for (jA = A_diag_i[i]+1; jA < A_diag_i[i+1]; jA++)
@@ -1160,8 +1203,8 @@ hypre_BoomerAMGCreateSabs(hypre_ParCSRMatrix    *A,
       }
 
       /* compute row entries of S */
-      S_diag_j[A_diag_i[i]] = -1;
-      if ((fabs(row_sum) > fabs(diag)*max_row_sum) && (max_row_sum < 1.0))
+      S_diag_j[A_diag_i[i]] = -1; /* reject diag entry */
+      if ( fabs(row_sum) < fabs(diag)*(2.0-max_row_sum) && max_row_sum < 1.0 )
       {
          /* make all dependencies weak */
          for (jA = A_diag_i[i]+1; jA < A_diag_i[i+1]; jA++)
@@ -1176,7 +1219,7 @@ hypre_BoomerAMGCreateSabs(hypre_ParCSRMatrix    *A,
       else
       {
          if (num_functions > 1)
-         { 
+         {
                for (jA = A_diag_i[i]+1; jA < A_diag_i[i+1]; jA++)
                {
                   if (fabs(A_diag_data[jA]) <= strength_threshold * row_scale
@@ -1269,7 +1312,7 @@ hypre_BoomerAMGCreateSabs(hypre_ParCSRMatrix    *A,
 /*--------------------------------------------------------------------------*/
 
 HYPRE_Int
-hypre_BoomerAMGCreateSCommPkg(hypre_ParCSRMatrix *A, 
+hypre_BoomerAMGCreateSCommPkg(hypre_ParCSRMatrix *A,
 			      hypre_ParCSRMatrix *S,
 			      HYPRE_Int		 **col_offd_S_to_A_ptr)
 {
@@ -1281,7 +1324,7 @@ hypre_BoomerAMGCreateSCommPkg(hypre_ParCSRMatrix *A,
    hypre_ParCSRCommHandle  *comm_handle;
    hypre_CSRMatrix         *A_offd = hypre_ParCSRMatrixOffd(A);
    HYPRE_BigInt  	   *col_map_offd_A = hypre_ParCSRMatrixColMapOffd(A);
-                  
+
    hypre_CSRMatrix    *S_diag = hypre_ParCSRMatrixDiag(S);
    hypre_CSRMatrix    *S_offd = hypre_ParCSRMatrixOffd(S);
    HYPRE_Int          *S_offd_i = hypre_CSRMatrixI(S_offd);
@@ -1289,11 +1332,11 @@ hypre_BoomerAMGCreateSCommPkg(hypre_ParCSRMatrix *A,
    HYPRE_BigInt       *col_map_offd_S = hypre_ParCSRMatrixColMapOffd(S);
 
    HYPRE_Int          *recv_procs_A = hypre_ParCSRCommPkgRecvProcs(comm_pkg_A);
-   HYPRE_Int          *recv_vec_starts_A = 
+   HYPRE_Int          *recv_vec_starts_A =
 				hypre_ParCSRCommPkgRecvVecStarts(comm_pkg_A);
-   HYPRE_Int          *send_procs_A = 
+   HYPRE_Int          *send_procs_A =
 				hypre_ParCSRCommPkgSendProcs(comm_pkg_A);
-   HYPRE_Int          *send_map_starts_A = 
+   HYPRE_Int          *send_map_starts_A =
 				hypre_ParCSRCommPkgSendMapStarts(comm_pkg_A);
    HYPRE_Int          *recv_procs_S;
    HYPRE_Int          *recv_vec_starts_S;
@@ -1308,12 +1351,12 @@ hypre_BoomerAMGCreateSCommPkg(hypre_ParCSRMatrix *A,
    HYPRE_Int          *recv_change;
 
    HYPRE_Int           num_variables   = hypre_CSRMatrixNumRows(S_diag);
-   HYPRE_Int	       num_cols_offd_A = hypre_CSRMatrixNumCols(A_offd);                 
+   HYPRE_Int	       num_cols_offd_A = hypre_CSRMatrixNumCols(A_offd);
    HYPRE_Int	       num_cols_offd_S;
    HYPRE_Int           i, j, jcol;
    HYPRE_Int           proc, cnt, proc_cnt, total_nz;
    HYPRE_BigInt        first_row;
-                      
+
    HYPRE_Int           ierr = 0;
 
    HYPRE_Int	       num_sends_A = hypre_ParCSRCommPkgNumSends(comm_pkg_A);
@@ -1356,7 +1399,7 @@ hypre_BoomerAMGCreateSCommPkg(hypre_ParCSRMatrix *A,
    }
 
 
-   num_cols_offd_S = cnt;  
+   num_cols_offd_S = cnt;
    recv_change = NULL;
    recv_procs_S = NULL;
    send_change = NULL;
@@ -1418,7 +1461,7 @@ hypre_BoomerAMGCreateSCommPkg(hypre_ParCSRMatrix *A,
          recv_vec_starts_S[i] = recv_vec_starts_A[i];
       }
       recv_vec_starts_S[num_recvs_A] = recv_vec_starts_A[num_recvs_A];
-   } 
+   }
 
    requests = hypre_CTAlloc(hypre_MPI_Request, num_sends_A+num_recvs_A, HYPRE_MEMORY_HOST);
    j=0;
@@ -1492,7 +1535,7 @@ hypre_BoomerAMGCreateSCommPkg(hypre_ParCSRMatrix *A,
           send_map_elmts_S[i] = (HYPRE_Int)(big_send_map_elmts_S[i]-first_row);
 
    hypre_ParCSRCommPkgSendMapElmts(comm_pkg_S) = send_map_elmts_S;
-  
+
    hypre_ParCSRMatrixCommPkg(S) = comm_pkg_S;
    hypre_ParCSRMatrixColMapOffd(S) = col_map_offd_S;
    hypre_CSRMatrixNumCols(S_offd) = num_cols_offd_S;
@@ -1504,15 +1547,16 @@ hypre_BoomerAMGCreateSCommPkg(hypre_ParCSRMatrix *A,
    *col_offd_S_to_A_ptr = col_offd_S_to_A;
 
    return ierr;
-} 
+}
 
 /*--------------------------------------------------------------------------
  * hypre_BoomerAMGCreate2ndS : creates strength matrix on coarse points
  * for second coarsening pass in aggressive coarsening (S*S+2S)
  *--------------------------------------------------------------------------*/
 
-HYPRE_Int hypre_BoomerAMGCreate2ndS( hypre_ParCSRMatrix *S, HYPRE_Int *CF_marker, 
-	HYPRE_Int num_paths, HYPRE_BigInt *coarse_row_starts, hypre_ParCSRMatrix **C_ptr)
+HYPRE_Int hypre_BoomerAMGCreate2ndS( hypre_ParCSRMatrix *S, HYPRE_Int *CF_marker,
+                                     HYPRE_Int num_paths, HYPRE_BigInt *coarse_row_starts,
+                                     hypre_ParCSRMatrix **C_ptr)
 {
 #ifdef HYPRE_PROFILE
    hypre_profile_times[HYPRE_TIMER_ID_CREATE_2NDS] -= hypre_MPI_Wtime();
@@ -1524,18 +1568,18 @@ HYPRE_Int hypre_BoomerAMGCreate2ndS( hypre_ParCSRMatrix *S, HYPRE_Int *CF_marker
    hypre_ParCSRCommHandle *comm_handle;
 
    hypre_CSRMatrix *S_diag = hypre_ParCSRMatrixDiag(S);
-   
+
    HYPRE_Int             *S_diag_i = hypre_CSRMatrixI(S_diag);
    HYPRE_Int             *S_diag_j = hypre_CSRMatrixJ(S_diag);
 
    hypre_CSRMatrix *S_offd = hypre_ParCSRMatrixOffd(S);
-   
+
    HYPRE_Int             *S_offd_i = hypre_CSRMatrixI(S_offd);
    HYPRE_Int             *S_offd_j = hypre_CSRMatrixJ(S_offd);
 
    HYPRE_Int	num_cols_diag_S = hypre_CSRMatrixNumCols(S_diag);
    HYPRE_Int	num_cols_offd_S = hypre_CSRMatrixNumCols(S_offd);
-   
+
    hypre_ParCSRMatrix *S2;
    HYPRE_BigInt		 *col_map_offd_C = NULL;
 
@@ -1552,7 +1596,7 @@ HYPRE_Int hypre_BoomerAMGCreate2ndS( hypre_ParCSRMatrix *S, HYPRE_Int *CF_marker
    HYPRE_Int             *C_offd_j=NULL;
 
    HYPRE_Int		    num_cols_offd_C = 0;
-   
+
    HYPRE_Int             *S_ext_diag_i = NULL;
    HYPRE_Int             *S_ext_diag_j = NULL;
    HYPRE_Int              S_ext_diag_size = 0;
@@ -1586,7 +1630,7 @@ HYPRE_Int hypre_BoomerAMGCreate2ndS( hypre_ParCSRMatrix *S, HYPRE_Int *CF_marker
    HYPRE_Int              i1, i2, i3;
    HYPRE_BigInt           big_i1;
    HYPRE_Int              jj1, jj2, jrow, j_cnt;
-   
+
    /*HYPRE_Int              cnt, cnt_offd, cnt_diag;*/
    HYPRE_Int 		    num_procs, my_id;
    HYPRE_Int 		    index;
@@ -1609,7 +1653,7 @@ HYPRE_Int hypre_BoomerAMGCreate2ndS( hypre_ParCSRMatrix *S, HYPRE_Int *CF_marker
 
    /*-----------------------------------------------------------------------
     *  Extract S_ext, i.e. portion of B that is stored on neighbor procs
-    *  and needed locally for matrix matrix product 
+    *  and needed locally for matrix matrix product
     *-----------------------------------------------------------------------*/
 
    hypre_MPI_Comm_size(comm, &num_procs);
@@ -1699,7 +1743,7 @@ HYPRE_Int hypre_BoomerAMGCreate2ndS( hypre_ParCSRMatrix *S, HYPRE_Int *CF_marker
 
       comm_handle = hypre_ParCSRCommHandleCreate( 21, comm_pkg, big_int_buf_data,
            fine_to_coarse_offd);
-                                                                                
+
       hypre_ParCSRCommHandleDestroy(comm_handle);
 
       int_buf_data = hypre_TAlloc(HYPRE_Int,  end, HYPRE_MEMORY_HOST);
@@ -1710,10 +1754,10 @@ HYPRE_Int hypre_BoomerAMGCreate2ndS( hypre_ParCSRMatrix *S, HYPRE_Int *CF_marker
       {
          int_buf_data[index - begin] = CF_marker[send_map_elmts[index]];
       }
-                                                                                
+
       comm_handle = hypre_ParCSRCommHandleCreate(11, comm_pkg, int_buf_data,
                 CF_marker_offd);
-                                                                                
+
       hypre_ParCSRCommHandleDestroy(comm_handle);
       hypre_TFree(int_buf_data, HYPRE_MEMORY_HOST);
       hypre_TFree(big_int_buf_data, HYPRE_MEMORY_HOST);
@@ -1746,19 +1790,19 @@ HYPRE_Int hypre_BoomerAMGCreate2ndS( hypre_ParCSRMatrix *S, HYPRE_Int *CF_marker
          S_int_i[j - begin + 1] = index;
          num_nonzeros += S_int_i[j - begin + 1];
       }
-                                                                                
+
 /*--------------------------------------------------------------------------
  * initialize communication
  *--------------------------------------------------------------------------*/
       if (num_procs > 1)
-         comm_handle = 
+         comm_handle =
 		hypre_ParCSRCommHandleCreate(11,comm_pkg,&S_int_i[1],&S_ext_i[1]);
 
       if (num_nonzeros) S_int_j = hypre_TAlloc(HYPRE_BigInt,  num_nonzeros, HYPRE_MEMORY_HOST);
 
       tmp_send_map_starts = hypre_CTAlloc(HYPRE_Int,  num_sends+1, HYPRE_MEMORY_HOST);
       tmp_recv_vec_starts = hypre_CTAlloc(HYPRE_Int,  num_recvs+1, HYPRE_MEMORY_HOST);
-   
+
       tmp_send_map_starts[0] = 0;
       j_cnt = 0;
       for (i=0; i < num_sends; i++)
@@ -1779,17 +1823,17 @@ HYPRE_Int hypre_BoomerAMGCreate2ndS( hypre_ParCSRMatrix *S, HYPRE_Int *CF_marker
          }
          tmp_send_map_starts[i+1] = j_cnt;
       }
-                                                                                
+
       tmp_comm_pkg = hypre_CTAlloc(hypre_ParCSRCommPkg, 1, HYPRE_MEMORY_HOST);
       hypre_ParCSRCommPkgComm(tmp_comm_pkg) = comm;
       hypre_ParCSRCommPkgNumSends(tmp_comm_pkg) = num_sends;
       hypre_ParCSRCommPkgNumRecvs(tmp_comm_pkg) = num_recvs;
-      hypre_ParCSRCommPkgSendProcs(tmp_comm_pkg) = 
+      hypre_ParCSRCommPkgSendProcs(tmp_comm_pkg) =
 		hypre_ParCSRCommPkgSendProcs(comm_pkg);
-      hypre_ParCSRCommPkgRecvProcs(tmp_comm_pkg) = 
+      hypre_ParCSRCommPkgRecvProcs(tmp_comm_pkg) =
 		hypre_ParCSRCommPkgRecvProcs(comm_pkg);
       hypre_ParCSRCommPkgSendMapStarts(tmp_comm_pkg) = tmp_send_map_starts;
-                                                                                
+
       hypre_ParCSRCommHandleDestroy(comm_handle);
       comm_handle = NULL;
 /*--------------------------------------------------------------------------
@@ -1800,9 +1844,9 @@ HYPRE_Int hypre_BoomerAMGCreate2ndS( hypre_ParCSRMatrix *S, HYPRE_Int *CF_marker
 
       for (i=0; i < recv_vec_starts[num_recvs]; i++)
                 S_ext_i[i+1] += S_ext_i[i];
-                                                                                
+
       num_nonzeros = S_ext_i[recv_vec_starts[num_recvs]];
-                                                                                
+
       if (num_nonzeros) S_ext_j = hypre_TAlloc(HYPRE_BigInt,  num_nonzeros, HYPRE_MEMORY_HOST);
 
       tmp_recv_vec_starts[0] = 0;
@@ -1810,7 +1854,7 @@ HYPRE_Int hypre_BoomerAMGCreate2ndS( hypre_ParCSRMatrix *S, HYPRE_Int *CF_marker
          tmp_recv_vec_starts[i+1] = S_ext_i[recv_vec_starts[i+1]];
 
       hypre_ParCSRCommPkgRecvVecStarts(tmp_comm_pkg) = tmp_recv_vec_starts;
-                                                                                
+
       comm_handle = hypre_ParCSRCommHandleCreate(21,tmp_comm_pkg,S_int_j,S_ext_j);
       hypre_ParCSRCommHandleDestroy(comm_handle);
       comm_handle = NULL;
@@ -1833,7 +1877,6 @@ HYPRE_Int hypre_BoomerAMGCreate2ndS( hypre_ParCSRMatrix *S, HYPRE_Int *CF_marker
       S_ext_offd_i = hypre_TAlloc(HYPRE_Int,  num_cols_offd_S+1, HYPRE_MEMORY_HOST);
       S_ext_offd_i[0] = 0;
 
-
       hypre_UnorderedBigIntSet found_set;
       hypre_UnorderedBigIntSetCreate(&found_set, S_ext_i[num_cols_offd_S] + num_cols_offd_S, 16*hypre_NumThreads());
 
@@ -1888,8 +1931,8 @@ HYPRE_Int hypre_BoomerAMGCreate2ndS( hypre_ParCSRMatrix *S, HYPRE_Int *CF_marker
             {
                big_i1 = S_ext_j[j];
                if (big_i1 < my_first_cpt || big_i1 > my_last_cpt)
-                  S_big_offd_j[S_ext_offd_size_private++] = big_i1; 
-                  //S_ext_offd_j[S_ext_offd_size_private++] = big_i1; 
+                  S_big_offd_j[S_ext_offd_size_private++] = big_i1;
+                  //S_ext_offd_j[S_ext_offd_size_private++] = big_i1;
                else
                   S_ext_diag_j[S_ext_diag_size_private++] = (HYPRE_Int)(big_i1 - my_first_cpt);
             }
@@ -1899,7 +1942,7 @@ HYPRE_Int hypre_BoomerAMGCreate2ndS( hypre_ParCSRMatrix *S, HYPRE_Int *CF_marker
       } // omp parallel
 
       temp = hypre_UnorderedBigIntSetCopyToArray(&found_set, &num_cols_offd_C);
-      
+
       hypre_UnorderedBigIntSetDestroy(&found_set);
       hypre_TFree(S_ext_i, HYPRE_MEMORY_HOST);
 
@@ -2114,11 +2157,11 @@ HYPRE_Int hypre_BoomerAMGCreate2ndS( hypre_ParCSRMatrix *S, HYPRE_Int *CF_marker
          for (ic = ic_begin; ic < ic_end; ic++)
          {
             /*--------------------------------------------------------------------
-             *  Set marker for diagonal entry, C_{i1,i1} (for square matrices). 
+             *  Set marker for diagonal entry, C_{i1,i1} (for square matrices).
              *--------------------------------------------------------------------*/
 
              i1 = coarse_to_fine[ic];
-       
+
              HYPRE_Int jj_row_begin_diag = num_nonzeros_diag;
              HYPRE_Int jj_row_begin_offd = num_nonzeros_offd;
 
@@ -2206,11 +2249,11 @@ HYPRE_Int hypre_BoomerAMGCreate2ndS( hypre_ParCSRMatrix *S, HYPRE_Int *CF_marker
          for (ic = ic_begin; ic < ic_end; ic++)
          {
             /*--------------------------------------------------------------------
-             *  Set marker for diagonal entry, C_{i1,i1} (for square matrices). 
+             *  Set marker for diagonal entry, C_{i1,i1} (for square matrices).
              *--------------------------------------------------------------------*/
 
              i1 = coarse_to_fine[ic];
-       
+
              HYPRE_Int jj_row_begin_diag = jj_count_diag;
              HYPRE_Int jj_row_begin_offd = jj_count_offd;
 
@@ -2400,11 +2443,11 @@ HYPRE_Int hypre_BoomerAMGCreate2ndS( hypre_ParCSRMatrix *S, HYPRE_Int *CF_marker
          for (ic = ic_begin; ic < ic_end; ic++)
          {
             /*--------------------------------------------------------------------
-             *  Set marker for diagonal entry, C_{i1,i1} (for square matrices). 
+             *  Set marker for diagonal entry, C_{i1,i1} (for square matrices).
              *--------------------------------------------------------------------*/
 
              i1 = coarse_to_fine[ic];
-       
+
              HYPRE_Int jj_row_begin_diag = num_nonzeros_diag;
              HYPRE_Int jj_row_begin_offd = num_nonzeros_offd;
 
@@ -2495,11 +2538,11 @@ HYPRE_Int hypre_BoomerAMGCreate2ndS( hypre_ParCSRMatrix *S, HYPRE_Int *CF_marker
          for (ic = ic_begin; ic < ic_end; ic++)
          {
             /*--------------------------------------------------------------------
-             *  Set marker for diagonal entry, C_{i1,i1} (for square matrices). 
+             *  Set marker for diagonal entry, C_{i1,i1} (for square matrices).
              *--------------------------------------------------------------------*/
 
              i1 = coarse_to_fine[ic];
-       
+
              HYPRE_Int jj_row_begin_diag = jj_count_diag;
              HYPRE_Int jj_row_begin_offd = jj_count_offd;
 
@@ -2633,23 +2676,23 @@ HYPRE_Int hypre_BoomerAMGCreate2ndS( hypre_ParCSRMatrix *S, HYPRE_Int *CF_marker
       } /* num_paths > 1 */
    } /* omp parallel */
 
-   S2 = hypre_ParCSRMatrixCreate(comm, global_num_coarse, 
+   S2 = hypre_ParCSRMatrixCreate(comm, global_num_coarse,
 	global_num_coarse, coarse_row_starts,
 	coarse_row_starts, num_cols_offd_C, C_diag_i[num_coarse], C_offd_i[num_coarse]);
 
    hypre_ParCSRMatrixOwnsRowStarts(S2) = 0;
 
    C_diag = hypre_ParCSRMatrixDiag(S2);
-   hypre_CSRMatrixI(C_diag) = C_diag_i; 
-   if (C_diag_i[num_coarse]) hypre_CSRMatrixJ(C_diag) = C_diag_j; 
+   hypre_CSRMatrixI(C_diag) = C_diag_i;
+   if (C_diag_i[num_coarse]) hypre_CSRMatrixJ(C_diag) = C_diag_j;
 
    C_offd = hypre_ParCSRMatrixOffd(S2);
-   hypre_CSRMatrixI(C_offd) = C_offd_i; 
+   hypre_CSRMatrixI(C_offd) = C_offd_i;
    hypre_ParCSRMatrixOffd(S2) = C_offd;
 
    if (num_cols_offd_C)
    {
-      if (C_offd_i[num_coarse]) hypre_CSRMatrixJ(C_offd) = C_offd_j; 
+      if (C_offd_i[num_coarse]) hypre_CSRMatrixJ(C_offd) = C_offd_j;
       hypre_ParCSRMatrixColMapOffd(S2) = col_map_offd_C;
    }
 
@@ -2665,8 +2708,8 @@ HYPRE_Int hypre_BoomerAMGCreate2ndS( hypre_ParCSRMatrix *S, HYPRE_Int *CF_marker
    hypre_TFree(S_marker_array, HYPRE_MEMORY_HOST);
    hypre_TFree(S_marker_offd_array, HYPRE_MEMORY_HOST);
 
-   hypre_TFree(S_marker, HYPRE_MEMORY_HOST);   
-   hypre_TFree(S_marker_offd, HYPRE_MEMORY_HOST);   
+   hypre_TFree(S_marker, HYPRE_MEMORY_HOST);
+   hypre_TFree(S_marker_offd, HYPRE_MEMORY_HOST);
    hypre_TFree(S_ext_diag_i, HYPRE_MEMORY_HOST);
    hypre_TFree(fine_to_coarse, HYPRE_MEMORY_HOST);
    hypre_TFree(coarse_to_fine, HYPRE_MEMORY_HOST);
@@ -2679,13 +2722,16 @@ HYPRE_Int hypre_BoomerAMGCreate2ndS( hypre_ParCSRMatrix *S, HYPRE_Int *CF_marker
    {
       hypre_TFree(S_ext_offd_j, HYPRE_MEMORY_HOST);
    }
-   if (num_cols_offd_S) 
+   if (num_cols_offd_S)
    {
       hypre_TFree(map_S_to_C, HYPRE_MEMORY_HOST);
       hypre_TFree(CF_marker_offd, HYPRE_MEMORY_HOST);
       hypre_TFree(fine_to_coarse_offd, HYPRE_MEMORY_HOST);
    }
 
+   hypre_CSRMatrixMemoryLocation(C_diag) = HYPRE_MEMORY_HOST;
+   hypre_CSRMatrixMemoryLocation(C_offd) = HYPRE_MEMORY_HOST;
+
    *C_ptr = S2;
 
 #ifdef HYPRE_PROFILE
@@ -2696,8 +2742,7 @@ HYPRE_Int hypre_BoomerAMGCreate2ndS( hypre_ParCSRMatrix *S, HYPRE_Int *CF_marker
    hypre_TFree(num_coarse_prefix_sum, HYPRE_MEMORY_HOST);
 
    return 0;
-   
-}            
+}
 
 /*--------------------------------------------------------------------------
  * hypre_BoomerAMGCorrectCFMarker : corrects CF_marker after aggr. coarsening
