@@ -1,14 +1,9 @@
-/*BHEADER**********************************************************************
- * Copyright (c) 2008,  Lawrence Livermore National Security, LLC.
- * Produced at the Lawrence Livermore National Laboratory.
- * This file is part of HYPRE.  See file COPYRIGHT for details.
+/******************************************************************************
+ * Copyright 1998-2019 Lawrence Livermore National Security, LLC and other
+ * HYPRE Project Developers. See the top-level COPYRIGHT file for details.
  *
- * HYPRE is free software; you can redistribute it and/or modify it under the
- * terms of the GNU Lesser General Public License (as published by the Free
- * Software Foundation) version 2.1 dated February 1999.
- *
- * $Revision$
- ***********************************************************************EHEADER*/
+ * SPDX-License-Identifier: (Apache-2.0 OR MIT)
+ ******************************************************************************/
 
 /******************************************************************************
  *
@@ -34,7 +29,7 @@ HYPRE_Int hypre_FillResponseParToVectorAll(void*, HYPRE_Int, HYPRE_Int, void*, M
 
 hypre_ParVector *
 hypre_ParVectorCreate( MPI_Comm   comm,
-                       HYPRE_BigInt  global_size, 
+                       HYPRE_BigInt  global_size,
                        HYPRE_BigInt *partitioning )
 {
    hypre_ParVector  *vector;
@@ -66,13 +61,13 @@ hypre_ParVectorCreate( MPI_Comm   comm,
    hypre_ParVectorFirstIndex(vector) = partitioning[0];
    hypre_ParVectorLastIndex(vector) = partitioning[1]-1;
    hypre_ParVectorPartitioning(vector) = partitioning;
-   hypre_ParVectorLocalVector(vector) = 
+   hypre_ParVectorLocalVector(vector) =
       hypre_SeqVectorCreate(partitioning[1]-partitioning[0]);
 #else
    hypre_ParVectorFirstIndex(vector) = partitioning[my_id];
    hypre_ParVectorLastIndex(vector) = partitioning[my_id+1] -1;
    hypre_ParVectorPartitioning(vector) = partitioning;
-   hypre_ParVectorLocalVector(vector) = 
+   hypre_ParVectorLocalVector(vector) =
       hypre_SeqVectorCreate(partitioning[my_id+1]-partitioning[my_id]);
 #endif
 
@@ -90,7 +85,7 @@ hypre_ParVectorCreate( MPI_Comm   comm,
 
 hypre_ParVector *
 hypre_ParMultiVectorCreate( MPI_Comm   comm,
-                            HYPRE_BigInt  global_size, 
+                            HYPRE_BigInt  global_size,
                             HYPRE_BigInt *partitioning,
                             HYPRE_Int  num_vectors )
 {
@@ -105,7 +100,7 @@ hypre_ParMultiVectorCreate( MPI_Comm   comm,
  * hypre_ParVectorDestroy
  *--------------------------------------------------------------------------*/
 
-HYPRE_Int 
+HYPRE_Int
 hypre_ParVectorDestroy( hypre_ParVector *vector )
 {
    if (vector)
@@ -134,7 +129,7 @@ hypre_ParVectorDestroy( hypre_ParVector *vector )
  * hypre_ParVectorInitialize
  *--------------------------------------------------------------------------*/
 
-HYPRE_Int 
+HYPRE_Int
 hypre_ParVectorInitialize( hypre_ParVector *vector )
 {
    if (!vector)
@@ -143,8 +138,8 @@ hypre_ParVectorInitialize( hypre_ParVector *vector )
       return hypre_error_flag;
    }
    hypre_SeqVectorInitialize(hypre_ParVectorLocalVector(vector));
-   hypre_ParVectorActualLocalSize(vector) 
-	= hypre_VectorSize(hypre_ParVectorLocalVector(vector));
+   hypre_ParVectorActualLocalSize(vector)
+      = hypre_VectorSize(hypre_ParVectorLocalVector(vector));
    return hypre_error_flag;
 }
 
@@ -152,7 +147,7 @@ hypre_ParVectorInitialize( hypre_ParVector *vector )
  * hypre_ParVectorSetDataOwner
  *--------------------------------------------------------------------------*/
 
-HYPRE_Int 
+HYPRE_Int
 hypre_ParVectorSetDataOwner( hypre_ParVector *vector,
                              HYPRE_Int        owns_data )
 {
@@ -170,7 +165,7 @@ hypre_ParVectorSetDataOwner( hypre_ParVector *vector,
  * hypre_ParVectorSetPartitioningOwner
  *--------------------------------------------------------------------------*/
 
-HYPRE_Int 
+HYPRE_Int
 hypre_ParVectorSetPartitioningOwner( hypre_ParVector *vector,
                                      HYPRE_Int        owns_partitioning )
 {
@@ -191,7 +186,7 @@ hypre_ParVectorSetPartitioningOwner( hypre_ParVector *vector,
  * probably this will do more harm than good, use hypre_ParMultiVectorCreate
  *--------------------------------------------------------------------------*/
 #if 0
-HYPRE_Int 
+HYPRE_Int
 hypre_ParVectorSetNumVectors( hypre_ParVector *vector,
                               HYPRE_Int        num_vectors )
 {
@@ -220,12 +215,12 @@ hypre_ParVector
    HYPRE_Int        i;
    FILE            *fp;
 
-   hypre_MPI_Comm_rank(comm,&my_id); 
-   hypre_MPI_Comm_size(comm,&num_procs); 
+   hypre_MPI_Comm_rank(comm,&my_id);
+   hypre_MPI_Comm_size(comm,&num_procs);
 
    partitioning = hypre_CTAlloc(HYPRE_BigInt, num_procs+1, HYPRE_MEMORY_HOST);
 
-   hypre_sprintf(new_file_name,"%s.INFO.%d",file_name,my_id); 
+   hypre_sprintf(new_file_name,"%s.INFO.%d",file_name,my_id);
    fp = fopen(new_file_name, "r");
    hypre_fscanf(fp, "%b\n", &global_size);
 #ifdef HYPRE_NO_GLOBAL_PARTITION
@@ -236,10 +231,10 @@ hypre_ParVector
    for (i=0; i < num_procs; i++)
       hypre_fscanf(fp, "%b\n", &partitioning[i]);
    fclose (fp);
-   partitioning[num_procs] = global_size; 
+   partitioning[num_procs] = global_size;
 #endif
    par_vector = hypre_CTAlloc(hypre_ParVector,  1, HYPRE_MEMORY_HOST);
-        
+
    hypre_ParVectorComm(par_vector) = comm;
    hypre_ParVectorGlobalSize(par_vector) = global_size;
 
@@ -256,10 +251,10 @@ hypre_ParVector
    hypre_ParVectorOwnsData(par_vector) = 1;
    hypre_ParVectorOwnsPartitioning(par_vector) = 1;
 
-   hypre_sprintf(new_file_name,"%s.%d",file_name,my_id); 
+   hypre_sprintf(new_file_name,"%s.%d",file_name,my_id);
    hypre_ParVectorLocalVector(par_vector) = hypre_SeqVectorRead(new_file_name);
 
-   /* multivector code not written yet >>> */
+   /* multivector code not written yet */
    hypre_assert( hypre_ParVectorNumVectors(par_vector) == 1 );
 
    return par_vector;
@@ -270,7 +265,7 @@ hypre_ParVector
  *--------------------------------------------------------------------------*/
 
 HYPRE_Int
-hypre_ParVectorPrint( hypre_ParVector  *vector, 
+hypre_ParVectorPrint( hypre_ParVector  *vector,
                       const char       *file_name )
 {
    char          new_file_name[80];
@@ -285,16 +280,16 @@ hypre_ParVectorPrint( hypre_ParVector  *vector,
       hypre_error_in_arg(1);
       return hypre_error_flag;
    }
-   local_vector = hypre_ParVectorLocalVector(vector); 
+   local_vector = hypre_ParVectorLocalVector(vector);
    comm = hypre_ParVectorComm(vector);
-   partitioning = hypre_ParVectorPartitioning(vector); 
-   global_size = hypre_ParVectorGlobalSize(vector); 
+   partitioning = hypre_ParVectorPartitioning(vector);
+   global_size = hypre_ParVectorGlobalSize(vector);
 
-   hypre_MPI_Comm_rank(comm,&my_id); 
-   hypre_MPI_Comm_size(comm,&num_procs); 
-   hypre_sprintf(new_file_name,"%s.%d",file_name,my_id); 
+   hypre_MPI_Comm_rank(comm,&my_id);
+   hypre_MPI_Comm_size(comm,&num_procs);
+   hypre_sprintf(new_file_name,"%s.%d",file_name,my_id);
    hypre_SeqVectorPrint(local_vector,new_file_name);
-   hypre_sprintf(new_file_name,"%s.INFO.%d",file_name,my_id); 
+   hypre_sprintf(new_file_name,"%s.INFO.%d",file_name,my_id);
    fp = fopen(new_file_name, "w");
    hypre_fprintf(fp, "%b\n", global_size);
 #ifdef HYPRE_NO_GLOBAL_PARTITION
@@ -318,7 +313,7 @@ hypre_ParVectorSetConstantValues( hypre_ParVector *v,
                                   HYPRE_Complex    value )
 {
    hypre_Vector *v_local = hypre_ParVectorLocalVector(v);
-           
+
    return hypre_SeqVectorSetConstantValues(v_local,value);
 }
 
@@ -334,10 +329,10 @@ hypre_ParVectorSetRandomValues( hypre_ParVector *v,
    hypre_Vector *v_local = hypre_ParVectorLocalVector(v);
 
    MPI_Comm     comm = hypre_ParVectorComm(v);
-   hypre_MPI_Comm_rank(comm,&my_id); 
+   hypre_MPI_Comm_rank(comm,&my_id);
 
    seed *= (my_id+1);
-           
+
    return hypre_SeqVectorSetRandomValues(v_local,seed);
 }
 
@@ -403,7 +398,7 @@ hypre_ParVectorAxpy( HYPRE_Complex    alpha,
 {
    hypre_Vector *x_local = hypre_ParVectorLocalVector(x);
    hypre_Vector *y_local = hypre_ParVectorLocalVector(y);
-           
+
    return hypre_SeqVectorAxpy( alpha, x_local, y_local);
 }
 
@@ -423,7 +418,7 @@ hypre_ParVectorMassAxpy( HYPRE_Complex   *alpha,
 
    for (i=0; i < k; i++)
       x_local[i] = hypre_ParVectorLocalVector(x[i]);
-           
+
    hypre_SeqVectorMassAxpy( alpha, x_local, y_local, k, unroll);
 
    hypre_TFree(x_local, HYPRE_MEMORY_SHARED);
@@ -442,10 +437,10 @@ hypre_ParVectorInnerProd( hypre_ParVector *x,
    MPI_Comm      comm    = hypre_ParVectorComm(x);
    hypre_Vector *x_local = hypre_ParVectorLocalVector(x);
    hypre_Vector *y_local = hypre_ParVectorLocalVector(y);
-           
+
    HYPRE_Real result = 0.0;
    HYPRE_Real local_result = hypre_SeqVectorInnerProd(x_local, y_local);
-   
+
 #ifdef HYPRE_PROFILE
    hypre_profile_times[HYPRE_TIMER_ID_ALL_REDUCE] -= hypre_MPI_Wtime();
 #endif
@@ -454,7 +449,7 @@ hypre_ParVectorInnerProd( hypre_ParVector *x,
 #ifdef HYPRE_PROFILE
    hypre_profile_times[HYPRE_TIMER_ID_ALL_REDUCE] += hypre_MPI_Wtime();
 #endif
-   
+
    return result;
 }
 
@@ -470,9 +465,9 @@ hypre_ParVectorMassInnerProd( hypre_ParVector *x,
    MPI_Comm      comm    = hypre_ParVectorComm(x);
    hypre_Vector *x_local = hypre_ParVectorLocalVector(x);
    HYPRE_Real *local_result;
-   HYPRE_Int i; 
+   HYPRE_Int i;
    hypre_Vector **y_local;
-   y_local = hypre_TAlloc(hypre_Vector *, k, HYPRE_MEMORY_SHARED);        
+   y_local = hypre_TAlloc(hypre_Vector *, k, HYPRE_MEMORY_SHARED);
 
    for (i=0; i < k; i++)
       y_local[i] = (hypre_Vector *) hypre_ParVectorLocalVector(y[i]);
@@ -480,7 +475,7 @@ hypre_ParVectorMassInnerProd( hypre_ParVector *x,
    local_result = hypre_CTAlloc(HYPRE_Real, k, HYPRE_MEMORY_SHARED);
 
    hypre_SeqVectorMassInnerProd(x_local, y_local, k, unroll, local_result);
-   
+
 #ifdef HYPRE_PROFILE
    hypre_profile_times[HYPRE_TIMER_ID_ALL_REDUCE] -= hypre_MPI_Wtime();
 #endif
@@ -490,9 +485,9 @@ hypre_ParVectorMassInnerProd( hypre_ParVector *x,
    hypre_profile_times[HYPRE_TIMER_ID_ALL_REDUCE] += hypre_MPI_Wtime();
 #endif
 
-   hypre_TFree(y_local, HYPRE_MEMORY_SHARED);   
-   hypre_TFree(local_result, HYPRE_MEMORY_SHARED);  
- 
+   hypre_TFree(y_local, HYPRE_MEMORY_SHARED);
+   hypre_TFree(local_result, HYPRE_MEMORY_SHARED);
+
    return hypre_error_flag;
 }
 
@@ -509,9 +504,9 @@ hypre_ParVectorMassDotpTwo ( hypre_ParVector *x, hypre_ParVector *y,
    hypre_Vector *x_local = hypre_ParVectorLocalVector(x);
    hypre_Vector *y_local = hypre_ParVectorLocalVector(y);
    HYPRE_Real *local_result, *result;
-   HYPRE_Int i; 
+   HYPRE_Int i;
    hypre_Vector **z_local;
-   z_local = hypre_TAlloc(hypre_Vector *, k, HYPRE_MEMORY_SHARED);        
+   z_local = hypre_TAlloc(hypre_Vector *, k, HYPRE_MEMORY_SHARED);
 
    for (i=0; i < k; i++)
       z_local[i] = (hypre_Vector *) hypre_ParVectorLocalVector(z[i]);
@@ -520,7 +515,7 @@ hypre_ParVectorMassDotpTwo ( hypre_ParVector *x, hypre_ParVector *y,
    result = hypre_CTAlloc(HYPRE_Real, 2*k, HYPRE_MEMORY_SHARED);
 
    hypre_SeqVectorMassDotpTwo(x_local, y_local, z_local, k, unroll, &local_result[0], &local_result[k]);
-   
+
 #ifdef HYPRE_PROFILE
    hypre_profile_times[HYPRE_TIMER_ID_ALL_REDUCE] -= hypre_MPI_Wtime();
 #endif
@@ -535,10 +530,10 @@ hypre_ParVectorMassDotpTwo ( hypre_ParVector *x, hypre_ParVector *y,
       result_x[i] = result[i];
       result_y[i] = result[k+i];
    }
-   hypre_TFree(z_local, HYPRE_MEMORY_SHARED);   
-   hypre_TFree(local_result, HYPRE_MEMORY_SHARED);  
-   hypre_TFree(result, HYPRE_MEMORY_SHARED);  
- 
+   hypre_TFree(z_local, HYPRE_MEMORY_SHARED);
+   hypre_TFree(local_result, HYPRE_MEMORY_SHARED);
+   hypre_TFree(result, HYPRE_MEMORY_SHARED);
+
    return hypre_error_flag;
 }
 
@@ -597,7 +592,7 @@ hypre_VectorToParVector ( MPI_Comm      comm,
    local_data = hypre_VectorData(local_vector);
    vecstride = hypre_VectorVectorStride(local_vector);
    idxstride = hypre_VectorIndexStride(local_vector);
-   /* <<< so far the only implemented multivector StorageMethod is 0 <<< */
+   /* so far the only implemented multivector StorageMethod is 0 */
    hypre_assert( idxstride==1 );
 
    if (my_id == 0)
@@ -636,12 +631,12 @@ hypre_VectorToParVector ( MPI_Comm      comm,
 
    return par_vector;
 }
-   
+
 /*--------------------------------------------------------------------------
  * hypre_ParVectorToVectorAll:
  * generates a Vector on every proc which has a piece of the data
  * from a ParVector on several procs in comm,
- * vec_starts needs to contain the partitioning across all procs in comm 
+ * vec_starts needs to contain the partitioning across all procs in comm
  *--------------------------------------------------------------------------*/
 
 hypre_Vector *
@@ -669,7 +664,7 @@ hypre_ParVectorToVectorAll( hypre_ParVector *par_v )
 #ifdef HYPRE_NO_GLOBAL_PARTITION
 
    HYPRE_Int *new_vec_starts;
-   
+
    HYPRE_Int num_contacts;
    HYPRE_Int contact_proc_list[1];
    HYPRE_Int contact_send_buf[1];
@@ -679,12 +674,12 @@ hypre_ParVectorToVectorAll( hypre_ParVector *par_v )
    HYPRE_Int *response_recv_buf_starts = NULL;
    hypre_DataExchangeResponse response_obj;
    hypre_ProcListElements send_proc_obj;
-   
+
    HYPRE_Int *send_info = NULL;
    hypre_MPI_Status  status1;
    HYPRE_Int count, tag1 = 112, tag2 = 223;
    HYPRE_Int start;
-   
+
 #endif
 
    hypre_MPI_Comm_size(comm, &num_procs);
@@ -692,7 +687,7 @@ hypre_ParVectorToVectorAll( hypre_ParVector *par_v )
 
 #ifdef HYPRE_NO_GLOBAL_PARTITION
 
-   local_size = (HYPRE_Int)(hypre_ParVectorLastIndex(par_v) - 
+   local_size = (HYPRE_Int)(hypre_ParVectorLastIndex(par_v) -
       hypre_ParVectorFirstIndex(par_v) + 1);
 
    /* determine procs which hold data of par_v and store ids in used_procs */
@@ -720,7 +715,7 @@ hypre_ParVectorToVectorAll( hypre_ParVector *par_v )
    send_proc_obj.storage_length = 10;
    send_proc_obj.id = hypre_CTAlloc(HYPRE_Int,  send_proc_obj.storage_length, HYPRE_MEMORY_HOST);
    send_proc_obj.vec_starts =
-      hypre_CTAlloc(HYPRE_Int,  send_proc_obj.storage_length + 1, HYPRE_MEMORY_HOST); 
+      hypre_CTAlloc(HYPRE_Int,  send_proc_obj.storage_length + 1, HYPRE_MEMORY_HOST);
    send_proc_obj.vec_starts[0] = 0;
    send_proc_obj.element_storage_length = 10;
    send_proc_obj.elements =
@@ -730,15 +725,15 @@ hypre_ParVectorToVectorAll( hypre_ParVector *par_v )
    response_obj.fill_response = hypre_FillResponseParToVectorAll;
    response_obj.data1 = NULL;
    response_obj.data2 = &send_proc_obj; /*this is where we keep info from contacts*/
-  
-   
-   hypre_DataExchangeList(num_contacts, 
-                          contact_proc_list, contact_send_buf, 
-                          contact_send_buf_starts, sizeof(HYPRE_Int), 
-                          //0, &response_obj, 
-                          sizeof(HYPRE_Int), &response_obj, 
+
+
+   hypre_DataExchangeList(num_contacts,
+                          contact_proc_list, contact_send_buf,
+                          contact_send_buf_starts, sizeof(HYPRE_Int),
+                          //0, &response_obj,
+                          sizeof(HYPRE_Int), &response_obj,
                           max_response_size, 1,
-                          comm, (void**) &response_recv_buf,       
+                          comm, (void**) &response_recv_buf,
                           &response_recv_buf_starts);
 
    /* now processor 0 should have a list of ranges for processors that have rows -
@@ -746,18 +741,18 @@ hypre_ParVectorToVectorAll( hypre_ParVector *par_v )
       and also an array of vec starts - and send to those who own row*/
    if (my_id)
    {
-      if (local_size)      
+      if (local_size)
       {
-         /* look for a message from processor 0 */         
+         /* look for a message from processor 0 */
          hypre_MPI_Probe(0, tag1, comm, &status1);
          hypre_MPI_Get_count(&status1, HYPRE_MPI_INT, &count);
-         
+
          send_info = hypre_CTAlloc(HYPRE_Int,  count, HYPRE_MEMORY_HOST);
          hypre_MPI_Recv(send_info, count, HYPRE_MPI_INT, 0, tag1, comm, &status1);
 
-         /* now unpack */  
+         /* now unpack */
          num_types = send_info[0];
-         used_procs =  hypre_CTAlloc(HYPRE_Int,  num_types, HYPRE_MEMORY_HOST);  
+         used_procs =  hypre_CTAlloc(HYPRE_Int,  num_types, HYPRE_MEMORY_HOST);
          new_vec_starts = hypre_CTAlloc(HYPRE_Int,  num_types+1, HYPRE_MEMORY_HOST);
 
          for (i=1; i<= num_types; i++)
@@ -782,9 +777,9 @@ hypre_ParVectorToVectorAll( hypre_ParVector *par_v )
    else /* my_id ==0 */
    {
       num_types = send_proc_obj.length;
-      used_procs =  hypre_CTAlloc(HYPRE_Int,  num_types, HYPRE_MEMORY_HOST);  
+      used_procs =  hypre_CTAlloc(HYPRE_Int,  num_types, HYPRE_MEMORY_HOST);
       new_vec_starts = hypre_CTAlloc(HYPRE_Int,  num_types+1, HYPRE_MEMORY_HOST);
-      
+
       new_vec_starts[0] = 0;
       for (i=0; i< num_types; i++)
       {
@@ -814,8 +809,8 @@ hypre_ParVectorToVectorAll( hypre_ParVector *par_v )
       {
          start = 1;
       }
-   
-      
+
+
       for (i=start; i < num_types; i++)
       {
          hypre_MPI_Isend(send_info, count, HYPRE_MPI_INT, used_procs[i],
@@ -841,7 +836,7 @@ hypre_ParVectorToVectorAll( hypre_ParVector *par_v )
       hypre_TFree(new_vec_starts, HYPRE_MEMORY_HOST);
       return NULL;
    }
-   
+
    /* everyone left has rows and knows: new_vec_starts, num_types, and used_procs */
 
    /* this vector should be rather small */
@@ -857,9 +852,9 @@ hypre_ParVectorToVectorAll( hypre_ParVector *par_v )
    requests = hypre_CTAlloc(hypre_MPI_Request,  num_requests, HYPRE_MEMORY_HOST);
    status = hypre_CTAlloc(hypre_MPI_Status,  num_requests, HYPRE_MEMORY_HOST);
 
-   /* initialize data exchange among used_procs and generate vector  - here we 
+   /* initialize data exchange among used_procs and generate vector  - here we
       send to ourself also*/
- 
+
    j = 0;
    for (i = 0; i < num_types; i++)
    {
@@ -873,18 +868,18 @@ hypre_ParVectorToVectorAll( hypre_ParVector *par_v )
       hypre_MPI_Isend(local_data, num_vectors*local_size, HYPRE_MPI_COMPLEX,
                       used_procs[i], tag2, comm, &requests[j++]);
    }
- 
+
    hypre_MPI_Waitall(num_requests, requests, status);
 
    if (num_requests)
    {
       hypre_TFree(requests, HYPRE_MEMORY_HOST);
-      hypre_TFree(status, HYPRE_MEMORY_HOST); 
+      hypre_TFree(status, HYPRE_MEMORY_HOST);
       hypre_TFree(used_procs, HYPRE_MEMORY_HOST);
    }
 
    hypre_TFree(new_vec_starts, HYPRE_MEMORY_HOST);
-   
+
 #else
 
    local_size = (HYPRE_Int)(vec_starts[my_id+1] - vec_starts[my_id]);
@@ -893,7 +888,7 @@ hypre_ParVectorToVectorAll( hypre_ParVector *par_v )
 
    if (!local_size)
       return NULL;
- 
+
    local_data = hypre_VectorData(local_vector);
    vector = hypre_SeqVectorCreate(global_size);
    hypre_VectorNumVectors(vector) = num_vectors;
@@ -907,18 +902,18 @@ hypre_ParVectorToVectorAll( hypre_ParVector *par_v )
       if (vec_starts[i+1]-vec_starts[i])
          num_types++;
    num_requests = 2*num_types;
- 
+
    used_procs = hypre_CTAlloc(HYPRE_Int,  num_types, HYPRE_MEMORY_HOST);
    j = 0;
    for (i=0; i < num_procs; i++)
       if (vec_starts[i+1]-vec_starts[i] && i-my_id)
          used_procs[j++] = i;
- 
+
    requests = hypre_CTAlloc(hypre_MPI_Request,  num_requests, HYPRE_MEMORY_HOST);
    status = hypre_CTAlloc(hypre_MPI_Status,  num_requests, HYPRE_MEMORY_HOST);
 
    /* initialize data exchange among used_procs and generate vector */
- 
+
    j = 0;
    for (i = 0; i < num_types; i++)
    {
@@ -932,17 +927,17 @@ hypre_ParVectorToVectorAll( hypre_ParVector *par_v )
       hypre_MPI_Isend(local_data, num_vectors*local_size, HYPRE_MPI_COMPLEX,
                       used_procs[i], 0, comm, &requests[j++]);
    }
- 
+
    for (i=0; i < num_vectors*local_size; i++)
       vector_data[vec_starts[my_id]+i] = local_data[i];
- 
+
    hypre_MPI_Waitall(num_requests, requests, status);
 
    if (num_requests)
    {
       hypre_TFree(used_procs, HYPRE_MEMORY_HOST);
       hypre_TFree(requests, HYPRE_MEMORY_HOST);
-      hypre_TFree(status, HYPRE_MEMORY_HOST); 
+      hypre_TFree(status, HYPRE_MEMORY_HOST);
    }
 
 #endif
@@ -975,13 +970,13 @@ hypre_ParVectorPrintIJ( hypre_ParVector *vector,
    global_size  = hypre_ParVectorGlobalSize(vector);
    partitioning = hypre_ParVectorPartitioning(vector);
 
-   /* multivector code not written yet >>> */
+   /* multivector code not written yet */
    hypre_assert( hypre_ParVectorNumVectors(vector) == 1 );
    if ( hypre_ParVectorNumVectors(vector) != 1 ) hypre_error_in_arg(1);
 
    hypre_MPI_Comm_rank(comm, &myid);
    hypre_MPI_Comm_size(comm, &num_procs);
-  
+
    hypre_sprintf(new_filename,"%s.%05d", filename, myid);
 
    if ((file = fopen(new_filename, "w")) == NULL)
@@ -1049,7 +1044,7 @@ hypre_ParVectorReadIJ( MPI_Comm          comm,
 
    hypre_MPI_Comm_size(comm, &num_procs);
    hypre_MPI_Comm_rank(comm, &myid);
-  
+
    hypre_sprintf(new_filename,"%s.%05d", filename, myid);
 
    if ((file = fopen(new_filename, "r")) == NULL)
@@ -1107,7 +1102,7 @@ hypre_ParVectorReadIJ( MPI_Comm          comm,
    *base_j_ptr = base_j;
    *vector_ptr = vector;
 
-   /* multivector code not written yet >>> */
+   /* multivector code not written yet */
    hypre_assert( hypre_ParVectorNumVectors(vector) == 1 );
    if ( hypre_ParVectorNumVectors(vector) != 1 ) hypre_error(HYPRE_ERROR_GENERIC);
 
@@ -1121,20 +1116,20 @@ hypre_ParVectorReadIJ( MPI_Comm          comm,
  *--------------------------------------------------------------------*/
 
 HYPRE_Int
-hypre_FillResponseParToVectorAll( void       *p_recv_contact_buf, 
+hypre_FillResponseParToVectorAll( void       *p_recv_contact_buf,
                                   HYPRE_Int   contact_size,
                                   HYPRE_Int   contact_proc,
-                                  void       *ro, 
+                                  void       *ro,
                                   MPI_Comm    comm,
-                                  void      **p_send_response_buf, 
+                                  void      **p_send_response_buf,
                                   HYPRE_Int  *response_message_size )
 {
    HYPRE_Int     myid;
    HYPRE_Int     i, index, count, elength;
 
    HYPRE_BigInt    *recv_contact_buf = (HYPRE_BigInt * ) p_recv_contact_buf;
- 
-   hypre_DataExchangeResponse  *response_obj = (hypre_DataExchangeResponse*)ro;  
+
+   hypre_DataExchangeResponse  *response_obj = (hypre_DataExchangeResponse*)ro;
 
    hypre_ProcListElements      *send_proc_obj = (hypre_ProcListElements*)response_obj->data2;
    hypre_MPI_Comm_rank(comm, &myid );
@@ -1143,40 +1138,40 @@ hypre_FillResponseParToVectorAll( void       *p_recv_contact_buf,
    if (send_proc_obj->length == send_proc_obj->storage_length)
    {
       send_proc_obj->storage_length +=10; /*add space for 10 more processors*/
-      send_proc_obj->id = hypre_TReAlloc(send_proc_obj->id, HYPRE_Int,  
+      send_proc_obj->id = hypre_TReAlloc(send_proc_obj->id, HYPRE_Int,
                                          send_proc_obj->storage_length, HYPRE_MEMORY_HOST);
       send_proc_obj->vec_starts =
-         hypre_TReAlloc(send_proc_obj->vec_starts, HYPRE_Int, 
+         hypre_TReAlloc(send_proc_obj->vec_starts, HYPRE_Int,
                         send_proc_obj->storage_length + 1, HYPRE_MEMORY_HOST);
    }
-  
-   /*initialize*/ 
+
+   /*initialize*/
    count = send_proc_obj->length;
    index = send_proc_obj->vec_starts[count]; /*this is the number of elements*/
 
-   /*send proc*/ 
-   send_proc_obj->id[count] = contact_proc; 
+   /*send proc*/
+   send_proc_obj->id[count] = contact_proc;
 
    /*do we need more storage for the elements?*/
    if (send_proc_obj->element_storage_length < index + contact_size)
    {
-      elength = hypre_max(contact_size, 10);   
+      elength = hypre_max(contact_size, 10);
       elength += index;
-      send_proc_obj->elements = hypre_TReAlloc(send_proc_obj->elements,  
+      send_proc_obj->elements = hypre_TReAlloc(send_proc_obj->elements,
                                                HYPRE_BigInt,  elength, HYPRE_MEMORY_HOST);
-      send_proc_obj->element_storage_length = elength; 
+      send_proc_obj->element_storage_length = elength;
    }
    /*populate send_proc_obj*/
-   for (i=0; i< contact_size; i++) 
-   { 
+   for (i=0; i< contact_size; i++)
+   {
       send_proc_obj->elements[index++] = recv_contact_buf[i];
    }
    send_proc_obj->vec_starts[count+1] = index;
    send_proc_obj->length++;
 
    /*output - no message to return (confirmation) */
-   *response_message_size = 0; 
-  
+   *response_message_size = 0;
+
    return hypre_error_flag;
 }
 
@@ -1186,20 +1181,17 @@ hypre_FillResponseParToVectorAll( void       *p_recv_contact_buf,
 
 HYPRE_Complex hypre_ParVectorLocalSumElts( hypre_ParVector * vector )
 {
-   return hypre_VectorSumElts( hypre_ParVectorLocalVector(vector) );
+   return hypre_SeqVectorSumElts( hypre_ParVectorLocalVector(vector) );
 }
+
+/*
 #ifdef HYPRE_USING_UNIFIED_MEMORY
 hypre_int hypre_ParVectorIsManaged(hypre_ParVector *vector){
   if (vector==NULL) return 1;
   return hypre_SeqVectorIsManaged(hypre_ParVectorLocalVector(vector));
 }
 #endif
-#ifdef HYPRE_USING_MAPPED_OPENMP_OFFLOAD
-void hypre_ParVectorUpdateHost(hypre_ParVector *p){
-#pragma omp target update from(p->local_vector->data[0:p->local_vector->size])
-  SetHRC(p->local_vector);
-}
-#endif
+*/
 
 HYPRE_Int
 hypre_ParVectorGetValues(hypre_ParVector *vector,
