@@ -1,17 +1,9 @@
-/*BHEADER**********************************************************************
- * Copyright (c) 2008,  Lawrence Livermore National Security, LLC.
- * Produced at the Lawrence Livermore National Laboratory.
- * This file is part of HYPRE.  See file COPYRIGHT for details.
+/******************************************************************************
+ * Copyright 1998-2019 Lawrence Livermore National Security, LLC and other
+ * HYPRE Project Developers. See the top-level COPYRIGHT file for details.
  *
- * HYPRE is free software; you can redistribute it and/or modify it under the
- * terms of the GNU Lesser General Public License (as published by the Free
- * Software Foundation) version 2.1 dated February 1999.
- *
- * $Revision$
- ***********************************************************************EHEADER*/
-
-
-
+ * SPDX-License-Identifier: (Apache-2.0 OR MIT)
+ ******************************************************************************/
 
 /******************************************************************************
  *
@@ -86,6 +78,12 @@ HYPRE_Int HYPRE_VectorInitialize( HYPRE_Vector vector );
 HYPRE_Int HYPRE_VectorPrint( HYPRE_Vector vector , char *file_name );
 HYPRE_Vector HYPRE_VectorRead( char *file_name );
 
+HYPRE_Int HYPRE_CSRMatrixDeviceSpGemmSetRownnzEstimateMethod( HYPRE_Int value );
+HYPRE_Int HYPRE_CSRMatrixDeviceSpGemmSetRownnzEstimateNSamples( HYPRE_Int value );
+HYPRE_Int HYPRE_CSRMatrixDeviceSpGemmSetRownnzEstimateMultFactor( HYPRE_Real value );
+HYPRE_Int HYPRE_CSRMatrixDeviceSpGemmSetHashType( char value );
+HYPRE_Int HYPRE_CSRMatrixDeviceSpGemmSetUseCusparse( HYPRE_Int value );
+
 typedef enum HYPRE_TimerID
 {
    // timers for solver phase
@@ -129,6 +127,16 @@ typedef enum HYPRE_TimerID
    HYPRE_TIMER_ID_RENUMBER_COLIDX_RAP,
    HYPRE_TIMER_ID_MERGE,
 
+   // csr matop
+   HYPRE_TIMER_ID_SPMM_ROWNNZ,
+   HYPRE_TIMER_ID_SPMM_ATTEMPT1,
+   HYPRE_TIMER_ID_SPMM_ATTEMPT2,
+   HYPRE_TIMER_ID_SPMM_SYMBOLIC,
+   HYPRE_TIMER_ID_SPMM_NUMERIC,
+   HYPRE_TIMER_ID_SPMM,
+   HYPRE_TIMER_ID_SPADD,
+   HYPRE_TIMER_ID_SPTRANS,
+
    HYPRE_TIMER_ID_COUNT
 } HYPRE_TimerID;
 
