@@ -1,14 +1,9 @@
-/*BHEADER**********************************************************************
- * Copyright (c) 2008,  Lawrence Livermore National Security, LLC.
- * Produced at the Lawrence Livermore National Laboratory.
- * This file is part of HYPRE.  See file COPYRIGHT for details.
+/******************************************************************************
+ * Copyright 1998-2019 Lawrence Livermore National Security, LLC and other
+ * HYPRE Project Developers. See the top-level COPYRIGHT file for details.
  *
- * HYPRE is free software; you can redistribute it and/or modify it under the
- * terms of the GNU Lesser General Public License (as published by the Free
- * Software Foundation) version 2.1 dated February 1999.
- *
- * $Revision$
- ***********************************************************************EHEADER*/
+ * SPDX-License-Identifier: (Apache-2.0 OR MIT)
+ ******************************************************************************/
 
 /******************************************************************************
  *
@@ -38,18 +33,14 @@ hypre_CSRMatrixMatvecOutOfPlace( HYPRE_Complex    alpha,
    HYPRE_Real time_begin = hypre_MPI_Wtime();
 #endif
 
-#if defined(HYPRE_USING_GPU) && defined(HYPRE_USING_UNIFIED_MEMORY) /* CUDA */
-   PUSH_RANGE_PAYLOAD("MATVEC",0, hypre_CSRMatrixNumRows(A));
+#if defined(HYPRE_USING_CUDA) /* CUDA */
 #ifdef HYPRE_BIGINT
-   HYPRE_Int ierr = hypre_CSRMatrixMatvecDeviceBIGINT( alpha,A,x,beta,b,y,offset );
+   HYPRE_Int ierr = hypre_CSRMatrixMatvecDeviceBIGINT(alpha, A, x, beta, b, y, offset);
 #else
-   HYPRE_Int ierr = hypre_CSRMatrixMatvecDevice( alpha,A,x,beta,b,y,offset );
+   HYPRE_Int ierr = hypre_CSRMatrixMatvecDevice(0, alpha, A, x, beta, b, y, offset);
 #endif
-   POP_RANGE;
-#elif defined(HYPRE_USING_OPENMP_OFFLOAD) /* OMP 4.5 */
-   PUSH_RANGE_PAYLOAD("MATVEC-OMP",0, hypre_CSRMatrixNumRows(A));
-   HYPRE_Int ierr = hypre_CSRMatrixMatvecOutOfPlaceOOMP( alpha,A,x,beta,b,y,offset );
-   POP_RANGE;
+#elif defined(HYPRE_USING_DEVICE_OPENMP) /* OMP 4.5 */
+   HYPRE_Int ierr = hypre_CSRMatrixMatvecOutOfPlaceOOMP(0, alpha, A, x, beta, b, y, offset);
 #else /* CPU */
    HYPRE_Complex    *A_data   = hypre_CSRMatrixData(A);
    HYPRE_Int        *A_i      = hypre_CSRMatrixI(A) + offset;
@@ -133,7 +124,7 @@ hypre_CSRMatrixMatvecOutOfPlace( HYPRE_Complex    alpha,
 
    temp = beta / alpha;
 
-/* use rownnz pointer to do the A*x multiplication  when num_rownnz is smaller than num_rows */
+   /* use rownnz pointer to do the A*x multiplication  when num_rownnz is smaller than num_rows */
 
    if (num_rownnz < xpar*(num_rows) || num_vectors > 1)
    {
@@ -162,8 +153,8 @@ hypre_CSRMatrixMatvecOutOfPlace( HYPRE_Complex    alpha,
       }
       else
       {
-            for (i = 0; i < num_rows*num_vectors; i++)
-               y_data[i] = b_data[i];
+         for (i = 0; i < num_rows*num_vectors; i++)
+            y_data[i] = b_data[i];
       }
 
 
@@ -242,172 +233,175 @@ hypre_CSRMatrixMatvecOutOfPlace( HYPRE_Complex    alpha,
 #pragma omp parallel private(i,jj,tempx)
 #endif
       {
-      HYPRE_Int iBegin = hypre_CSRMatrixGetLoadBalancedPartitionBegin(A);
-      HYPRE_Int iEnd = hypre_CSRMatrixGetLoadBalancedPartitionEnd(A);
-      hypre_assert(iBegin <= iEnd);
-      hypre_assert(iBegin >= 0 && iBegin <= num_rows);
-      hypre_assert(iEnd >= 0 && iEnd <= num_rows);
+         HYPRE_Int iBegin = hypre_CSRMatrixGetLoadBalancedPartitionBegin(A);
+         HYPRE_Int iEnd = hypre_CSRMatrixGetLoadBalancedPartitionEnd(A);
+         hypre_assert(iBegin <= iEnd);
+         hypre_assert(iBegin >= 0 && iBegin <= num_rows);
+         hypre_assert(iEnd >= 0 && iEnd <= num_rows);
 
-      if (0 == temp)
-      {
-         if (1 == alpha) // JSP: a common path
+         if (0 == temp)
          {
-            for (i = iBegin; i < iEnd; i++)
+            if (1 == alpha) // JSP: a common path
             {
-               tempx = 0.0;
-               for (jj = A_i[i]; jj < A_i[i+1]; jj++)
+               for (i = iBegin; i < iEnd; i++)
                {
-                  tempx += A_data[jj] * x_data[A_j[jj]];
+                  tempx = 0.0;
+                  for (jj = A_i[i]; jj < A_i[i+1]; jj++)
+                  {
+                     tempx += A_data[jj] * x_data[A_j[jj]];
+                  }
+                  y_data[i] = tempx;
                }
-               y_data[i] = tempx;
-            }
-         } // y = A*x
-         else if (-1 == alpha)
+            } // y = A*x
+            else if (-1 == alpha)
+            {
+               for (i = iBegin; i < iEnd; i++)
+               {
+                  tempx = 0.0;
+                  for (jj = A_i[i]; jj < A_i[i+1]; jj++)
+                  {
+                     tempx -= A_data[jj] * x_data[A_j[jj]];
+                  }
+                  y_data[i] = tempx;
+               }
+            } // y = -A*x
+            else
+            {
+               for (i = iBegin; i < iEnd; i++)
+               {
+                  tempx = 0.0;
+                  for (jj = A_i[i]; jj < A_i[i+1]; jj++)
+                  {
+                     tempx += A_data[jj] * x_data[A_j[jj]];
+                  }
+                  y_data[i] = alpha*tempx;
+               }
+            } // y = alpha*A*x
+         } // temp == 0
+         else if (-1 == temp) // beta == -alpha
          {
-            for (i = iBegin; i < iEnd; i++)
+            if (1 == alpha) // JSP: a common path
             {
-               tempx = 0.0;
-               for (jj = A_i[i]; jj < A_i[i+1]; jj++)
+               for (i = iBegin; i < iEnd; i++)
                {
-                  tempx -= A_data[jj] * x_data[A_j[jj]];
+                  tempx = -b_data[i];
+                  for (jj = A_i[i]; jj < A_i[i+1]; jj++)
+                  {
+                     tempx += A_data[jj] * x_data[A_j[jj]];
+                  }
+                  y_data[i] = tempx;
                }
-               y_data[i] = tempx;
-            }
-         } // y = -A*x
+            } // y = A*x - y
+            else if (-1 == alpha) // JSP: a common path
+            {
+               for (i = iBegin; i < iEnd; i++)
+               {
+                  tempx = b_data[i];
+                  for (jj = A_i[i]; jj < A_i[i+1]; jj++)
+                  {
+                     tempx -= A_data[jj] * x_data[A_j[jj]];
+                  }
+                  y_data[i] = tempx;
+               }
+            } // y = -A*x + y
+            else
+            {
+               for (i = iBegin; i < iEnd; i++)
+               {
+                  tempx = -b_data[i];
+                  for (jj = A_i[i]; jj < A_i[i+1]; jj++)
+                  {
+                     tempx += A_data[jj] * x_data[A_j[jj]];
+                  }
+                  y_data[i] = alpha*tempx;
+               }
+            } // y = alpha*(A*x - y)
+         } // temp == -1
+         else if (1 == temp)
+         {
+            if (1 == alpha) // JSP: a common path
+            {
+               for (i = iBegin; i < iEnd; i++)
+               {
+                  tempx = b_data[i];
+                  for (jj = A_i[i]; jj < A_i[i+1]; jj++)
+                  {
+                     tempx += A_data[jj] * x_data[A_j[jj]];
+                  }
+                  y_data[i] = tempx;
+               }
+            } // y = A*x + y
+            else if (-1 == alpha)
+            {
+               for (i = iBegin; i < iEnd; i++)
+               {
+                  tempx = -b_data[i];
+                  for (jj = A_i[i]; jj < A_i[i+1]; jj++)
+                  {
+                     tempx -= A_data[jj] * x_data[A_j[jj]];
+                  }
+                  y_data[i] = tempx;
+               }
+            } // y = -A*x - y
+            else
+            {
+               for (i = iBegin; i < iEnd; i++)
+               {
+                  tempx = b_data[i];
+                  for (jj = A_i[i]; jj < A_i[i+1]; jj++)
+                  {
+                     tempx += A_data[jj] * x_data[A_j[jj]];
+                  }
+                  y_data[i] = alpha*tempx;
+               }
+            } // y = alpha*(A*x + y)
+         }
          else
          {
-            for (i = iBegin; i < iEnd; i++)
+            if (1 == alpha) // JSP: a common path
             {
-               tempx = 0.0;
-               for (jj = A_i[i]; jj < A_i[i+1]; jj++)
+               for (i = iBegin; i < iEnd; i++)
                {
-                  tempx += A_data[jj] * x_data[A_j[jj]];
+                  tempx = b_data[i]*temp;
+                  for (jj = A_i[i]; jj < A_i[i+1]; jj++)
+                  {
+                     tempx += A_data[jj] * x_data[A_j[jj]];
+                  }
+                  y_data[i] = tempx;
                }
-               y_data[i] = alpha*tempx;
-            }
-         } // y = alpha*A*x
-      } // temp == 0
-      else if (-1 == temp) // beta == -alpha
-      {
-         if (1 == alpha) // JSP: a common path
-         {
-            for (i = iBegin; i < iEnd; i++)
+            } // y = A*x + temp*y
+            else if (-1 == alpha)
             {
-               tempx = -b_data[i];
-               for (jj = A_i[i]; jj < A_i[i+1]; jj++)
+               for (i = iBegin; i < iEnd; i++)
                {
-                  tempx += A_data[jj] * x_data[A_j[jj]];
+                  tempx = -b_data[i]*temp;
+                  for (jj = A_i[i]; jj < A_i[i+1]; jj++)
+                  {
+                     tempx -= A_data[jj] * x_data[A_j[jj]];
+                  }
+                  y_data[i] = tempx;
                }
-               y_data[i] = tempx;
-            }
-         } // y = A*x - y
-         else if (-1 == alpha) // JSP: a common path
-         {
-            for (i = iBegin; i < iEnd; i++)
+            } // y = -A*x - temp*y
+            else
             {
-               tempx = b_data[i];
-               for (jj = A_i[i]; jj < A_i[i+1]; jj++)
+               for (i = iBegin; i < iEnd; i++)
                {
-                  tempx -= A_data[jj] * x_data[A_j[jj]];
+                  tempx = b_data[i]*temp;
+                  for (jj = A_i[i]; jj < A_i[i+1]; jj++)
+                  {
+                     tempx += A_data[jj] * x_data[A_j[jj]];
+                  }
+                  y_data[i] = alpha*tempx;
                }
-               y_data[i] = tempx;
-            }
-         } // y = -A*x + y
-         else
-         {
-            for (i = iBegin; i < iEnd; i++)
-            {
-               tempx = -b_data[i];
-               for (jj = A_i[i]; jj < A_i[i+1]; jj++)
-               {
-                  tempx += A_data[jj] * x_data[A_j[jj]];
-               }
-               y_data[i] = alpha*tempx;
-            }
-         } // y = alpha*(A*x - y)
-      } // temp == -1
-      else if (1 == temp)
-      {
-         if (1 == alpha) // JSP: a common path
-         {
-            for (i = iBegin; i < iEnd; i++)
-            {
-               tempx = b_data[i];
-               for (jj = A_i[i]; jj < A_i[i+1]; jj++)
-               {
-                  tempx += A_data[jj] * x_data[A_j[jj]];
-               }
-               y_data[i] = tempx;
-            }
-         } // y = A*x + y
-         else if (-1 == alpha)
-         {
-            for (i = iBegin; i < iEnd; i++)
-            {
-               tempx = -b_data[i];
-               for (jj = A_i[i]; jj < A_i[i+1]; jj++)
-               {
-                  tempx -= A_data[jj] * x_data[A_j[jj]];
-               }
-               y_data[i] = tempx;
-            }
-         } // y = -A*x - y
-         else
-         {
-            for (i = iBegin; i < iEnd; i++)
-            {
-               tempx = b_data[i];
-               for (jj = A_i[i]; jj < A_i[i+1]; jj++)
-               {
-                  tempx += A_data[jj] * x_data[A_j[jj]];
-               }
-               y_data[i] = alpha*tempx;
-            }
-         } // y = alpha*(A*x + y)
-      }
-      else
-      {
-         if (1 == alpha) // JSP: a common path
-         {
-            for (i = iBegin; i < iEnd; i++)
-            {
-               tempx = b_data[i]*temp;
-               for (jj = A_i[i]; jj < A_i[i+1]; jj++)
-               {
-                  tempx += A_data[jj] * x_data[A_j[jj]];
-               }
-               y_data[i] = tempx;
-            }
-         } // y = A*x + temp*y
-         else if (-1 == alpha)
-         {
-            for (i = iBegin; i < iEnd; i++)
-            {
-               tempx = -b_data[i]*temp;
-               for (jj = A_i[i]; jj < A_i[i+1]; jj++)
-               {
-                  tempx -= A_data[jj] * x_data[A_j[jj]];
-               }
-               y_data[i] = tempx;
-            }
-         } // y = -A*x - temp*y
-         else
-         {
-            for (i = iBegin; i < iEnd; i++)
-            {
-               tempx = b_data[i]*temp;
-               for (jj = A_i[i]; jj < A_i[i+1]; jj++)
-               {
-                  tempx += A_data[jj] * x_data[A_j[jj]];
-               }
-               y_data[i] = alpha*tempx;
-            }
-         } // y = alpha*(A*x + temp*y)
-      } // temp != 0 && temp != -1 && temp != 1
+            } // y = alpha*(A*x + temp*y)
+         } // temp != 0 && temp != -1 && temp != 1
       } // omp parallel
    }
 
-   if (x == y) hypre_SeqVectorDestroy(x_tmp);
+   if (x == y)
+   {
+      hypre_SeqVectorDestroy(x_tmp);
+   }
 
 #endif /* CPU */
 
@@ -428,18 +422,6 @@ hypre_CSRMatrixMatvec( HYPRE_Complex    alpha,
    return hypre_CSRMatrixMatvecOutOfPlace(alpha, A, x, beta, y, y, 0);
 }
 
-#if defined (HYPRE_USING_UNIFIED_MEMORY)
-HYPRE_Int
-hypre_CSRMatrixMatvec3( HYPRE_Complex    alpha,
-                       hypre_CSRMatrix *A,
-                       hypre_Vector    *x,
-                       HYPRE_Complex    beta,
-                       hypre_Vector    *y     )
-{
-   return hypre_CSRMatrixMatvecOutOfPlaceOOMP3(alpha, A, x, beta, y, y, 0);
-}
-#endif
-
 /*--------------------------------------------------------------------------
  * hypre_CSRMatrixMatvecT
  *
@@ -457,6 +439,11 @@ hypre_CSRMatrixMatvecT( HYPRE_Complex    alpha,
                         HYPRE_Complex    beta,
                         hypre_Vector    *y     )
 {
+#if defined(HYPRE_USING_CUDA) /* CUDA */
+   HYPRE_Int ierr = hypre_CSRMatrixMatvecDevice(1, alpha, A, x, beta, y, y, 0 );
+#elif defined(HYPRE_USING_DEVICE_OPENMP) /* OMP 4.5 */
+   HYPRE_Int ierr = hypre_CSRMatrixMatvecOutOfPlaceOOMP(1, alpha, A, x, beta, y, y, 0);
+#else /* CPU */
    HYPRE_Complex    *A_data    = hypre_CSRMatrixData(A);
    HYPRE_Int        *A_i       = hypre_CSRMatrixI(A);
    HYPRE_Int        *A_j       = hypre_CSRMatrixJ(A);
@@ -653,18 +640,21 @@ hypre_CSRMatrixMatvecT( HYPRE_Complex    alpha,
 #pragma omp parallel for private(i) HYPRE_SMP_SCHEDULE
 #endif
       for (i = 0; i < num_cols*num_vectors; i++)
+      {
          y_data[i] *= alpha;
+      }
    }
 
    if (x == y) hypre_SeqVectorDestroy(x_tmp);
 
+#endif
+
    return ierr;
 }
 
 /*--------------------------------------------------------------------------
  * hypre_CSRMatrixMatvec_FF
  *--------------------------------------------------------------------------*/
-
 HYPRE_Int
 hypre_CSRMatrixMatvec_FF( HYPRE_Complex    alpha,
                           hypre_CSRMatrix *A,
@@ -787,9 +777,11 @@ hypre_CSRMatrixMatvec_FF( HYPRE_Complex    alpha,
 
    return ierr;
 }
-#if defined(HYPRE_USING_GPU) && defined(HYPRE_USING_UNIFIED_MEMORY)
+
+#if defined(HYPRE_USING_CUDA)
 HYPRE_Int
-hypre_CSRMatrixMatvecDevice( HYPRE_Complex    alpha,
+hypre_CSRMatrixMatvecDevice( HYPRE_Int        trans,
+                             HYPRE_Complex    alpha,
                              hypre_CSRMatrix *A,
                              hypre_Vector    *x,
                              HYPRE_Complex    beta,
@@ -800,67 +792,70 @@ hypre_CSRMatrixMatvecDevice( HYPRE_Complex    alpha,
 #ifdef HYPRE_BIGINT
    hypre_error_w_msg(HYPRE_ERROR_GENERIC,"ERROR: hypre_CSRMatvecDevice should not be called when bigint is enabled!");
 #else
-  static cusparseHandle_t handle;
-  static cusparseMatDescr_t descr;
-  static HYPRE_Int FirstCall=1;
-  cusparseStatus_t status;
-  static cudaStream_t s[10];
-  static HYPRE_Int myid;
 
-  if (b!=y){
+   cusparseHandle_t handle = hypre_HandleCusparseHandle(hypre_handle);
+   cusparseMatDescr_t descr = hypre_HandleCusparseMatDescr(hypre_handle);
 
-    PUSH_RANGE_PAYLOAD("MEMCPY",1,y->size-offset);
-    VecCopy(y->data,b->data,(y->size-offset),HYPRE_STREAM(4));
-    POP_RANGE
-  }
+   hypre_CSRMatrixPrefetch(A, HYPRE_MEMORY_DEVICE);
+   hypre_SeqVectorPrefetch(x, HYPRE_MEMORY_DEVICE);
+   hypre_SeqVectorPrefetch(b, HYPRE_MEMORY_DEVICE);
 
-  if (x==y) hypre_error_w_msg(HYPRE_ERROR_GENERIC,"ERROR::x and y are the same pointer in hypre_CSRMatrixMatvecDevice\n");
+   if (b != y)
+   {
+      hypre_SeqVectorPrefetch(y, HYPRE_MEMORY_DEVICE);
+   }
 
-  if (FirstCall){
-    PUSH_RANGE("FIRST_CALL",4);
+   if (b != y)
+   {
+      HYPRE_THRUST_CALL( copy_n, b->data, y->size-offset, y->data );
+   }
 
-    handle=getCusparseHandle();
+   if (x == y)
+   {
+      hypre_error_w_msg(HYPRE_ERROR_GENERIC,"ERROR::x and y are the same pointer in hypre_CSRMatrixMatvecDevice\n");
+   }
 
-    status= cusparseCreateMatDescr(&descr);
-    if (status != CUSPARSE_STATUS_SUCCESS) {
-      hypre_error_w_msg(HYPRE_ERROR_GENERIC,"ERROR:: Matrix descriptor initialization failed\n");
-      return hypre_error_flag;
-    }
+   // TODO
+   if (offset != 0)
+   {
+      hypre_printf("WARNING:: Offset is not zero in hypre_CSRMatrixMatvecDevice :: \n");
+   }
 
-    cusparseSetMatType(descr,CUSPARSE_MATRIX_TYPE_GENERAL);
-    cusparseSetMatIndexBase(descr,CUSPARSE_INDEX_BASE_ZERO);
+   hypre_assert(offset == 0);
 
-    FirstCall=0;
-    hypre_int jj;
-    for(jj=0;jj<5;jj++)
-      s[jj]=HYPRE_STREAM(jj);
-    nvtxNameCudaStreamA(s[4], "HYPRE_COMPUTE_STREAM");
-    hypre_MPI_Comm_rank(hypre_MPI_COMM_WORLD, &myid );
-    myid++;
-    POP_RANGE;
-  }
+   if (trans)
+   {
+      HYPRE_Complex *csc_a = hypre_TAlloc(HYPRE_Complex, A->num_nonzeros, HYPRE_MEMORY_DEVICE);
+      HYPRE_Int     *csc_j = hypre_TAlloc(HYPRE_Int,     A->num_nonzeros, HYPRE_MEMORY_DEVICE);
+      HYPRE_Int     *csc_i = hypre_TAlloc(HYPRE_Int,     A->num_cols+1,   HYPRE_MEMORY_DEVICE);
 
-  PUSH_RANGE("PREFETCH+SPMV",2);
+      HYPRE_CUSPARSE_CALL( cusparseDcsr2csc(handle, A->num_rows, A->num_cols, A->num_nonzeros,
+                           A->data, A->i, A->j, csc_a, csc_j, csc_i,
+                           CUSPARSE_ACTION_NUMERIC, CUSPARSE_INDEX_BASE_ZERO) );
 
-  hypre_CSRMatrixPrefetchToDevice(A);
-  hypre_SeqVectorPrefetchToDevice(x);
-  hypre_SeqVectorPrefetchToDevice(y);
+      HYPRE_CUSPARSE_CALL( cusparseDcsrmv(handle, CUSPARSE_OPERATION_NON_TRANSPOSE,
+                           A->num_cols, A->num_rows, A->num_nonzeros,
+                           &alpha, descr,
+                           csc_a, csc_i, csc_j,
+                           x->data, &beta, y->data) );
 
-  //if (offset!=0) hypre_printf("WARNING:: Offset is not zero in hypre_CSRMatrixMatvecDevice :: \n");
-  cusparseErrchk(cusparseDcsrmv(handle ,
-                 CUSPARSE_OPERATION_NON_TRANSPOSE,
-                 A->num_rows-offset, A->num_cols, A->num_nonzeros,
-                 &alpha, descr,
-                 A->data ,A->i+offset,A->j,
-                 x->data, &beta, y->data+offset));
+      hypre_TFree(csc_a, HYPRE_MEMORY_DEVICE);
+      hypre_TFree(csc_i, HYPRE_MEMORY_DEVICE);
+      hypre_TFree(csc_j, HYPRE_MEMORY_DEVICE);
+   }
+   else
+   {
+      HYPRE_CUSPARSE_CALL( cusparseDcsrmv(handle, CUSPARSE_OPERATION_NON_TRANSPOSE,
+                           A->num_rows-offset, A->num_cols, A->num_nonzeros,
+                           &alpha, descr,
+                           A->data, A->i+offset, A->j,
+                           x->data, &beta, y->data+offset) );
+   }
 
-  if (!GetAsyncMode()){
-  hypre_CheckErrorDevice(cudaStreamSynchronize(s[4]));
-  }
-  POP_RANGE;
+   hypre_SyncCudaComputeStream(hypre_handle);
 #endif
-  return hypre_error_flag;
 
+   return hypre_error_flag;
 }
 
 HYPRE_Int
@@ -873,88 +868,10 @@ hypre_CSRMatrixMatvecDeviceBIGINT( HYPRE_Complex    alpha,
                        HYPRE_Int offset )
 {
 #ifdef HYPRE_BIGINT
-  static cusparseHandle_t handle;
-  static cusparseMatDescr_t descr;
-  static HYPRE_Int FirstCall=1;
-  cusparseStatus_t status;
-  static cudaStream_t s[10];
-  static HYPRE_Int myid;
-
-  if (b!=y){
-
-    PUSH_RANGE_PAYLOAD("MEMCPY",1,y->size-offset);
-    VecCopy(y->data,b->data,(y->size-offset),HYPRE_STREAM(4));
-    POP_RANGE
-  }
-
-  if (x==y) fprintf(stderr,"ERROR::x and y are the same pointer in hypre_CSRMatrixMatvecDevice\n");
-
-  if (FirstCall){
-    PUSH_RANGE("FIRST_CALL",4);
-
-    handle=getCusparseHandle();
-
-    status= cusparseCreateMatDescr(&descr);
-    if (status != CUSPARSE_STATUS_SUCCESS) {
-      printf("ERROR:: Matrix descriptor initialization failed\n");
-      exit(2);
-    }
-
-    cusparseSetMatType(descr,CUSPARSE_MATRIX_TYPE_GENERAL);
-    cusparseSetMatIndexBase(descr,CUSPARSE_INDEX_BASE_ZERO);
-
-    FirstCall=0;
-    hypre_int jj;
-    for(jj=0;jj<5;jj++)
-      s[jj]=HYPRE_STREAM(jj);
-    nvtxNameCudaStreamA(s[4], "HYPRE_COMPUTE_STREAM");
-    hypre_MPI_Comm_rank(hypre_MPI_COMM_WORLD, &myid );
-    myid++;
-    POP_RANGE;
-  }
-
-  PUSH_RANGE("PREFETCH+SPMV",2);
-  hypre_int  num_rows     = hypre_CSRMatrixNumRows(A);
-  hypre_int  num_cols     = hypre_CSRMatrixNumCols(A);
-  hypre_int  num_nonzeros = hypre_CSRMatrixNumNonzeros(A);
-
-  if (A->i_short==NULL) {
-
-
-
-    A->i_short = hypre_CTAlloc(hypre_int,  num_rows + 1, HYPRE_MEMORY_SHARED);
-    A->j_short = hypre_CTAlloc(hypre_int,  num_nonzeros, HYPRE_MEMORY_SHARED);
-
-    hypre_CSRMatrixPrefetchToDevice(A);
-    hypre_CSRMatrixPrefetchToDeviceBIGINT(A);
-
-    BigToSmallCopy(A->i_short,A->i,num_rows+1,0);
-    BigToSmallCopy(A->j_short,A->j,num_nonzeros,0);
-
-    hypre_CheckErrorDevice(cudaStreamSynchronize(0));
-    //hypre_printf("BIGINT MOD :: Arrays copied \n");
-  }
-
-  //hypre_CSRMatrixPrefetchToDevice(A);
-  hypre_SeqVectorPrefetchToDevice(x);
-  hypre_SeqVectorPrefetchToDevice(y);
-
-  if (offset!=0) hypre_error_w_msg(HYPRE_ERROR_GENERIC, "WARNING:: Offset is not zero in hypre_CSRMatrixMatvecDevice \n");
-
-  cusparseErrchk(cusparseDcsrmv(handle ,
-                                CUSPARSE_OPERATION_NON_TRANSPOSE,
-                                num_rows-offset, num_cols, num_nonzeros,
-                                &alpha, descr,
-                                A->data ,A->i_short+offset,A->j_short,
-                                x->data, &beta, y->data+offset));
-
-  if (!GetAsyncMode()){
-  hypre_CheckErrorDevice(cudaStreamSynchronize(s[4]));
-  }
-  POP_RANGE;
+#error "TODO BigInt"
 #endif
   return 0;
-
 }
 
 #endif
+
