@@ -1,14 +1,10 @@
-/*BHEADER**********************************************************************
- * Copyright (c) 2008,  Lawrence Livermore National Security, LLC.
- * Produced at the Lawrence Livermore National Laboratory.
- * This file is part of HYPRE.  See file COPYRIGHT for details.
+/******************************************************************************
+ * Copyright 1998-2019 Lawrence Livermore National Security, LLC and other
+ * HYPRE Project Developers. See the top-level COPYRIGHT file for details.
  *
- * HYPRE is free software; you can redistribute it and/or modify it under the
- * terms of the GNU Lesser General Public License (as published by the Free
- * Software Foundation) version 2.1 dated February 1999.
- *
- * $Revision$
- ***********************************************************************EHEADER*/
+ * SPDX-License-Identifier: (Apache-2.0 OR MIT)
+ ******************************************************************************/
+
 #ifndef hypre_MV_HEADER
 #define hypre_MV_HEADER
 
@@ -56,39 +52,36 @@ typedef struct
    /* Does the CSRMatrix create/destroy `data', `i', `j'? */
    HYPRE_Int      owns_data;
 
-   HYPRE_Complex  *data;
+   HYPRE_Complex *data;
 
    /* for compressing rows in matrix multiplication  */
    HYPRE_Int     *rownnz;
    HYPRE_Int      num_rownnz;
 
-#ifdef HYPRE_USING_UNIFIED_MEMORY
-  /* Flag to keeping track of prefetching */
-  HYPRE_Int on_device;
-#endif
-#ifdef HYPRE_USING_MAPPED_OPENMP_OFFLOAD
-  HYPRE_Int mapped;
-#endif
+   /* memory location of arrays i, j, data */
+   HYPRE_Int      memory_location;
+
 //#ifdef HYPRE_BIGINT
 //   hypre_int *i_short, *j_short;
 //#endif
-   
+
 } hypre_CSRMatrix;
 
 /*--------------------------------------------------------------------------
  * Accessor functions for the CSR Matrix structure
  *--------------------------------------------------------------------------*/
 
-#define hypre_CSRMatrixData(matrix)         ((matrix) -> data)
-#define hypre_CSRMatrixI(matrix)            ((matrix) -> i)
-#define hypre_CSRMatrixJ(matrix)            ((matrix) -> j)
-#define hypre_CSRMatrixBigJ(matrix)         ((matrix) -> big_j)
-#define hypre_CSRMatrixNumRows(matrix)      ((matrix) -> num_rows)
-#define hypre_CSRMatrixNumCols(matrix)      ((matrix) -> num_cols)
-#define hypre_CSRMatrixNumNonzeros(matrix)  ((matrix) -> num_nonzeros)
-#define hypre_CSRMatrixRownnz(matrix)       ((matrix) -> rownnz)
-#define hypre_CSRMatrixNumRownnz(matrix)    ((matrix) -> num_rownnz)
-#define hypre_CSRMatrixOwnsData(matrix)     ((matrix) -> owns_data)
+#define hypre_CSRMatrixData(matrix)           ((matrix) -> data)
+#define hypre_CSRMatrixI(matrix)              ((matrix) -> i)
+#define hypre_CSRMatrixJ(matrix)              ((matrix) -> j)
+#define hypre_CSRMatrixBigJ(matrix)           ((matrix) -> big_j)
+#define hypre_CSRMatrixNumRows(matrix)        ((matrix) -> num_rows)
+#define hypre_CSRMatrixNumCols(matrix)        ((matrix) -> num_cols)
+#define hypre_CSRMatrixNumNonzeros(matrix)    ((matrix) -> num_nonzeros)
+#define hypre_CSRMatrixRownnz(matrix)         ((matrix) -> rownnz)
+#define hypre_CSRMatrixNumRownnz(matrix)      ((matrix) -> num_rownnz)
+#define hypre_CSRMatrixOwnsData(matrix)       ((matrix) -> owns_data)
+#define hypre_CSRMatrixMemoryLocation(matrix) ((matrix) -> memory_location)
 
 HYPRE_Int hypre_CSRMatrixGetLoadBalancedPartitionBegin( hypre_CSRMatrix *A );
 HYPRE_Int hypre_CSRMatrixGetLoadBalancedPartitionEnd( hypre_CSRMatrix *A );
@@ -139,7 +132,7 @@ typedef struct
 typedef struct
 {
    void               *matrix;
-   HYPRE_Int               (*ColMap)(HYPRE_Int, void *);
+   HYPRE_Int          (*ColMap)(HYPRE_Int, void *);
    void               *MapData;
 
 } hypre_MappedMatrix;
@@ -173,8 +166,8 @@ typedef struct
 
 typedef struct
 {
-   HYPRE_Int                  num_submatrices;
-   HYPRE_Int                 *submatrix_types;
+   HYPRE_Int             num_submatrices;
+   HYPRE_Int            *submatrix_types;
    void                **submatrices;
 
 } hypre_MultiblockMatrix;
@@ -211,10 +204,13 @@ typedef struct
 typedef struct
 {
    HYPRE_Complex  *data;
-   HYPRE_Int      size;
+   HYPRE_Int       size;
 
    /* Does the Vector create/destroy `data'? */
-   HYPRE_Int      owns_data;
+   HYPRE_Int       owns_data;
+
+   /* memory location of array data */
+   HYPRE_Int       memory_location;
 
    /* For multivectors...*/
    HYPRE_Int   num_vectors;  /* the above "size" is size of one vector */
@@ -225,59 +221,54 @@ typedef struct
       With rowwise storage, vj[i] = data[ j + num_vectors*i] */
    HYPRE_Int  vecstride, idxstride;
    /* ... so vj[i] = data[ j*vecstride + i*idxstride ] regardless of row_storage.*/
-#ifdef HYPRE_USING_GPU
-  HYPRE_Int on_device;
-#endif
-#ifdef HYPRE_USING_MAPPED_OPENMP_OFFLOAD
-  HYPRE_Int mapped;
-  HYPRE_Int drc; /* device ref count */
-  HYPRE_Int hrc; /* host ref count */
-#endif
-
 } hypre_Vector;
 
 /*--------------------------------------------------------------------------
  * Accessor functions for the Vector structure
  *--------------------------------------------------------------------------*/
 
-#define hypre_VectorData(vector)      ((vector) -> data)
-#define hypre_VectorSize(vector)      ((vector) -> size)
-#define hypre_VectorOwnsData(vector)  ((vector) -> owns_data)
-#define hypre_VectorNumVectors(vector) ((vector) -> num_vectors)
+#define hypre_VectorData(vector)                  ((vector) -> data)
+#define hypre_VectorSize(vector)                  ((vector) -> size)
+#define hypre_VectorOwnsData(vector)              ((vector) -> owns_data)
+#define hypre_VectorMemoryLocation(vector)        ((vector) -> memory_location)
+#define hypre_VectorNumVectors(vector)            ((vector) -> num_vectors)
 #define hypre_VectorMultiVecStorageMethod(vector) ((vector) -> multivec_storage_method)
-#define hypre_VectorVectorStride(vector) ((vector) -> vecstride )
-#define hypre_VectorIndexStride(vector) ((vector) -> idxstride )
+#define hypre_VectorVectorStride(vector)          ((vector) -> vecstride )
+#define hypre_VectorIndexStride(vector)           ((vector) -> idxstride )
 
 
 #endif
 
-#ifndef hypre_GPUKERNELS_HEADER
-#define hypre_GPUKERNELS_HEADER
-#ifdef HYPRE_USING_GPU
-#include <cuda_runtime_api.h>
-HYPRE_Int VecScaleScalar(HYPRE_Real *u, const HYPRE_Real alpha,  HYPRE_Int num_rows,cudaStream_t s);
-void DiagScaleVector(HYPRE_Real *x, HYPRE_Real *y, HYPRE_Real *A_data, HYPRE_Int *A_i, HYPRE_Int num_rows, cudaStream_t s);
-void VecCopy(HYPRE_Real* tgt, const HYPRE_Real* src, HYPRE_Int size,cudaStream_t s);
-void VecSet(HYPRE_Real* tgt, HYPRE_Int size, HYPRE_Real value, cudaStream_t s);
-void VecScale(HYPRE_Real *u, HYPRE_Real *v, HYPRE_Real *l1_norm, HYPRE_Int num_rows,cudaStream_t s);
-void VecScaleSplit(HYPRE_Real *u, HYPRE_Real *v, HYPRE_Real *l1_norm, HYPRE_Int num_rows,cudaStream_t s);
-void CudaCompileFlagCheck();
-void PackOnDevice(HYPRE_Complex *send_data,HYPRE_Complex *x_local_data, HYPRE_Int *send_map, HYPRE_Int begin, HYPRE_Int end,cudaStream_t s);
-#endif
-#endif
-
 /* csr_matop.c */
+hypre_CSRMatrix *hypre_CSRMatrixAddHost ( hypre_CSRMatrix *A , hypre_CSRMatrix *B );
 hypre_CSRMatrix *hypre_CSRMatrixAdd ( hypre_CSRMatrix *A , hypre_CSRMatrix *B );
 hypre_CSRMatrix *hypre_CSRMatrixBigAdd ( hypre_CSRMatrix *A , hypre_CSRMatrix *B );
+hypre_CSRMatrix *hypre_CSRMatrixMultiplyHost ( hypre_CSRMatrix *A , hypre_CSRMatrix *B );
 hypre_CSRMatrix *hypre_CSRMatrixMultiply ( hypre_CSRMatrix *A , hypre_CSRMatrix *B );
 hypre_CSRMatrix *hypre_CSRMatrixDeleteZeros ( hypre_CSRMatrix *A , HYPRE_Real tol );
+HYPRE_Int hypre_CSRMatrixTransposeHost ( hypre_CSRMatrix *A , hypre_CSRMatrix **AT , HYPRE_Int data );
 HYPRE_Int hypre_CSRMatrixTranspose ( hypre_CSRMatrix *A , hypre_CSRMatrix **AT , HYPRE_Int data );
 HYPRE_Int hypre_CSRMatrixReorder ( hypre_CSRMatrix *A );
 HYPRE_Complex hypre_CSRMatrixSumElts ( hypre_CSRMatrix *A );
+HYPRE_Real hypre_CSRMatrixFnorm( hypre_CSRMatrix *A );
+HYPRE_Int hypre_CSRMatrixSplit(hypre_CSRMatrix *Bs_ext, HYPRE_BigInt first_col_diag_B, HYPRE_BigInt last_col_diag_B, HYPRE_Int num_cols_offd_B, HYPRE_BigInt *col_map_offd_B, HYPRE_Int *num_cols_offd_C_ptr, HYPRE_BigInt **col_map_offd_C_ptr, hypre_CSRMatrix **Bext_diag_ptr, hypre_CSRMatrix **Bext_offd_ptr);
+hypre_CSRMatrix * hypre_CSRMatrixAddPartial( hypre_CSRMatrix *A, hypre_CSRMatrix *B, HYPRE_Int *row_nums);
+
+/* csr_matop_device.c */
+#if defined(HYPRE_USING_CUDA)
+hypre_CSRMatrix *hypre_CSRMatrixAddDevice ( hypre_CSRMatrix *A , hypre_CSRMatrix *B );
+hypre_CSRMatrix *hypre_CSRMatrixMultiplyDevice ( hypre_CSRMatrix *A , hypre_CSRMatrix *B );
+HYPRE_Int hypre_CSRMatrixSplitDevice(hypre_CSRMatrix *B_ext, HYPRE_BigInt first_col_diag_B, HYPRE_BigInt last_col_diag_B, HYPRE_Int num_cols_offd_B, HYPRE_BigInt *col_map_offd_B, HYPRE_Int **map_B_to_C_ptr, HYPRE_Int *num_cols_offd_C_ptr, HYPRE_BigInt **col_map_offd_C_ptr, hypre_CSRMatrix **B_ext_diag_ptr, hypre_CSRMatrix **B_ext_offd_ptr);
+HYPRE_Int hypre_CSRMatrixTransposeDevice ( hypre_CSRMatrix *A , hypre_CSRMatrix **AT , HYPRE_Int data );
+hypre_CSRMatrix* hypre_CSRMatrixAddPartialDevice( hypre_CSRMatrix *A, hypre_CSRMatrix *B, HYPRE_Int *row_nums);
+HYPRE_Int hypre_CSRMatrixColNNzRealDevice( hypre_CSRMatrix *A, HYPRE_Real *colnnz);
+HYPRE_Int hypre_CSRMatrixMoveDiagFirstDevice( hypre_CSRMatrix  *A );
+#endif
 
 /* csr_matrix.c */
 hypre_CSRMatrix *hypre_CSRMatrixCreate ( HYPRE_Int num_rows , HYPRE_Int num_cols , HYPRE_Int num_nonzeros );
 HYPRE_Int hypre_CSRMatrixDestroy ( hypre_CSRMatrix *matrix );
+HYPRE_Int hypre_CSRMatrixInitialize_v2( hypre_CSRMatrix *matrix, HYPRE_Int bigInit, HYPRE_Int memory_location );
 HYPRE_Int hypre_CSRMatrixInitialize ( hypre_CSRMatrix *matrix );
 HYPRE_Int hypre_CSRMatrixBigInitialize ( hypre_CSRMatrix *matrix );
 HYPRE_Int hypre_CSRMatrixBigJtoJ ( hypre_CSRMatrix *matrix );
@@ -285,32 +276,27 @@ HYPRE_Int hypre_CSRMatrixJtoBigJ ( hypre_CSRMatrix *matrix );
 HYPRE_Int hypre_CSRMatrixSetDataOwner ( hypre_CSRMatrix *matrix , HYPRE_Int owns_data );
 HYPRE_Int hypre_CSRMatrixSetRownnz ( hypre_CSRMatrix *matrix );
 hypre_CSRMatrix *hypre_CSRMatrixRead ( char *file_name );
-HYPRE_Int hypre_CSRMatrixPrint ( hypre_CSRMatrix *matrix , char *file_name );
+HYPRE_Int hypre_CSRMatrixPrint ( hypre_CSRMatrix *matrix, const char *file_name );
+HYPRE_Int hypre_CSRMatrixPrint2( hypre_CSRMatrix *matrix, const char *file_name );
 HYPRE_Int hypre_CSRMatrixPrintHB ( hypre_CSRMatrix *matrix_input , char *file_name );
+HYPRE_Int hypre_CSRMatrixPrintMM( hypre_CSRMatrix *matrix, HYPRE_Int basei, HYPRE_Int basej, HYPRE_Int trans, const char *file_name );
 HYPRE_Int hypre_CSRMatrixCopy ( hypre_CSRMatrix *A , hypre_CSRMatrix *B , HYPRE_Int copy_data );
-hypre_CSRMatrix *hypre_CSRMatrixClone ( hypre_CSRMatrix *A );
+hypre_CSRMatrix *hypre_CSRMatrixClone ( hypre_CSRMatrix *A, HYPRE_Int copy_data );
+hypre_CSRMatrix *hypre_CSRMatrixClone_v2( hypre_CSRMatrix *A, HYPRE_Int copy_data, HYPRE_Int memory_location );
 hypre_CSRMatrix *hypre_CSRMatrixUnion ( hypre_CSRMatrix *A , hypre_CSRMatrix *B , HYPRE_BigInt *col_map_offd_A , HYPRE_BigInt *col_map_offd_B , HYPRE_BigInt **col_map_offd_C );
-#ifdef HYPRE_USING_UNIFIED_MEMORY
-void hypre_CSRMatrixPrefetchToDevice(hypre_CSRMatrix *A);
-void hypre_CSRMatrixPrefetchToDeviceBIGINT(hypre_CSRMatrix *A);
-void hypre_CSRMatrixPrefetchToHost(hypre_CSRMatrix *A);
-hypre_int hypre_CSRMatrixIsManaged(hypre_CSRMatrix *a);
-#endif
-#ifdef HYPRE_USING_MAPPED_OPENMP_OFFLOAD
-void hypre_CSRMatrixMapToDevice(hypre_CSRMatrix *A);
-void hypre_CSRMatrixUpdateToDevice(hypre_CSRMatrix *A);
-void hypre_CSRMatrixUnMapFromDevice(hypre_CSRMatrix *A);
-#endif
+HYPRE_Int hypre_CSRMatrixPrefetch( hypre_CSRMatrix *A, HYPRE_Int to_location);
+//hypre_int hypre_CSRMatrixIsManaged(hypre_CSRMatrix *a);
+
 /* csr_matvec.c */
 // y[offset:end] = alpha*A[offset:end,:]*x + beta*b[offset:end]
 HYPRE_Int hypre_CSRMatrixMatvecOutOfPlace ( HYPRE_Complex alpha , hypre_CSRMatrix *A , hypre_Vector *x , HYPRE_Complex beta , hypre_Vector *b, hypre_Vector *y, HYPRE_Int offset );
-HYPRE_Int hypre_CSRMatrixMatvecOutOfPlaceOOMP ( HYPRE_Complex alpha , hypre_CSRMatrix *A , hypre_Vector *x , HYPRE_Complex beta , hypre_Vector *b, hypre_Vector *y, HYPRE_Int offset );
+HYPRE_Int hypre_CSRMatrixMatvecOutOfPlaceOOMP (HYPRE_Int trans, HYPRE_Complex alpha , hypre_CSRMatrix *A , hypre_Vector *x , HYPRE_Complex beta , hypre_Vector *b, hypre_Vector *y, HYPRE_Int offset );
 // y = alpha*A + beta*y
 HYPRE_Int hypre_CSRMatrixMatvec ( HYPRE_Complex alpha , hypre_CSRMatrix *A , hypre_Vector *x , HYPRE_Complex beta , hypre_Vector *y );
 HYPRE_Int hypre_CSRMatrixMatvecT ( HYPRE_Complex alpha , hypre_CSRMatrix *A , hypre_Vector *x , HYPRE_Complex beta , hypre_Vector *y );
 HYPRE_Int hypre_CSRMatrixMatvec_FF ( HYPRE_Complex alpha , hypre_CSRMatrix *A , hypre_Vector *x , HYPRE_Complex beta , hypre_Vector *y , HYPRE_Int *CF_marker_x , HYPRE_Int *CF_marker_y , HYPRE_Int fpt );
-#ifdef HYPRE_USING_GPU
-HYPRE_Int hypre_CSRMatrixMatvecDevice( HYPRE_Complex alpha , hypre_CSRMatrix *A , hypre_Vector *x , HYPRE_Complex beta , hypre_Vector *b, hypre_Vector *y, HYPRE_Int offset );
+#ifdef HYPRE_USING_CUDA
+HYPRE_Int hypre_CSRMatrixMatvecDevice(HYPRE_Int trans, HYPRE_Complex alpha , hypre_CSRMatrix *A , hypre_Vector *x , HYPRE_Complex beta , hypre_Vector *b, hypre_Vector *y, HYPRE_Int offset );
 HYPRE_Int hypre_CSRMatrixMatvecDeviceBIGINT( HYPRE_Complex alpha , hypre_CSRMatrix *A , hypre_Vector *x , HYPRE_Complex beta , hypre_Vector *b, hypre_Vector *y, HYPRE_Int offset );
 #endif
 /* genpart.c */
@@ -383,6 +369,7 @@ HYPRE_Int hypre_MultiblockMatrixSetSubmatrix ( hypre_MultiblockMatrix *matrix ,
 hypre_Vector *hypre_SeqVectorCreate ( HYPRE_Int size );
 hypre_Vector *hypre_SeqMultiVectorCreate ( HYPRE_Int size , HYPRE_Int num_vectors );
 HYPRE_Int hypre_SeqVectorDestroy ( hypre_Vector *vector );
+HYPRE_Int hypre_SeqVectorInitialize_v2( hypre_Vector *vector, HYPRE_Int memory_location );
 HYPRE_Int hypre_SeqVectorInitialize ( hypre_Vector *vector );
 HYPRE_Int hypre_SeqVectorSetDataOwner ( hypre_Vector *vector , HYPRE_Int owns_data );
 hypre_Vector *hypre_SeqVectorRead ( char *file_name );
@@ -392,7 +379,8 @@ HYPRE_Int hypre_SeqVectorSetRandomValues ( hypre_Vector *v , HYPRE_Int seed );
 HYPRE_Int hypre_SeqVectorCopy ( hypre_Vector *x , hypre_Vector *y );
 hypre_Vector *hypre_SeqVectorCloneDeep ( hypre_Vector *x );
 hypre_Vector *hypre_SeqVectorCloneShallow ( hypre_Vector *x );
-HYPRE_Int hypre_SeqVectorScale ( HYPRE_Complex alpha , hypre_Vector *y );
+HYPRE_Int hypre_SeqVectorScale( HYPRE_Complex alpha, hypre_Vector *y );
+
 HYPRE_Int hypre_SeqVectorAxpy ( HYPRE_Complex alpha , hypre_Vector *x , hypre_Vector *y );
 HYPRE_Real hypre_SeqVectorInnerProd ( hypre_Vector *x , hypre_Vector *y );
 HYPRE_Int hypre_SeqVectorMassInnerProd(hypre_Vector *x, hypre_Vector **y, HYPRE_Int k, HYPRE_Int unroll, HYPRE_Real *result);
@@ -404,54 +392,25 @@ HYPRE_Int hypre_SeqVectorMassDotpTwo8(hypre_Vector *x, hypre_Vector *y , hypre_V
 HYPRE_Int hypre_SeqVectorMassAxpy(HYPRE_Complex *alpha, hypre_Vector **x, hypre_Vector *y, HYPRE_Int k, HYPRE_Int unroll);
 HYPRE_Int hypre_SeqVectorMassAxpy4(HYPRE_Complex *alpha, hypre_Vector **x, hypre_Vector *y, HYPRE_Int k);
 HYPRE_Int hypre_SeqVectorMassAxpy8(HYPRE_Complex *alpha, hypre_Vector **x, hypre_Vector *y, HYPRE_Int k);
-HYPRE_Complex hypre_VectorSumElts ( hypre_Vector *vector );
-#ifdef HYPRE_USING_UNIFIED_MEMORY
-HYPRE_Complex hypre_VectorSumAbsElts ( hypre_Vector *vector );
-HYPRE_Int hypre_SeqVectorCopyDevice ( hypre_Vector *x , hypre_Vector *y );
-HYPRE_Int hypre_SeqVectorAxpyDevice( HYPRE_Complex alpha , hypre_Vector *x , hypre_Vector *y );
-HYPRE_Real hypre_SeqVectorInnerProdDevice ( hypre_Vector *x , hypre_Vector *y );
-/*void  hypre_SeqVectorMassInnerProdDevice ( hypre_Vector *x , hypre_Vector **y, HYPRE_Int k, HYPRE_Real * result);
-void hypre_SeqVectorMassAxpyDevice(HYPRE_Complex * alpha, hypre_Vector **x, hypre_Vector *y, HYPRE_Int k);*/
+HYPRE_Complex hypre_SeqVectorSumElts ( hypre_Vector *vector );
+HYPRE_Int hypre_SeqVectorPrefetch(hypre_Vector *x, HYPRE_Int to_location);
+//hypre_int hypre_SeqVectorIsManaged(hypre_Vector *x);
 
-void hypre_SeqVectorPrefetchToDevice(hypre_Vector *x);
-void hypre_SeqVectorPrefetchToHost(hypre_Vector *x);
-void hypre_SeqVectorPrefetchToDeviceInStream(hypre_Vector *x, HYPRE_Int index);
-hypre_int hypre_SeqVectorIsManaged(hypre_Vector *x);
-#endif
-#ifdef HYPRE_USING_MAPPED_OPENMP_OFFLOAD
-void hypre_SeqVectorMapToDevice(hypre_Vector *x);
-void hypre_SeqVectorUnMapFromDevice(hypre_Vector *x);
-void hypre_SeqVectorUpdateDevice(hypre_Vector *x);
-void hypre_SeqVectorUpdateHost(hypre_Vector *x);
-#endif
+//HYPRE_Int hypre_CSRMatrixMatvecOutOfPlaceOOMP3( HYPRE_Complex alpha, hypre_CSRMatrix *A, hypre_Vector *x, HYPRE_Complex beta, hypre_Vector *b, hypre_Vector *y, HYPRE_Int offset);
 
-HYPRE_Int hypre_CSRMatrixMatvecOutOfPlaceOOMP3( HYPRE_Complex alpha, hypre_CSRMatrix *A, hypre_Vector *x, HYPRE_Complex beta, hypre_Vector *b, hypre_Vector *y, HYPRE_Int offset);
+#if defined(HYPRE_USING_CUDA)
+
+HYPRE_Int hypreDevice_CSRSpAdd(HYPRE_Int ma, HYPRE_Int mb, HYPRE_Int n, HYPRE_Int nnzA, HYPRE_Int nnzB, HYPRE_Int *d_ia, HYPRE_Int *d_ja, HYPRE_Complex *d_aa, HYPRE_Int *d_ib, HYPRE_Int *d_jb, HYPRE_Complex *d_ab, HYPRE_Int *d_num_b, HYPRE_Int *nnzC_out, HYPRE_Int **d_ic_out, HYPRE_Int **d_jc_out, HYPRE_Complex **d_ac_out);
+
+HYPRE_Int hypreDevice_CSRSpTrans(HYPRE_Int m, HYPRE_Int n, HYPRE_Int nnzA, HYPRE_Int *d_ia, HYPRE_Int *d_ja, HYPRE_Complex *d_aa, HYPRE_Int **d_ic_out, HYPRE_Int **d_jc_out, HYPRE_Complex **d_ac_out, HYPRE_Int want_data);
+
+HYPRE_Int hypreDevice_CSRSpGemm(HYPRE_Int m, HYPRE_Int k, HYPRE_Int n, HYPRE_Int nnza, HYPRE_Int nnzb, HYPRE_Int *d_ia, HYPRE_Int *d_ja, HYPRE_Complex *d_a, HYPRE_Int *d_ib, HYPRE_Int *d_jb, HYPRE_Complex *d_b, HYPRE_Int **d_ic_out, HYPRE_Int **d_jc_out, HYPRE_Complex **d_c_out, HYPRE_Int *nnzC);
+
+#endif
 
 #ifdef __cplusplus
 }
 #endif
 
-#ifdef HYPRE_USING_MAPPED_OPENMP_OFFLOAD
-inline void UpdateHRC(hypre_Vector *v){
-  v->hrc++;
-}
-inline void UpdateDRC(hypre_Vector *v){
-  v->drc++;
-}
-inline void SetHRC(hypre_Vector *v){
-  v->hrc=v->drc;
-}
-inline void SetDRC(hypre_Vector *v){
-  v->drc=v->hrc;
-}
-inline void SyncVectorToDevice(hypre_Vector *v){
-  if (v->hrc>v->drc) hypre_SeqVectorUpdateDevice(v);
-}
-inline void SyncVectorToHost(hypre_Vector *v){
-  if (v->drc>v->hrc) hypre_SeqVectorUpdateHost(v);
-}
-#endif
-
-void printRC(hypre_Vector *x,char *id);
-#endif
+#endif /* #ifndef hypre_MV_HEADER */
 
