@@ -1,14 +1,9 @@
-/*BHEADER**********************************************************************
- * Copyright (c) 2008,  Lawrence Livermore National Security, LLC.
- * Produced at the Lawrence Livermore National Laboratory.
- * This file is part of HYPRE.  See file COPYRIGHT for details.
+/******************************************************************************
+ * Copyright 1998-2019 Lawrence Livermore National Security, LLC and other
+ * HYPRE Project Developers. See the top-level COPYRIGHT file for details.
  *
- * HYPRE is free software; you can redistribute it and/or modify it under the
- * terms of the GNU Lesser General Public License (as published by the Free
- * Software Foundation) version 2.1 dated February 1999.
- *
- * $Revision$
- ***********************************************************************EHEADER*/
+ * SPDX-License-Identifier: (Apache-2.0 OR MIT)
+ ******************************************************************************/
 
 #include "_hypre_struct_ls.h"
 
@@ -152,6 +147,40 @@ HYPRE_StructHybridSetSolverType( HYPRE_StructSolver solver,
    return( hypre_HybridSetSolverType( (void *) solver, solver_type ) );
 }
 
+/*--------------------------------------------------------------------------
+ *--------------------------------------------------------------------------*/
+
+HYPRE_Int
+HYPRE_StructHybridSetRecomputeResidual( HYPRE_StructSolver  solver,
+                                        HYPRE_Int           recompute_residual )
+{
+   return( hypre_HybridSetRecomputeResidual( (void *) solver, recompute_residual ) );
+}
+
+HYPRE_Int
+HYPRE_StructHybridGetRecomputeResidual( HYPRE_StructSolver  solver,
+                                        HYPRE_Int          *recompute_residual )
+{
+   return( hypre_HybridGetRecomputeResidual( (void *) solver, recompute_residual ) );
+}
+
+/*--------------------------------------------------------------------------
+ *--------------------------------------------------------------------------*/
+
+HYPRE_Int
+HYPRE_StructHybridSetRecomputeResidualP( HYPRE_StructSolver  solver,
+                                         HYPRE_Int           recompute_residual_p )
+{
+   return( hypre_HybridSetRecomputeResidualP( (void *) solver, recompute_residual_p ) );
+}
+
+HYPRE_Int
+HYPRE_StructHybridGetRecomputeResidualP( HYPRE_StructSolver  solver,
+                                         HYPRE_Int          *recompute_residual_p )
+{
+   return( hypre_HybridGetRecomputeResidualP( (void *) solver, recompute_residual_p ) );
+}
+
 /*--------------------------------------------------------------------------
  *--------------------------------------------------------------------------*/
 
