@@ -1,14 +1,9 @@
-/*BHEADER**********************************************************************
- * Copyright (c) 2008,  Lawrence Livermore National Security, LLC.
- * Produced at the Lawrence Livermore National Laboratory.
- * This file is part of HYPRE.  See file COPYRIGHT for details.
+/******************************************************************************
+ * Copyright 1998-2019 Lawrence Livermore National Security, LLC and other
+ * HYPRE Project Developers. See the top-level COPYRIGHT file for details.
  *
- * HYPRE is free software; you can redistribute it and/or modify it under the
- * terms of the GNU Lesser General Public License (as published by the Free
- * Software Foundation) version 2.1 dated February 1999.
- *
- * $Revision$
- ***********************************************************************EHEADER*/
+ * SPDX-License-Identifier: (Apache-2.0 OR MIT)
+ ******************************************************************************/
 
 /******************************************************************************
  *
@@ -86,11 +81,8 @@ void BoxLoopforall(HYPRE_Int policy, HYPRE_Int length, LOOP_BODY loop_body)
    else if (policy == HYPRE_MEMORY_DEVICE)
    {
       HYPRE_Int gridSize = (length + BLOCKSIZE - 1) / BLOCKSIZE;
-      if (gridSize == 0)
-      {
-         gridSize = 1;
-      }
-      forall_kernel<<<gridSize, BLOCKSIZE>>>(loop_body, length);
+      const dim3 gDim(gridSize), bDim(BLOCKSIZE);
+      HYPRE_CUDA_LAUNCH( forall_kernel, gDim, bDim, loop_body, length );
    }
    else if (policy == 2)
    {
@@ -125,7 +117,8 @@ void ReductionBoxLoopforall(HYPRE_Int policy, HYPRE_Int length, LOOP_BODY Reduct
       hypre_printf("length= %d, blocksize = %d, gridsize = %d\n",
                    length, BLOCKSIZE, gridSize);
       */
-      reductionforall_kernel<<<gridSize, BLOCKSIZE>>>(ReductionLoop, length);
+      const dim3 gDim(gridSize), bDim(BLOCKSIZE);
+      HYPRE_CUDA_LAUNCH( reductionforall_kernel, gDim, bDim, ReductionLoop, length );
    }
 }
 
