@@ -1,14 +1,9 @@
-/*BHEADER**********************************************************************
- * Copyright (c) 2008,  Lawrence Livermore National Security, LLC.
- * Produced at the Lawrence Livermore National Laboratory.
- * This file is part of HYPRE.  See file COPYRIGHT for details.
+/******************************************************************************
+ * Copyright 1998-2019 Lawrence Livermore National Security, LLC and other
+ * HYPRE Project Developers. See the top-level COPYRIGHT file for details.
  *
- * HYPRE is free software; you can redistribute it and/or modify it under the
- * terms of the GNU Lesser General Public License (as published by the Free
- * Software Foundation) version 2.1 dated February 1999.
- *
- * $Revision$
- ***********************************************************************EHEADER*/
+ * SPDX-License-Identifier: (Apache-2.0 OR MIT)
+ ******************************************************************************/
 
 /******************************************************************************
  *
@@ -18,60 +13,16 @@
 
 #include "_hypre_utilities.h"
 #include "../struct_mv/_hypre_struct_mv.h"
+
 #ifdef HYPRE_USE_UMALLOC
 #undef HYPRE_USE_UMALLOC
 #endif
 
-/* global variables for device OpenMP */
-#if defined(HYPRE_USING_DEVICE_OPENMP)
-HYPRE_Int hypre__global_offload = 0;
-HYPRE_Int hypre__offload_device_num;
-HYPRE_Int hypre__offload_host_num;
-/* stats */
-size_t hypre__target_allc_count = 0;
-size_t hypre__target_free_count = 0;
-size_t hypre__target_allc_bytes = 0;
-size_t hypre__target_free_bytes = 0;
-
-size_t hypre__target_htod_count = 0;
-size_t hypre__target_dtoh_count = 0;
-size_t hypre__target_htod_bytes = 0;
-size_t hypre__target_dtoh_bytes = 0;
-#endif
-
 /******************************************************************************
  *
  * Helper routines
  *
  *****************************************************************************/
-/*--------------------------------------------------------------------------
- * hypre_RedefMemLocation
- *   Redefine location based on the selected memory model in hypre_memory.h
- *--------------------------------------------------------------------------*/
-static inline HYPRE_Int hypre_RedefMemLocation(HYPRE_Int location)
-{
-   if (location == HYPRE_MEMORY_HOST)
-   {
-      return HYPRE_MEMORY_HOST_ACT;
-   }
-
-   if (location == HYPRE_MEMORY_DEVICE)
-   {
-      return HYPRE_MEMORY_DEVICE_ACT;
-   }
-
-   if (location == HYPRE_MEMORY_SHARED)
-   {
-      return HYPRE_MEMORY_SHARED_ACT;
-   }
-
-   if (location == HYPRE_MEMORY_HOST_PINNED)
-   {
-      return HYPRE_MEMORY_HOST_PINNED_ACT;
-   }
-
-   return HYPRE_MEMORY_UNSET;
-}
 
 /*--------------------------------------------------------------------------
  * hypre_OutOfMemory
@@ -97,7 +48,7 @@ hypre_WrongMemoryLocation()
 static inline size_t
 hypre_GetPadMemsize(void *ptr, HYPRE_Int location)
 {
-   location = hypre_RedefMemLocation(location);
+   location = hypre_GetActualMemLocation(location);
 
    /* no stored size for host memory */
    if (location == HYPRE_MEMORY_HOST)
@@ -187,8 +138,8 @@ hypre_DeviceMalloc(size_t size, HYPRE_Int zeroinit)
    HYPRE_OMPOffload(hypre__offload_device_num, ptr, size, "enter", "alloc");
 #elif defined(HYPRE_USING_CUDA)
    /* cudaMalloc */
-   hypre_CheckErrorDevice( cudaMalloc(&ptr, size + sizeof(size_t)*HYPRE_MEM_PAD_LEN) );
-   hypre_CheckErrorDevice( cudaDeviceSynchronize() );
+   HYPRE_CUDA_CALL( cudaMalloc(&ptr, size + sizeof(size_t)*HYPRE_MEM_PAD_LEN) );
+   HYPRE_CUDA_CALL( cudaDeviceSynchronize() );
    hypre_Memcpy(ptr, &size, sizeof(size_t), HYPRE_MEMORY_DEVICE, HYPRE_MEMORY_HOST);
    size_t *sp = (size_t*) ptr;
    ptr = (void*) (&sp[HYPRE_MEM_PAD_LEN]);
@@ -211,8 +162,9 @@ hypre_UnifiedMalloc(size_t size, HYPRE_Int zeroinit)
 #if defined(HYPRE_USING_CUDA) || defined(HYPRE_USING_DEVICE_OPENMP)
    size_t count = size + sizeof(size_t)*HYPRE_MEM_PAD_LEN;
    /* with UM, managed memory alloc */
-   hypre_CheckErrorDevice( cudaMallocManaged(&ptr, count, CUDAMEMATTACHTYPE) );
-   hypre_CheckErrorDevice( cudaMemAdvise(ptr, count, cudaMemAdviseSetPreferredLocation, HYPRE_DEVICE) );
+   HYPRE_CUDA_CALL( cudaMallocManaged(&ptr, count, cudaMemAttachGlobal) );
+   HYPRE_Int device = hypre_HandleCudaDevice(hypre_handle);
+   HYPRE_CUDA_CALL( cudaMemAdvise(ptr, count, cudaMemAdviseSetPreferredLocation, device) );
    size_t *sp = (size_t*) ptr;
    sp[0] = size;
    ptr = (void*) (&sp[HYPRE_MEM_PAD_LEN]);
@@ -234,9 +186,9 @@ hypre_HostPinnedMalloc(size_t size, HYPRE_Int zeroinit)
 
 #if defined(HYPRE_USING_CUDA) || defined(HYPRE_USING_DEVICE_OPENMP)
    /* TODO which one of the following two? */
-   /* hypre_CheckErrorDevice( cudaHostAlloc(&ptr,size + sizeof(size_t)*HYPRE_MEM_PAD_LEN,
+   /* HYPRE_CUDA_CALL( cudaHostAlloc(&ptr,size + sizeof(size_t)*HYPRE_MEM_PAD_LEN,
                                             cudaHostAllocMapped)); */
-   hypre_CheckErrorDevice( cudaMallocHost(&ptr, size + sizeof(size_t)*HYPRE_MEM_PAD_LEN) );
+   HYPRE_CUDA_CALL( cudaMallocHost(&ptr, size + sizeof(size_t)*HYPRE_MEM_PAD_LEN) );
    size_t *sp = (size_t*) ptr;
    sp[0] = size;
    ptr = (void*) (&sp[HYPRE_MEM_PAD_LEN]);
@@ -261,7 +213,7 @@ hypre_MAlloc_core(size_t size, HYPRE_Int zeroinit, HYPRE_Int location)
 
    void *ptr = NULL;
 
-   location = hypre_RedefMemLocation(location);
+   location = hypre_GetActualMemLocation(location);
 
    switch (location)
    {
@@ -330,8 +282,8 @@ hypre_DeviceFree(void *ptr)
    size_t size = ((size_t *) ptr)[-HYPRE_MEM_PAD_LEN];
    HYPRE_OMPOffload(hypre__offload_device_num, ptr, size, "exit", "delete");
 #elif defined(HYPRE_USING_CUDA)
-   /* cudaFree((size_t *) ptr - HYPRE_MEM_PAD_LEN); */
-   cudaSafeFree(ptr, HYPRE_MEM_PAD_LEN);
+   HYPRE_CUDA_CALL( cudaFree((size_t *) ptr - HYPRE_MEM_PAD_LEN) );
+   //cudaSafeFree(ptr, HYPRE_MEM_PAD_LEN);
 #endif
 }
 
@@ -340,8 +292,9 @@ hypre_UnifiedFree(void *ptr)
 {
 #if defined(HYPRE_USING_CUDA) || defined(HYPRE_USING_DEVICE_OPENMP)
    /* with UM, managed memory free */
-   /* cudaFree((size_t *) ptr - HYPRE_MEM_PAD_LEN); */
-   cudaSafeFree(ptr, HYPRE_MEM_PAD_LEN);
+   //HYPRE_CUDA_CALL( cudaFree((size_t *) ptr - HYPRE_MEM_PAD_LEN) );
+   cudaFree((size_t *) ptr - HYPRE_MEM_PAD_LEN);
+   //cudaSafeFree(ptr, HYPRE_MEM_PAD_LEN);
 #endif
 }
 
@@ -350,8 +303,8 @@ hypre_HostPinnedFree(void *ptr)
 {
 #if defined(HYPRE_USING_CUDA) || defined(HYPRE_USING_DEVICE_OPENMP)
    /* page-locked memory on the host */
-   /* cudaFreeHost((size_t *) ptr - HYPRE_MEM_PAD_LEN); */
-   cudaSafeFree(ptr, HYPRE_MEM_PAD_LEN);
+   HYPRE_CUDA_CALL( cudaFreeHost((size_t *) ptr - HYPRE_MEM_PAD_LEN) );
+   //cudaSafeFree(ptr, HYPRE_MEM_PAD_LEN);
 #endif
 }
 
@@ -363,7 +316,7 @@ hypre_Free(void *ptr, HYPRE_Int location)
       return;
    }
 
-   location = hypre_RedefMemLocation(location);
+   location = hypre_GetActualMemLocation(location);
 
    switch (location)
    {
@@ -414,7 +367,7 @@ hypre_Device_Unified_HostPinned_ReAlloc(void *ptr, size_t size, HYPRE_Int locati
 void *
 hypre_ReAlloc(void *ptr, size_t size, HYPRE_Int location)
 {
-   location = hypre_RedefMemLocation(location);
+   location = hypre_GetActualMemLocation(location);
 
    if (size == 0)
    {
@@ -462,11 +415,42 @@ hypre_Memcpy(void *dst, void *src, size_t size, HYPRE_Int loc_dst, HYPRE_Int loc
 {
    if (dst == NULL || src == NULL)
    {
+      if (size)
+      {
+         hypre_printf("hypre_Memcpy warning: copy %ld bytes from %p to %p !\n", size, src, dst);
+      }
+
       return;
    }
 
-   loc_dst = hypre_RedefMemLocation(loc_dst);
-   loc_src = hypre_RedefMemLocation(loc_src);
+   loc_dst = hypre_GetActualMemLocation(loc_dst);
+   loc_src = hypre_GetActualMemLocation(loc_src);
+
+   /* special uses for GPU shared memory prefetch */
+#if defined(HYPRE_USING_UNIFIED_MEMORY)
+   if ( dst == src && loc_src == HYPRE_MEMORY_SHARED && (loc_dst == HYPRE_MEMORY_DEVICE || loc_dst == HYPRE_MEMORY_HOST) )
+   {
+      /* src (== dst) must point to cuda unified memory */
+      if (loc_dst == HYPRE_MEMORY_DEVICE)
+      {
+         HYPRE_CUDA_CALL(
+         cudaMemPrefetchAsync(src, size, hypre_HandleCudaDevice(hypre_handle),
+                              hypre_HandleCudaPrefetchStream(hypre_handle))
+         );
+      }
+      else if (loc_dst == HYPRE_MEMORY_HOST)
+      {
+         HYPRE_CUDA_CALL(
+         cudaMemPrefetchAsync(src, size, cudaCpuDeviceId,
+                              hypre_HandleCudaPrefetchStream(hypre_handle))
+         );
+      }
+
+      /* HYPRE_CUDA_CALL( cudaStreamSynchronize(hypre_HandleCudaPrefetchStream(hypre_handle)) ); */
+
+      return;
+   }
+#endif
 
    /* 4 x 4 = 16 cases = 9 + 2 + 2 + 2 + 1 */
    /* 9: Host   <-- Host, Host   <-- Shared, Host   <-- Pinned,
@@ -484,7 +468,7 @@ hypre_Memcpy(void *dst, void *src, size_t size, HYPRE_Int loc_dst, HYPRE_Int loc
    if (loc_dst == HYPRE_MEMORY_SHARED || loc_src == HYPRE_MEMORY_SHARED)
    {
 #if defined(HYPRE_USING_CUDA) || defined(HYPRE_USING_DEVICE_OPENMP)
-      cudaMemcpy(dst, src, size, cudaMemcpyDeviceToDevice);
+      HYPRE_CUDA_CALL( cudaMemcpy(dst, src, size, cudaMemcpyDeviceToDevice) );
 #endif
       return;
    }
@@ -498,7 +482,7 @@ hypre_Memcpy(void *dst, void *src, size_t size, HYPRE_Int loc_dst, HYPRE_Int loc
       memcpy(dst, src, size);
       HYPRE_OMPOffload(hypre__offload_device_num, dst, size, "update", "to");
 #elif defined(HYPRE_USING_CUDA)
-      cudaMemcpy(dst, src, size, cudaMemcpyHostToDevice);
+      HYPRE_CUDA_CALL( cudaMemcpy(dst, src, size, cudaMemcpyHostToDevice) );
 #endif
       return;
    }
@@ -513,7 +497,7 @@ hypre_Memcpy(void *dst, void *src, size_t size, HYPRE_Int loc_dst, HYPRE_Int loc
       HYPRE_OMPOffload(hypre__offload_device_num, src, size, "update", "from");
       memcpy(dst, src, size);
 #elif defined(HYPRE_USING_CUDA)
-      cudaMemcpy( dst, src, size, cudaMemcpyDeviceToHost);
+      HYPRE_CUDA_CALL( cudaMemcpy( dst, src, size, cudaMemcpyDeviceToHost) );
 #endif
       return;
    }
@@ -529,7 +513,7 @@ hypre_Memcpy(void *dst, void *src, size_t size, HYPRE_Int loc_dst, HYPRE_Int loc
       memcpy(dst, src, size);
       HYPRE_OMPOffload(hypre__offload_device_num, dst, size, "update", "to");
 #elif defined(HYPRE_USING_CUDA)
-      cudaMemcpy(dst, src, size, cudaMemcpyDeviceToDevice);
+      HYPRE_CUDA_CALL( cudaMemcpy(dst, src, size, cudaMemcpyDeviceToDevice) );
 #endif
       return;
    }
@@ -541,16 +525,26 @@ hypre_Memcpy(void *dst, void *src, size_t size, HYPRE_Int loc_dst, HYPRE_Int loc
  * hypre_Memset
  * "Sets the first num bytes of the block of memory pointed by ptr to the specified value
  * (*** interpreted as an unsigned char ***)"
+ * http://www.cplusplus.com/reference/cstring/memset/
  *--------------------------------------------------------------------------*/
 void *
 hypre_Memset(void *ptr, HYPRE_Int value, size_t num, HYPRE_Int location)
 {
-   if (ptr == NULL || num == 0)
+   if (num == 0)
    {
       return ptr;
    }
 
-   location = hypre_RedefMemLocation(location);
+   if (ptr == NULL)
+   {
+      if (num)
+      {
+         hypre_printf("hypre_Memset warning: set values for %ld bytes at %p !\n", num, ptr);
+      }
+      return ptr;
+   }
+
+   location = hypre_GetActualMemLocation(location);
 
 #if defined(HYPRE_DEVICE_OPENMP_ALLOC)
    unsigned char *ucptr = (unsigned char *) ptr;
@@ -579,7 +573,7 @@ hypre_Memset(void *ptr, HYPRE_Int value, size_t num, HYPRE_Int location)
          memset(ptr, value, num);
          HYPRE_OMPOffload(hypre__offload_device_num, ptr, num, "update", "to");
 #elif defined(HYPRE_USING_CUDA)
-         cudaMemset(ptr, value, num);
+         HYPRE_CUDA_CALL( cudaMemset(ptr, value, num) );
 #endif
          break;
       case HYPRE_MEMORY_SHARED :
@@ -594,3 +588,65 @@ hypre_Memset(void *ptr, HYPRE_Int value, size_t num, HYPRE_Int location)
    return ptr;
 }
 
+HYPRE_Int
+hypre_GetMemoryLocation(const void *ptr, HYPRE_Int *memory_location)
+{
+   HYPRE_Int ierr = 0;
+
+#if defined(HYPRE_USING_CUDA) || defined(HYPRE_USING_DEVICE_OPENMP)
+   struct cudaPointerAttributes attr;
+   *memory_location = HYPRE_MEMORY_UNSET;
+
+#if (CUDART_VERSION >= 10000)
+   HYPRE_CUDA_CALL( cudaPointerGetAttributes(&attr, ptr) );
+   if (attr.type == cudaMemoryTypeUnregistered)
+   {
+      *memory_location = HYPRE_MEMORY_HOST;
+   }
+   else if (attr.type == cudaMemoryTypeHost)
+   {
+      *memory_location = HYPRE_MEMORY_HOST_PINNED;
+   }
+   else if (attr.type == cudaMemoryTypeDevice)
+   {
+      *memory_location = HYPRE_MEMORY_DEVICE;
+   }
+   else if (attr.type == cudaMemoryTypeManaged)
+   {
+      *memory_location = HYPRE_MEMORY_SHARED;
+   }
+#else
+   cudaError_t err = cudaPointerGetAttributes(&attr, ptr);
+   if (err != cudaSuccess)
+   {
+      ierr = 1;
+
+      /* clear the error */
+      cudaGetLastError();
+
+      if (err == cudaErrorInvalidValue)
+      {
+         *memory_location = HYPRE_MEMORY_HOST;
+      }
+   }
+   else if (attr.isManaged)
+   {
+      *memory_location = HYPRE_MEMORY_SHARED;
+   }
+   else if (attr.memoryType == cudaMemoryTypeDevice)
+   {
+      *memory_location = HYPRE_MEMORY_DEVICE;
+   }
+   else if (attr.memoryType == cudaMemoryTypeHost)
+   {
+      *memory_location = HYPRE_MEMORY_HOST_PINNED;
+   }
+#endif
+
+#else /* #if defined(HYPRE_USING_CUDA) || defined(HYPRE_USING_DEVICE_OPENMP) */
+   *memory_location = HYPRE_MEMORY_HOST;
+#endif
+
+   return ierr;
+}
+
