@@ -1,15 +1,9 @@
-/*BHEADER**********************************************************************
- * Copyright (c) 2008,  Lawrence Livermore National Security, LLC.
- * Produced at the Lawrence Livermore National Laboratory.
- * This file is part of HYPRE.  See file COPYRIGHT for details.
+/******************************************************************************
+ * Copyright 1998-2019 Lawrence Livermore National Security, LLC and other
+ * HYPRE Project Developers. See the top-level COPYRIGHT file for details.
  *
- * HYPRE is free software; you can redistribute it and/or modify it under the
- * terms of the GNU Lesser General Public License (as published by the Free
- * Software Foundation) version 2.1 dated February 1999.
- *
- * $Revision$
- ***********************************************************************EHEADER*/
-
+ * SPDX-License-Identifier: (Apache-2.0 OR MIT)
+ ******************************************************************************/
 
 #include <math.h>
 #include "_hypre_utilities.h"
@@ -121,7 +115,7 @@ void hypre_swap3_d(HYPRE_Real  *v,
 {
    HYPRE_Int temp;
    HYPRE_Real temp_d;
-   
+
 
    temp_d = v[i];
    v[i] = v[j];
@@ -140,14 +134,14 @@ void hypre_swap3_d(HYPRE_Real  *v,
 void hypre_BigSwap4_d(HYPRE_Real  *v,
                   HYPRE_BigInt  *w,
                   HYPRE_Int  *z,
-                  HYPRE_Int *y, 
+                  HYPRE_Int *y,
                   HYPRE_Int  i,
                   HYPRE_Int  j )
 {
    HYPRE_Int temp;
    HYPRE_BigInt big_temp;
    HYPRE_Real temp_d;
-   
+
 
    temp_d = v[i];
    v[i] = v[j];
@@ -204,10 +198,10 @@ void hypre_qsort0( HYPRE_Int *v,
 /*--------------------------------------------------------------------------
  *--------------------------------------------------------------------------*/
 
-void hypre_qsort1( HYPRE_Int *v,
-	     HYPRE_Real *w,
-             HYPRE_Int  left,
-             HYPRE_Int  right )
+void hypre_qsort1( HYPRE_Int  *v,
+                   HYPRE_Real *w,
+                   HYPRE_Int   left,
+                   HYPRE_Int   right )
 {
    HYPRE_Int i, last;
 
@@ -282,28 +276,58 @@ void hypre_qsort2i( HYPRE_Int *v,
 
 /*   sort on w (HYPRE_Real), move v (AB 11/04) */
 
-
 void hypre_qsort2( HYPRE_Int *v,
-	     HYPRE_Real *w,
-             HYPRE_Int  left,
-             HYPRE_Int  right )
+                   HYPRE_Real *w,
+                   HYPRE_Int  left,
+                   HYPRE_Int  right )
 {
    HYPRE_Int i, last;
 
    if (left >= right)
+   {
       return;
+   }
    hypre_swap2( v, w, left, (left+right)/2);
    last = left;
    for (i = left+1; i <= right; i++)
+   {
       if (w[i] < w[left])
       {
          hypre_swap2(v, w, ++last, i);
       }
+   }
    hypre_swap2(v, w, left, last);
    hypre_qsort2(v, w, left, last-1);
    hypre_qsort2(v, w, last+1, right);
 }
 
+/*--------------------------------------------------------------------------
+ *--------------------------------------------------------------------------*/
+/* sort both v and w, in place, but based only on entries in w */
+void hypre_qsort2abs( HYPRE_Int  *v,
+                      HYPRE_Real *w,
+                      HYPRE_Int   left,
+                      HYPRE_Int   right )
+{
+   HYPRE_Int i, last;
+   if (left >= right)
+   {
+      return;
+   }
+   hypre_swap2( v, w, left, (left+right)/2);
+   last = left;
+   for (i = left+1; i <= right; i++)
+   {
+      if (fabs(w[i]) > fabs(w[left]))
+      {
+         hypre_swap2(v, w, ++last, i);
+      }
+   }
+   hypre_swap2(v, w, left, last);
+   hypre_qsort2abs(v, w, left, last-1);
+   hypre_qsort2abs(v, w, last+1, right);
+}
+
 /*--------------------------------------------------------------------------
  *--------------------------------------------------------------------------*/
 
@@ -413,8 +437,6 @@ void hypre_qsort_abs(HYPRE_Real *w,
 }
 
 
-
-
 /*--------------------------------------------------------------------------
  *--------------------------------------------------------------------------*/
 
@@ -596,3 +618,127 @@ void hypre_BigQsort0( HYPRE_BigInt *v,
    hypre_BigQsort0(v, last+1, right);
 }
 
+// Recursive DFS search.
+static void hypre_search_row(HYPRE_Int row,
+                             const HYPRE_Int *row_ptr,
+                             const HYPRE_Int *col_inds,
+                             const HYPRE_Complex *data,
+                      HYPRE_Int *visited,
+                      HYPRE_Int *ordering,
+                      HYPRE_Int *order_ind)
+{
+   // If this row has not been visited, call recursive DFS on nonzero
+   // column entries 
+   if (!visited[row])
+   {
+      HYPRE_Int j;
+      visited[row] = 1;
+      for (j=row_ptr[row]; j<row_ptr[row+1]; j++)
+      {
+         HYPRE_Int col = col_inds[j];
+         hypre_search_row(col, row_ptr, col_inds, data,
+                         visited, ordering, order_ind);
+      }
+      // Add node to ordering *after* it has been searched
+      ordering[*order_ind] = row;
+      *order_ind += 1;
+   }
+}
+
+
+// Find topological ordering on acyclic CSR matrix. That is, find ordering
+// of matrix to be triangular.
+//
+// INPUT
+// -----
+//    - rowptr[], colinds[], data[] form a CSR structure for nxn matrix
+//    - ordering[] should be empty array of length n
+void hypre_topo_sort(const HYPRE_Int *row_ptr,
+                     const HYPRE_Int *col_inds,
+                     const HYPRE_Complex *data,
+                     HYPRE_Int *ordering,
+                     HYPRE_Int n)
+{
+   HYPRE_Int *visited = hypre_CTAlloc(HYPRE_Int, n, HYPRE_MEMORY_HOST);
+   HYPRE_Int order_ind = 0;
+   HYPRE_Int temp_row = 0;
+   while (order_ind < n)
+   {
+      hypre_search_row(temp_row, row_ptr, col_inds, data,
+                       visited, ordering, &order_ind);
+      temp_row += 1;
+      if (temp_row == n)
+      {
+         temp_row = 0;
+      }
+   }
+   hypre_TFree(visited, HYPRE_MEMORY_HOST);
+}
+
+
+// Recursive DFS search.
+static void hypre_dense_search_row(HYPRE_Int row,
+                                   const HYPRE_Complex *L,
+                                   HYPRE_Int *visited,
+                                   HYPRE_Int *ordering,
+                                   HYPRE_Int *order_ind,
+                                   HYPRE_Int n,
+                                   HYPRE_Int is_col_major)
+{
+   // If this row has not been visited, call recursive DFS on nonzero
+   // column entries
+   if (!visited[row])
+   {
+      HYPRE_Int col;
+      visited[row] = 1;
+      for (col=0; col<n; col++)
+      {
+         HYPRE_Complex val;
+         if (is_col_major)
+         {
+            val = L[col*n + row];
+         }
+         else
+         {
+            val = L[row*n + col];
+         }
+         if (fabs(val) > 1e-14)
+         {
+            hypre_dense_search_row(col, L, visited, ordering, order_ind, n, is_col_major);
+         }
+      }
+      // Add node to ordering *after* it has been searched
+      ordering[*order_ind] = row;
+      *order_ind += 1;
+   }
+}
+
+
+// Find topological ordering of acyclic dense matrix in column major
+// format. That is, find ordering of matrix to be triangular.
+//
+// INPUT
+// -----
+//    - L[] : dense nxn matrix in column major format
+//    - ordering[] should be empty array of length n
+//    - row is the row to start the search from
+void hypre_dense_topo_sort(const HYPRE_Complex *L,
+                           HYPRE_Int *ordering,
+                           HYPRE_Int n,
+                           HYPRE_Int is_col_major)
+{
+   HYPRE_Int *visited = hypre_CTAlloc(HYPRE_Int, n, HYPRE_MEMORY_HOST);
+   HYPRE_Int order_ind = 0;
+   HYPRE_Int temp_row = 0;
+   while (order_ind < n)
+   {
+      hypre_dense_search_row(temp_row, L, visited, ordering, &order_ind, n, is_col_major);
+      temp_row += 1;
+      if (temp_row == n)
+      {
+         temp_row = 0;
+      }
+   }
+   hypre_TFree(visited, HYPRE_MEMORY_HOST);
+}
+
